/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

require([
    "aps/ResourceStore",
    "dojo/Deferred",
    "aps/WizardData",
    "aps/xhr",
    "dijit/registry",
    "aps/load",
    "./js/common.js"
], function (ResourceStore, Deferred, WizardData, xhr, registry, load, common) {
    var userRequest = new Deferred(),
        tenant = aps.context.vars.tenant,
        user = aps.context.params.user,
        isNew = common.getCurrentView() === "ccp--suwizard--overview";

    if (isNew) {
        if (typeof(aps.context.params.objects[0]) != 'undefined') {
            var userData = aps.context.params.objects[0],
                login = aps.context.params.user.login.toLowerCase(),
                dataForPrevStep = {};

            userRequest.resolve(userData);

            dataForPrevStep[login] = userData;
            WizardData.put(dataForPrevStep);
        } else {
            load(["aps/PageContainer", {id: "apsPageContainer"}, [
                ["aps/Message", {
                    description: _('The backup service will not be activated for this user due to lack of resources.'),
                    type: "warning"
                }
                ]
            ]]);
        }
    } else {
        var resourceStore = new ResourceStore({
            apsType: common.TYPES.USERS,
            target: "/aps/2/resources/?eq(serviceUserId," + aps.context.params.user.aps.id + ")"
        });
        resourceStore.query().then(function (userList) {
            var ownCloudUser = {user_quota: "-"};
            if (userList.length > 0)
                ownCloudUser = userList[0];
            userRequest.resolve(ownCloudUser);
        });
    }

    userRequest.then(function (cloudBackupUser) {
        function getOverviewValue(counterField) {
            return counterField == common.QUOTA_UNLIMITED_VALUE ? _('Unlimited') : counterField;
        }

        function getStorageOverview(storageLimit) {
            return storageLimit == common.QUOTA_UNLIMITED_VALUE ? _('Unlimited') : storageLimit / common.BYTES_IN_GB;
        }

        function printNotification(value) {
            return value ? _('Yes') : _('No');
        }

        xhr.get("/aps/2/resources/" + tenant.aps.id + "/profile").then(function (profiles) {
            var userQuota = profiles.length ? profiles[0].userQuota : common.USER_QUOTA.STRICT;

            load(
                ["aps/PageContainer", {id: "apsPageContainer"}, [
                    ["aps/FieldSet", [
                        ["aps/Output", {
                            label: _("User name"),
                            value: common.escapeAngularBrackets(user.displayName + ' (' + user.login + ')')
                        }],
                        ["aps/Output", {
                            label: _('Administrator privileges'),
                            value: printNotification(cloudBackupUser.privileged)
                        }]
                    ]],
                    ["aps/FieldSet", {
                        escapeHTML: false,
                        title: _('Notifications')
                    }, [
                        ["aps/Output", {
                            label: _('Management notifications'),
                            value: printNotification(cloudBackupUser.notificationsManagement)
                        }],
                        ["aps/Output", {
                            label: _('Backup notifications'),
                            value: common.getBackupNotificationText(cloudBackupUser.notifyBackup)
                        }]
                    ]],
                    ["aps/FieldSet", {
                        escapeHTML: false,
                        title: _('Quotas'),
                        visible: userQuota != common.USER_QUOTA.OFF
                    }, (userQuota == common.USER_QUOTA.OFF ? [] : [
                        ["aps/Output", {
                            label: _('Cloud Storage (GB)'),
                            value: getStorageOverview(cloudBackupUser.userStorageUsage.limit)
                        }],
                        ["aps/Output", {
                            label: _('Physical workstations'),
                            visible: tenant.WorkstationCount && tenant.WorkstationCount.limit !== null,
                            value: getOverviewValue(cloudBackupUser.UserWorkstationCount.limit)
                        }],
                        ["aps/Output", {
                            label: _('Physical servers'),
                            visible: tenant.ServerCount && tenant.ServerCount.limit !== null,
                            value: getOverviewValue(cloudBackupUser.userServerCount.limit)
                        }],
                        ["aps/Output", {
                            label: _('Virtual machines'),
                            visible: tenant.VmCount && tenant.VmCount.limit !== null,
                            value: getOverviewValue(cloudBackupUser.UserVmCount.limit)
                        }],
                        ["aps/Output", {
                            label: _('Mobile devices'),
                            visible: tenant.MobileDeviceCount && tenant.MobileDeviceCount.limit !== null,
                            value: getOverviewValue(cloudBackupUser.UserMobileDeviceCount.limit)
                        }],
                        ["aps/Output", {
                            label: _('Web Hosting Servers'),
                            visible: tenant.WebHostingServerCount && tenant.WebHostingServerCount.limit !== null,
                            value: getOverviewValue(cloudBackupUser.userWebHostingServerCount.limit)
                        }],
                        ["aps/Output", {
                            label: _('Websites'),
                            visible: tenant.WebsiteCount && tenant.WebsiteCount.limit !== null,
                            value: getOverviewValue(cloudBackupUser.UserWebsiteCount.limit)
                        }]
                    ])]
                ]]
            );
        });
    });
});

