/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([
    "dijit/registry",
    "dojo/_base/declare",
    "aps/nav/ViewPlugin",
    "aps/xhr",
    "./../components/getStorageOutput",
    "./../widgets/base/Status"
], function (
    registry,
    declare,
    ViewPlugin,
    xhr,
    getStorageOutput,
    Status
) {
    var enduserStatuses = {
        INACTIVE: "0",
        ACTIVE: "1",
        IN_PROGRESS: "2",
        NOT_ACTIVATED: "3"
    };

    return declare(ViewPlugin, {
        _tenant: null,
        _statistic: null,

        _getOrCreateResourceUsage: function (statistic) {
            var self = this;
            return getStorageOutput({
                label: "",
                style: "padding: 0px;",
                noneLegend: _("None", {}, {}, self),
                limitedLegend: _("${usage} ${usageUnit} of ${limit} ${limitUnit} Cloud Storage used", {}, {}, self),
                unlimitedLegend: _("${usage} ${usageUnit} Cloud Storage used", {}, {}, self),
                usage: statistic.userStorageUsage.usage || 0,
                limit: statistic.userStorageUsage.limit || 0
            }, {}, {}, self);
        },

        _getOrCreateStatus: function (statusName) {
            var self = this,
                statusInfo = {
                    updating: {"label": _("UPDATING", {}, {}, self), "type": "default", isLoad: true},
                    deleting: {"label": _("DELETING", {}, {}, self), "type": "danger", isLoad: true},
                    disabled: {"label": _("DISABLED", {}, {}, self), "type": "default", isLoad: true},
                    notActivated: {"label": _("NOT ACTIVATED", {}, {}, self), "type": "warning"},
                    activating: {"label": _("ACTIVATING", {}, {}, self), "type": "warning", isLoad: true, useIcon: false},
                    activated: {"label": _("ACTIVATED", {}, {}, self), "type": "success"},
                    off: {"label": _("OFF", {}, {}, self), "type": "disabled"}
                };

            return new Status({
                useIcon: false,
                gridSize: 'md-12',
                status: statusName,
                statusInfo: statusInfo
            });
        },

        _createState: function (service) {
            var self = this,
                state = {
                    title: _("Backup", {}, {}, self)
                };

            if (!service) {
                state.status = _("off", {}, {}, self);
                state.widget = self._getOrCreateStatus('off');
                state.gridWidget = self._getOrCreateStatus('off');
                return state;
            }

            switch (service.aps.status) {
                case "aps:proto":
                case "aps:provisioning":
                    state.status = _("Updating", {}, {}, self);
                    state.widget = self._getOrCreateStatus('updating');
                    state.gridWidget = self._getOrCreateStatus('updating');

                    break;
                case "aps:unprovisioning":
                case "aps:deleted":
                    state.status = _("Deleting", {}, {}, self);
                    state.widget = self._getOrCreateStatus('deleting');
                    state.gridWidget = self._getOrCreateStatus('deleting');

                    break;
                case "aps:ready":
                    if (service.userstatus == enduserStatuses.ACTIVE) {
                        var usage = self._statistic && self._statistic[service.mspUserId] ?
                            self._statistic[service.mspUserId] : service;
                        state.widget = self._getOrCreateResourceUsage(usage);
                        state.gridWidget = self._getOrCreateResourceUsage(usage);
                    } else {
                        var statusName;
                        switch (service.userstatus) {
                            case enduserStatuses.NOT_ACTIVATED:
                                statusName = 'notActivated';
                                break;
                            default:
                                statusName = 'disabled';
                        }
                        state.status = statusName == 'notActivated' ? _("Not activated", {}, {}, self) : _("Disabled", {}, {}, self);
                        state.widget = self._getOrCreateStatus(statusName);
                        state.gridWidget = self._getOrCreateStatus(statusName);
                    }

                    break;
            }

            return state;
        },

        init: function (mediator) {
            var self = this;
            mediator.getServicesList = function () {
                return [{
                    "title": _("Backup", {}, {}, self)
                }];
            };
            mediator.getState = function (users) {
                var result = {};

                users.forEach(function (user) {
                    var services = user.services || [],
                        service = services.filter(function (service) {
                            return service.aps.subscription === self._tenant.aps.subscription;
                        }).pop(),
                        state = self._createState(service);
                    result[user.aps.id] = [state];
                });

                return result;
            };
        },

        onContext: function (context) {
            var self = this;
            self._tenant = context.vars.tenant;
            xhr.get("/aps/2/resources/" + self._tenant.aps.id + "/updateStatistics", {sync: true}).then(function (response) {
                self._statistic = response.userUsage;
            });
        }
    });
});
