/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([
    "dojo/_base/declare",
    "dojo/promise/all",
    "dijit/registry",
    "aps/_View",
    "aps/xhr",
    "aps/Output",
    "aps/nav/ViewPluginManager",
    "./../components/getTenantResourcesTile",
    "./../components/getAdminGrid",
    "./../../common",
    "./../../tags",
    "./../../messages"
], function (
    declare,
    all,
    registry,
    _View,
    xhr,
    Output,
    ViewPluginManager,
    getTenantResourcesTile,
    getAdminGrid,
    common,
    tags,
    messages
) {
    return declare(_View, {
        _administrationTilesId: null,
        _gettingStartedTileId: null,
        _administratorsContainerId: null,
        _tenant: null,

        _getSubscriptionMediator: function () {
            var managerSubscription = new ViewPluginManager("subscriptionPlugin");
            return managerSubscription.init().then(function () {
                return managerSubscription.get("mediator");
            });
        },

        _getSubscriptionTile: function (mediator, subscriptionId) {
            return mediator.getWidget(subscriptionId)[0].data;
        },

        init: function () {
            var self = this;

            self._administrationTilesId = self.genId('getting_started');
            self._gettingStartedTileId = self.genId('administration_tiles');
            self._administratorsContainerId = self.genId('administrators_container');
            return [
                ['aps/Tile', {
                    id: self._gettingStartedTileId,
                    title: _("Getting started"),
                    gridSize: "md-8 xs-12",
                    buttons: [
                        {
                            iconClass: "fa fa-external-link",
                            title: _("Log into Management Console"),
                            autoBusy: false,
                            onClick: function () {
                                var w = window.open('', '_blank');
                                xhr.get("/aps/2/resources/" + self._tenant.aps.id + "/redirectUrl").then(function (response) {
                                    w.location.href = response.url;
                                }, function (error) {
                                    w.close();
                                    console.error("ERR: \n" + JSON.stringify(error));
                                    messages.displayError(error);
                                    self.cancel();
                                });
                            }
                        }
                    ]
                }],
                ['aps/Tiles', {
                    id: self._administrationTilesId,
                    title: _("My subscription")
                }],
                ['aps/Container',{
                    id:  self._administratorsContainerId,
                    title: _("Administrators"),
                    gridSize: "md-12 xs-12"
                }]
            ];
        },
        onHide: function () {
            messages.removeAll();
        },
        onContext: function () {
            var self = this,
                tenant = aps.context.vars.tenant;
            self._tenant = tenant;

            /**
             * Statistics Widget
             *
             * @param backupStatistic
             * @param subscriptionMediator
             */
            var drawStatistics = function(backupStatistic, subscriptionMediator) {
                var subscriptionId = tenant.subscriptionId,
                    administrationTiles = self.byId(self._administrationTilesId);

                administrationTiles.removeAll();
                administrationTiles.addChild(getTenantResourcesTile(backupStatistic.tenant, subscriptionId, tenant.subscription.trial, false));
                administrationTiles.addChild(self._getSubscriptionTile(subscriptionMediator, subscriptionId));
            };

            all({
                subscriptionMediator: self._getSubscriptionMediator(),
                backupStatistic: xhr.get("/aps/2/resources/" + tenant.aps.id + "/updateStatistics"),
                administratorGuideLink: xhr.get("/aps/2/resources/" + tenant.aps.id + "/administratorGuideLink"),
                backupAdminGrid: getAdminGrid(self._tenant)
            }).then(function (response) {
                var backupAdminGrid = response.backupAdminGrid,
                    adminContainer = self.byId(self._administratorsContainerId),
                    gettingStartedTile = self.byId(self._gettingStartedTileId),
                    gettingStartedOutput = new Output({
                        value: _("This is a powerful hybrid cloud backup solution for service providers that protects all your customers' systems and mobile devices, anytime, anywhere, faster than anyone else.") +
                        tags.br() + _("Log into Management Console as an administrator to manage your customers and their backups.") +
                        tags.br() + _("Learn more about the service - Administrator's guide.") +
                        tags.br()  + tags.a(response.administratorGuideLink, _("Administrator's guide"), {'target' : '_blank'}),
                        escapeHTML: false
                    });
                adminContainer.removeAll();
                adminContainer.addChild(backupAdminGrid);

                gettingStartedTile.removeAll();
                gettingStartedTile.addChild(gettingStartedOutput);


                drawStatistics(response.backupStatistic, response.subscriptionMediator);

                aps.apsc.hideLoading();
            }, messages.displayError);
        }
    });
});

