/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([
    "aps/Memory",
    "aps/_View",
    "aps/xhr",
    "aps/FieldSet",
    "dojo/_base/declare",
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getPlainValue",
    "dojox/mvc/getStateful",
    "./../../../common",
    "./../../../messages"
], function (
    Memory,
    _View,
    xhr,
    FieldSet,
    declare,
    registry,
    at,
    getPlainValue,
    getStateful,
    common,
    messages
) {

    return declare(_View, {
        _adminModel: null,
        _tenant: null,

        init: function () {
            var self = this;
            var adminModel = getStateful({
                    login: "",
                    email: "",
                    firstName: "",
                    lastName: "",
                    notificationsBackup: common.NOTIFICATION_LEVEL.MORE,
                    notificationsManagement: 0
                }),
                enableOptions = [
                    {value: "1", label: _("Enabled")},
                    {value: "0", label: _("Disabled")}
                ];
            self._adminModel = adminModel;

            return [
                ['aps/Tile', {
                    id: self.genId('new_administrator'),
                    title: _("Getting started"),
                    gridSize: "md-8 xs-12"
                }, [
                    ["aps/FieldSet", {title: _("General")}, [
                        ["aps/TextBox", {
                            id: self.genId('login'),
                            label: _("Login"),
                            required: true,
                            trim: true,
                            value: at(adminModel, "login")
                        }],
                        ["aps/TextBox", {
                            id: self.genId('email'),
                            label: _("Email"),
                            required: true,
                            trim: true,
                            pattern: '^[^\\s@]+@[^\\s@]+\\.[^\\s@.]+$',
                            invalidMessage: _('Please enter a valid email.'),
                            value: at(adminModel, "email")
                        }],
                        ["aps/TextBox", {
                            id: self.genId("first_name"),
                            label: _("First name"),
                            required: false,
                            trim: true,
                            value: at(adminModel, "firstName")
                        }],
                        ["aps/TextBox", {
                            id: self.genId("last_name"),
                            label: _("Last name"),
                            required: false,
                            trim: true,
                            value: at(adminModel, "lastName")
                        }]
                    ]],
                    ["aps/FieldSet", {title: _("Notifications")}, [
                        ["aps/Select", {
                            id: self.genId("notification_select"),
                            label: _("Backup notifications"),
                            store: new Memory({data: common.getBackupNotificationOptions(), idProperty: "value"}),
                            value: at(adminModel, "notificationsBackup")
                        }],
                        ["aps/Select", {
                            id: self.genId("quota_select"),
                            label: _("Business notifications"),
                            store: new Memory({data: enableOptions, idProperty: "value"}),
                            value: at(adminModel, "notificationsManagement")
                        }]
                    ]],
                    ["aps/Output", {
                        value: _("Note: The administrator receives notifications from all customers.")
                    }]
                ]]
            ];
        },
        onHide: function () {
            messages.removeAll();
        },
        onContext: function (context) {
            var self = this;
            self._tenant = context.vars.tenant;
            var profile = self._tenant.profile;

            self._adminModel.set({
                notificationsBackup: profile.backupNotification,
                notificationsManagement: profile.managementNotification
            });
            aps.apsc.hideLoading();
        },
        onSubmit: function () {
            var self = this,
                tenant = self._tenant,
                adminModel = self._adminModel;

            if (!self.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }

            xhr.get("/aps/2/resources/" + tenant.aps.id + "/loginExists", {
                query: {
                    login: adminModel.login,
                    datacenterId: ''
                }
            }).then(function (existing) {
                if (existing) {
                    messages.displayWarning(_("This login already exists."));
                    aps.apsc.cancelProcessing();
                    return;
                }
                xhr("/aps/2/resources/" + tenant.aps.id + "/administrators/", {
                    method: "POST",
                    query: {
                        datacenterId: ''
                    },
                    data: JSON.stringify(getPlainValue(adminModel)),
                    headers: {"Content-Type": "application/json"}
                }).then(function () {
                    aps.apsc.gotoView("nextcp-ccp-partner");
                }, messages.displayError);
            }, messages.displayError);
        }
    });
});
