/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getStateful",
    "dojox/mvc/getPlainValue",
    "aps/load",
    "aps/WizardData",
    "aps/Memory",
    "aps/CheckBox",
    "aps/TextBox",
    "aps/Output",
    "aps/ResourceStore",
    "aps/Container",
    "./js/messages.js",
    "./js/uomConverter.js",
    "./js/pcp-service-template/wizard/wizard.js",
    "./js/common.js"
],function(registry, at, getStateful, getPlainValue, load, wd, Memory, CheckBox, TextBox, Output, Store,  Container, messages, uomConverter, wizard, common) {

    var resourcesStore = new Store({
        target: "/aps/2/resources/"+aps.context.vars.subscription.aps.id+"/resources"
    });

    resourcesStore.query().then(function (objects) {

        var references = [],
            profiles = [],
            services = [],
            counters = [],
            WIZARD_DATA_KEY = "acronisBackupSTWizard",
            wizardData = wd.get() && wd.get()[WIZARD_DATA_KEY],
            i, apsType, object;

        for(i=0; i<objects.length; ++i) {
            object = getStateful(objects[i]);
            if (object.hasOwnProperty("apsType") &&  typeof(object.apsType) ===  "string") {
                apsType = object.apsType;
                if (common.isGlobals(apsType)) {
                    references.push(object);
                }
                else if (common.isUsers(apsType) || common.isTenant(apsType)){
                    if (object.hasOwnProperty('property')) {
                        counters.push(object);
                    }
                    else {
                        services.push(object);
                    }
                }
                else if (common.isProfile(apsType)) {
                    profiles.push(object);
                }
            }
        }

        function loadBackupProfiles(apsId, restore) {
            function resourceJoinApsId(resources) {
                var a = [];
                resources.forEach(function(resource){
                    if (resource.hasOwnProperty("apsId")) {
                        a.push(resource.apsId);
                    }
                });
                return a.join();
            }

            var profilesStore = new Store({
                target: "/aps/2/resources/" + apsId + "/storages",
                baseQuery: "in(aps.id,("+ resourceJoinApsId(profiles) +"))"
            });

            profilesStore.query().then(function(backupProfiles){
                var store = new Memory({
                        data: backupProfiles,
                        idProperty: "aps.id"
                    }),
                    profilesGrid = registry.byId("profilesGrid");

                profilesGrid.set({
                    store: store,
                    selectionArray: []
                });

                if (restore && wizardData) {
                    profiles.some(function(profile){
                        if (wizardData.reference.profileId && profile.id == wizardData.reference.profileId) {
                            profilesGrid.set("selectionArray", [profile.apsId]);
                            return true;
                        }
                        return false;
                    });
                }
            }).otherwise(messages.displayError);
        }

        function getSelectedResources() {
            var result = {
                    counters: {},
                    reference: {
                        id: null,
                        profileId: null
                    }
                },
                profilesSelection = registry.byId("profilesGrid").get("selectionArray"),
                i;

            result.reference.id =  references.length === 1 ?
                references[0].id : registry.byId("referencesSelect").get("value");

            if (profilesSelection.length) {
                for (i = 0; i < profiles.length; ++i) {
                    if (profiles[i].apsId === profilesSelection[0]) {
                        result.reference.profileId = profiles[i].id;
                        break;
                    }
                }
            }

            for (i = 0; i < counters.length; ++i){
                var counter = counters[i],
                    id = counter.id,
                    checkBox = registry.byId("checkBox-" + id),
                    limit, textBox;

                if (!checkBox.get('checked')) {
                    textBox = registry.byId("textBox-" + id);
                    limit = textBox.get("value");

                    if (limit !== "" && !isNaN(limit)) {
                        result.counters[id] = limit;
                    }
                }
            }

            return result;
        }

        var countersStore = new Memory({ data: counters, idProperty: "apsId"}),
            columns = [
                { "name": _("ID"),        "field": "apsId", sortable: false, visible: false},
                { "name": _("Resource"),  "field": "title", sortable: false  },
                { "name": _("Unlimited"), sortable: false,
                    renderCell: function (row) {
                        var hasData = !!(wizardData && wizardData.counters.hasOwnProperty(row.id));
                        return new CheckBox({
                            id: "checkBox-"+row.id,
                            value: row.id,
                            checked: !hasData,
                            onClick: function(e){
                                var id = this.get('value'),
                                    unlimited = e.target.checked,
                                    textBox = registry.byId("textBox-"+id);

                                textBox.set({
                                    disabled:  unlimited,
                                    required: !unlimited
                                });
                            }
                        });
                    }
                },
                { "name": _("Limit"), "field": "limit", sortable: false ,
                    renderCell: function (row) {
                        var hasData = !!(wizardData && wizardData.counters.hasOwnProperty(row.id)),
                            limit = new TextBox({
                                id: "textBox-" + row.id,
                                disabled: !hasData,
                                required: hasData,
                                value: hasData ? wizardData.counters[row.id] : "",
                                "class": "f-limits",
                                autoSize: false,
                                pattern:  "^[0-9]+$",
                                missingMessage: _("This value is required.")
                            }),
                            unit = new Output({
                                value: row.unit === "unit" ? _("unit") : _("GB")
                            }),
                            limitContainer = new Container({id: "limit-" + row.id});

                        limitContainer.addChild(limit);
                        limitContainer.addChild(unit);

                        return limitContainer;
                    }
                }
            ],
            widgets = [ wizard.getWizardControlView(wizard.VIEWS.LIMITS)];

        if (references.length > 1) {
            widgets.push( ["aps/FieldSet", [
                ["aps/Select", {id:"referencesSelect", "name": "referencesSelect", label: _("Instance")}]
            ]]);
        }

        widgets.push( ["aps/Container", {title:_("Backup Profiles")}, [
            ["aps/Grid", {
                id: "profilesGrid",
                selectionMode: "single",
                sort: {
                    attribute: "name"
                },
                columns: [
                    { field: "aps.id", name: _("ID"), visible: false },
                    { field: "name", name: _("Name"), sortable: false}
                ],
                showPaging: false
            }]
        ]]);

        widgets.push(["aps/Container", {title:_("Resources")}, [
            ["aps/Grid",{
                columns:    columns,
                store:      countersStore,
                showPaging: false
            }]
        ]]);

        load(["aps/PageContainer", { id: "apsPageContainer" }, widgets]).then(function () {

            var referencesSelect = registry.byId("referencesSelect");
            if (references.length > 1) {
                var applicationsStore = new Store({target: "/aps/2/applications/"}),
                    firstSelected = true;

                referencesSelect.on("change", function(){
                    var id = referencesSelect.get("value"),
                        selectedId;

                    references.some(function(item){
                        if (item.id === id) {
                            selectedId = item.apsId;
                            return true;
                        }
                        return false;
                    });

                    loadBackupProfiles(selectedId, firstSelected);
                    firstSelected = false;
                });

                applicationsStore.query().then(function(applications){
                    applications.forEach(function(instance){
                        if (instance.aps.type !== common.APP_ID) {
                            return;
                        }
                        var savedReferenceId = wizardData &&  wizardData.reference.id;
                        references.some(function(item, index){
                            if (item.apsId === instance.globals.aps.id) {
                                var name = instance.aps["package"]["name"];

                                referencesSelect.addOption({
                                    label: name,
                                    value: item.id,
                                    selected: savedReferenceId ? item.id == savedReferenceId : index === 0
                                });
                                return true;
                            }
                            return false;
                        });
                    });
                }).otherwise(messages.displayError);
            }
            else {
                loadBackupProfiles(references[0].apsId, true);
            }

            aps.app.onPrev = function () {
                var wizardData = {};
                wizardData[WIZARD_DATA_KEY] = getSelectedResources();
                wd.put(wizardData);
                aps.apsc.gotoView(aps.context.params.firstScreenView);
            };

            aps.app.onSubmit = function () {
                var page = registry.byId("apsPageContainer");
                if (!page.validate()) {
                    aps.apsc.cancelProcessing();
                    return;
                }

                var id, limit, i, object,
                    resources = [];

                if (references.length > 1){
                    id = parseInt(referencesSelect.get("value"), 10);
                }
                else {
                    id = parseInt(references[0].id, 10);
                }

                limit = 1;

                object = {
                    id: id,
                    limit: limit
                };

                resources.push(object);


                for(i=0; i < services.length; ++i){
                    object = services[i];
                    id = parseInt(object.id, 10);
                    limit = common.isUsers(object.apsType) ? -1 : 1;
                    resources.push({
                        id:    id,
                        limit:  limit
                    });
                }

                for(i=0; i < counters.length; ++i){
                    object = counters[i];
                    id = parseInt(object.id, 10);
                    var checkBox = registry.byId("checkBox-"+id);
                    if (checkBox.get('checked')) {
                        limit = -1;
                    }
                    else {
                        var textBox = registry.byId("textBox-"+id);
                        limit = parseInt(textBox.get("value"), 10);
                        if (object.unit!=="unit"){
                            limit = uomConverter.convert(limit, "GB", object.unit);
                        }
                    }
                    resources.push({
                        id:       id,
                        limit:  limit
                    });
                }

                var grid = registry.byId("profilesGrid"),
                    sel = grid.get("selectionArray");

                if (sel.length >= 1) {
                    var apsId = sel[0];
                    for(i=0; i < profiles.length; ++i) {
                        if (profiles[i].apsId === apsId) {
                            resources.push({
                                id:     parseInt(profiles[i].id,10),
                                limit:  1
                            });
                            break;
                        }
                    }
                }

                wd.put(null);
                aps.apsc.gotoView(
                    aps.context.params.submitScreenView,
                    0,
                    {
                        template:  aps.context.params.stemplate,
                        resources: resources
                    }
                );
            };
        }).otherwise(messages.displayError);
    }).otherwise(messages.displayError);

    aps.app.onPrev = function () {
        // go back before loaded
        aps.apsc.gotoView(aps.context.params.firstScreenView);
    };

    aps.app.onCancel = function () {
        wd.put(null);
        aps.apsc.gotoView(aps.context.params.cancelScreenView);
    };
});
