/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "aps/FieldSet",
    "aps/load",
    "aps/Output",
    "aps/ResourceStore",
    "aps/xhr",
    "dijit/registry",
    "dojo/_base/array",
    "dojo/Deferred",
    "app/common",
    "app/messages"
], function (
    FieldSet,
    load,
    Output,
    Store,
    xhr,
    registry,
    arr,
    Deferred,
    common,
    messages
) {
    function removeProfile(id) {
        var deferred = new Deferred();
        xhr("/aps/2/resources/" + id + "/unbind_resource_type", {
            method: "POST"
        }).then(function () {
            profilesStore.remove(id).then(deferred.resolve, deferred.reject);
        }, deferred.reject);
        return deferred;
    }

    var globals = aps.context.vars.globals,
        profilesStore = new Store({
            target: "/aps/2/resources/" + globals.aps.id + "/storages" // profiles
        });

    load(["aps/PageContainer", {id: "apsPageContainer"}, [
        ["aps/Container", {id: common.genId('container'), style: "margin-bottom: 12px;"}, [
            ["aps/Output", {
                id: common.genId('output'),
                value: _("A backup profile is a set of options for the backup service provisioning and management. Each backup profile is represented as a resource which must be included in a service template.")
            }]
        ]],
        ["aps/Grid", {
            id: "profiles_grid",
            selectionMode: "multiple",
            apsResourceViewId: common.getRelatedViewId('view'),
            sort: {
                attribute: "name"
            },
            showPaging: true,
            columns: [
                {field: "aps.id", name: _("ID"), visible: false},
                {field: "resourceTypeId", name: _("Resource ID"), filter: {title: _("Resource ID")}},
                {field: "name", name: _("Name"), type: "resourceName", filter: {title: _("Name")}},
                {
                    field: "mspGroupType", name: _("Account type"),
                    renderCell: function (row) {
                        return row.mspGroupType == common.GROUP_TYPE.PARTNER ? _("Service provider") : _("Customer");
                    }
                }
            ],
            store: profilesStore
        }, [
            ["aps/Toolbar", {id: "profiles_toolbar"}, [
                ["aps/ToolbarButton", {id: "profiles_new", iconClass: "sb-add-new", label: _("New")}],
                ["aps/ToolbarButton", {
                    id: "profiles_delete",
                    iconClass: "sb-delete",
                    label: _("Delete"),
                    requireItems: true
                }]
            ]]
        ]]
    ]]).then(function () {

        registry.byId("profiles_new").on("click", function () {
            aps.apsc.gotoView(common.getRelatedViewId('new'));
        });

        registry.byId("profiles_delete").on("click", function () {
            var btn = this,
                grid = registry.byId("profiles_grid"),
                sel = grid.get("selectionArray"),
                counter = sel.length;

            if (!confirm(_("Are you sure that you want to delete the following backup profiles: {0}?").replace("{0}", counter))) {
                btn.cancel();
                return;
            }

            messages.removeAll();

            arr.forEach(sel, function (id) {
                console.log("Deleting the backup profile with ID '{0}'...".replace("{0}", id));

                removeProfile(id).then(function () {
                    console.log("The backup profile with ID '{0}' has been deleted.".replace("{0}", id));
                    sel.splice(sel.indexOf(id), 1);
                    grid.refresh();
                    if (--counter === 0) {
                        btn.cancel();
                    }
                }, function (err) {
                    messages.displayErrorAndGo(err);
                    if (--counter === 0) {
                        btn.cancel();
                    }
                });
            });
        });
    }, messages.displayError);
});
