/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "aps/load",
    "aps/Memory",
    "aps/ResourceStore",
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getPlainValue",
    "dojox/mvc/getStateful",
    "app/common",
    "app/messages",
    "app/storagesHelper"
], function (
    load,
    Memory,
    Store,
    registry,
    at,
    getPlainValue,
    getStateful,
    common,
    messages,
    storagesHelper
) {
    var globals = aps.context.vars.globals,
        profilesStore = new Store({
            target: "/aps/2/resources/" + globals.aps.id + "/storages"
        }),
        profileModel = getStateful({
            aps: {type: common.TYPES.PROFILE},
            name: "",
            datacenters: [],
            storages: [],
            storagePolicy: common.STORAGE_POLICY.PARENT,
            backupNotification: common.NOTIFICATION_LEVEL.MORE.toString(),
            managementNotification: common.STATUS.ENABLED,
            userQuota: common.USER_QUOTA.STRICT,
            freemiumMailboxStorage: 0,
            mspGroupType: common.GROUP_TYPE.CUSTOMER
        }),
        isFreemiumMailboxStorageEnabled = globals.minimalCommitment; // Enabled only for Ingram

    storagesHelper.loadDatacenters(globals).then(function (datacenters) {
        var datacentersSelectOptions = storagesHelper.makeDatacentersOptions(datacenters),
            selectedStorages = getStateful([]),
            selectedDatacenters = getStateful([]);
        storagesHelper.loadStorages(datacenters).then(function (storagesLoaded) {
            var storagesSelectOptions = storagesHelper.makeStoragesOptions(storagesLoaded);
            selectedStorages.watchElements(function () {
                profileModel.set("storages", selectedStorages.map(function (value) {
                    var selectedOption = storagesSelectOptions.find(function (option) {
                        return option.value === value;
                    });
                    return {
                        datacenterId: selectedOption.datacenterId,
                        storageId: selectedOption.storageId
                    };
                }));
            });

            selectedDatacenters.watchElements(function () {
                profileModel.set("datacenters", selectedDatacenters.map(function (value) {
                    var selectedOption = datacentersSelectOptions.find(function (option) {
                        return option.value === value;
                    });
                    return {
                        datacenterId: selectedOption.value
                    };
                }));
            });

            var widgets = ["aps/PageContainer", {id: "apsPageContainer"}, [
                ["aps/FieldSet", {id: common.genId('fieldset'), title: _("General")}, [
                    ["aps/TextBox", {
                        id: "name",
                        label: _("Backup profile name"),
                        value: at(profileModel, "name"),
                        trim: true,
                        required: true
                    }]
                ]],
                ["aps/FieldSet", {id: common.genId('fieldset'), title: _("Options")}, [
                    ["aps/Select", {
                        id: "mspGroupType",
                        label: _("Account type"),
                        value: at(profileModel, "mspGroupType"),
                        required: true,
                        store: new Memory({
                            data: [
                                {value: common.GROUP_TYPE.CUSTOMER, label: _("Customer")},
                                {value: common.GROUP_TYPE.PARTNER, label: _("Service provider")}
                            ],
                            idProperty: "value"
                        })
                    }],
                    ["aps/Select", {
                        id: "storagePolicy",
                        label: _("Storage"),
                        value: at(profileModel, "storagePolicy").transform({
                            format: function (value) {
                                return String(value);
                            }
                        }),
                        store: new Memory({
                            data: [
                                {value: common.STORAGE_POLICY.PARENT, label: _("Selected storage")},
                                {value: common.STORAGE_POLICY.OWN, label: _("Resellers must use their own storage")}
                            ], idProperty: "value"
                        }),
                        visible: at(profileModel, "mspGroupType").transform({
                            format: function (value) {
                                return value != common.GROUP_TYPE.PARTNER;
                            }
                        }).direction(at.from)
                    }],
                    ["aps/MultiSelect", {
                        id: "storages",
                        titleLeft: _("Available:"),
                        titleRight: _("Selected:"),
                        values: selectedStorages,
                        store: new Memory({data: storagesSelectOptions, idProperty: "value"})
                    }],
                    ["aps/MultiSelect", {
                        id: "datacenters",
                        label: _("Datacenter"),
                        titleLeft: _("Available:"),
                        titleRight: _("Selected:"),
                        values: selectedDatacenters,
                        store: new Memory({data: datacentersSelectOptions, idProperty: "value"}),
                        hint: _("Changing this option will not impact the existing subscriptions.")
                    }],
                    ["aps/Select", {
                        id: "backupNotification",
                        label: _("Backup notifications"),
                        value: at(profileModel, "backupNotification"),
                        store: new Memory({data: common.getBackupNotificationOptions(), idProperty: "value"})
                    }],
                    ["aps/Select", {
                        id: "managementNotification",
                        label: _("Business notifications"),
                        value: at(profileModel, "managementNotification"),
                        store: new Memory({data: common.getEnabledDisabledOptions(), idProperty: "value"})
                    }],
                    ["aps/CheckBox", {
                        id: "quotaEnabled",
                        label: _("Manage user quotas"),
                        checked: at(profileModel, "userQuota").transform({
                            format: function (value) {
                                return value != common.USER_QUOTA.OFF;
                            },
                            parse: function (value) {
                                return value ? common.USER_QUOTA.STRICT : common.USER_QUOTA.OFF;
                            }
                        }),
                        visible: at(profileModel, "mspGroupType").transform({
                            format: function (value) {
                                return value != common.GROUP_TYPE.PARTNER;
                            }
                        }).direction(at.from)
                    }],
                    ["aps/CheckBox", {
                        id: "overLimit",
                        label: _("Do not allow to set user quotas exceeding subscription limits"),
                        checked: at(profileModel, "userQuota").transform({
                            format: function (value) {
                                return value == common.USER_QUOTA.STRICT;
                            },
                            parse: function (value) {
                                return value ? common.USER_QUOTA.STRICT :
                                    (registry.byId("quotaEnabled").checked ? common.USER_QUOTA.SOFT : common.USER_QUOTA.OFF);
                            }
                        }),
                        disabled: at(profileModel, "userQuota").transform({
                            format: function (value) {
                                return value == common.USER_QUOTA.OFF;
                            }
                        }).direction(at.from),
                        visible: at(profileModel, "mspGroupType").transform({
                            format: function (value) {
                                return value != common.GROUP_TYPE.PARTNER;
                            }
                        }).direction(at.from)
                    }],
                    ["aps/TextBox", {
                        id: "freemiumMailboxStorage",
                        label: _("Free cloud storage per Microsoft Office 365 seat (GB)"),
                        visible: at(profileModel, "mspGroupType").transform({
                            format: function (value) {
                                return value != common.GROUP_TYPE.PARTNER && isFreemiumMailboxStorageEnabled;
                            }
                        }).direction(at.from),
                        trim: true,
                        placeHolder: _('Unlimited'),
                        pattern: "[0-9]*",
                        value: at(profileModel, "freemiumMailboxStorage").transform({
                            format: function (value) {
                                return value == common.QUOTA_UNLIMITED_VALUE ?
                                    '' :
                                    Math.round(Number(value) / common.BYTES_IN_GB);
                            },
                            parse: function (value) {
                                return value === '' ?
                                    common.QUOTA_UNLIMITED_VALUE :
                                    Number(value) * common.BYTES_IN_GB;
                            }
                        })
                    }]
                ]]
            ]];

            load(widgets).then(function () {
                function refreshVisible() {
                    var storagesSelect = registry.byId("storages"),
                        datacentersSelect = registry.byId("datacenters"),
                        showDatacenters = profileModel.mspGroupType == common.GROUP_TYPE.PARTNER,
                        showStorages = !showDatacenters && profileModel.storagePolicy != common.STORAGE_POLICY.OWN;

                    datacentersSelect.set("visible", showDatacenters);
                    storagesSelect.set("visible", showStorages);
                }

                refreshVisible();
                profileModel.watch(refreshVisible);

                aps.app.onSubmit = function () {
                    var page = registry.byId("apsPageContainer");
                    if (!page.validate()) {
                        aps.apsc.cancelProcessing();
                        return;
                    }

                    if (profileModel.mspGroupType != common.GROUP_TYPE.PARTNER) {
                        if (profileModel.storagePolicy == common.STORAGE_POLICY.PARENT && !profileModel.storages.length) {
                            messages.displayWarning(_("You have to choose a storage."));
                            aps.apsc.cancelProcessing();
                            return;
                        }
                        if (profileModel.storagePolicy == common.STORAGE_POLICY.OWN) {
                            profileModel.storages = [];
                        }
                        profileModel.datacenters = [];
                    } else {
                        if (!profileModel.datacenters.length) {
                            messages.displayWarning(_("You have to choose a datacenter."));
                            aps.apsc.cancelProcessing();
                            return;
                        }
                        profileModel.storages = [];
                    }

                    profilesStore.put(getPlainValue(profileModel)).then(function () {
                        aps.apsc.gotoView(common.getRelatedViewId('list'));
                    }, messages.displayError);
                };
            }, messages.displayError);
        });
    }, messages.displayError);

    aps.app.onCancel = function () {
        aps.apsc.gotoView(common.getRelatedViewId('list'));
    };
});
