/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "aps/load",
    "dojo/Deferred",
    "app/common",
    "app/messages",
    "app/storagesHelper"
], function (
    load,
    Deferred,
    common,
    messages,
    storagesHelper
) {
    var globals = aps.context.vars.globals,
        profile = aps.context.vars.profile,
        storageLocation = profile.storageLocation,
        isFreemiumMailboxStorageEnabled = globals.minimalCommitment; // Enabled only for Ingram

    function fetchOutput() {
        var deferred = new Deferred();
        storagesHelper.loadDatacenters(globals).then(function (allDatacenters) {
            if (profile.mspGroupType == common.GROUP_TYPE.PARTNER) {
                var profileDatacenters = storagesHelper.filterDatacenters(allDatacenters, profile);
                var datacentersOutput = storagesHelper.makeDatacentersOutput(profileDatacenters);
                deferred.resolve(datacentersOutput);
                return;
            }
            storagesHelper.loadStorages(allDatacenters).then(function (allStorages) {
                var profileStorages = storagesHelper.filterStorages(allStorages, profile),
                    storagesOutput = storagesHelper.makeStoragesOutput(profileStorages, profile);
                deferred.resolve(storagesOutput);
            }, deferred.reject);
        }, deferred.reject);
        return deferred;
    }

    fetchOutput().then(function (output) {
        var widgets = ["aps/PageContainer", {id: "apsPageContainer"}, [
            ["aps/FieldSet", {id: common.genId('fieldset'), title: _("General")}, [
                ["aps/Output", {
                    id: common.genId("name"),
                    label: _("Backup profile name"),
                    value: profile.name
                }],
                ["aps/Output", {
                    id: common.genId("resourceType"),
                    label: _("Resource"),
                    value: profile.resourceTypeId
                }]
            ]],
            ["aps/FieldSet", {id: common.genId('fieldset'), title: _("Options")}, [
                ["aps/Output", {
                    id: common.genId('fieldset'),
                    label: _("Account type"),
                    value: profile.mspGroupType == common.GROUP_TYPE.PARTNER ? _("Service provider") : _("Customer")
                }],
                ["aps/Output", {
                    id: common.genId("output"),
                    label: profile.mspGroupType == common.GROUP_TYPE.PARTNER ? _("Datacenter") : _("Storage"),
                    value: output,
                    escapeHTML: false
                }],
                ["aps/Output", {
                    id: common.genId("storageLocation"),
                    label: _("Backup locations"),
                    value: storageLocation == common.BACKUP_LOCATIONS.LOCAL_AND_CLOUD ? _("Local and cloud backups") : _("Cloud only backups"),
                    visible: profile.storageLocation != null
                }],
                ["aps/Output", {
                    id: common.genId('output'),
                    label: _("Backup notifications"),
                    value: common.getBackupNotificationText(profile.backupNotification)
                }],
                ["aps/Output", {
                    id: common.genId('output'),
                    label: _("Business notifications"),
                    value: profile.managementNotification == common.STATUS.ENABLED ? _("Enabled") : _("Disabled")
                }],
                ["aps/Output", {
                    id: common.genId('output'),
                    visible: profile.mspGroupType != common.GROUP_TYPE.PARTNER,
                    label: _("Manage user quotas"),
                    value: profile.userQuota != common.USER_QUOTA.OFF ? _("Yes") : _("No")
                }],
                ["aps/Output", {
                    id: common.genId('output'),
                    visible: profile.mspGroupType != common.GROUP_TYPE.PARTNER,
                    label: _("Do not allow to set user quotas exceeding subscription limits"),
                    value: profile.userQuota == common.USER_QUOTA.STRICT ? _("Yes") : _("No")
                }],
                ["aps/Output", {
                    id: common.genId('output'),
                    label: _("Free cloud storage per Microsoft Office 365 seat (GB)"),
                    visible: isFreemiumMailboxStorageEnabled && profile.mspGroupType != common.GROUP_TYPE.PARTNER,
                    value: profile.freemiumMailboxStorage == common.QUOTA_UNLIMITED_VALUE ?
                        _("Unlimited") :
                        Math.round(Number(profile.freemiumMailboxStorage || 0) / common.BYTES_IN_GB)
                }]
            ]]
        ]];

        load(widgets).then(function () {
            aps.app.onSubmit = function () {
                aps.apsc.gotoView(common.getRelatedViewId('edit'), profile.aps.id);
            };
        }, messages.displayError);
    }, messages.displayError);

    aps.app.onCancel = function () {
        aps.apsc.gotoView(common.getRelatedViewId('list'));
    };
});

