/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

require([
    "dojox/mvc/at",
    "dojox/mvc/getStateful",
    "dojox/mvc/getPlainValue",
    "dijit/registry",
    "aps/load",
    "aps/ResourceStore",
    "./js/common.js",
    "./js/messages.js"
], function (
    at,
    getStateful,
    getPlainValue,
    registry,
    load,
    Store,
    common,
    messages
) {
    var placeholder = "**********",
        datacenter = aps.context.vars.datacenter,
        datacenterStore = new Store({
            target: "/aps/2/resources/"
        }),
        datacenterModel = getStateful({
            aps: {id: datacenter.aps.id},
            id: datacenter.id,
            name: datacenter.name,
            url: datacenter.url,
            login: datacenter.login,
            password: ''
        });

    var widgets = ["aps/PageContainer", {id: "apsPageContainer"}, [
            ["aps/FieldSet", {title: _("General")}, [
            ["aps/TextBox", {
                id: "name",
                label: _("Name"),
                value: at(datacenterModel, "name"),
                required: true,
                trim: true
            }],
            ["aps/TextBox", {
                id: "url",
                label: _("URL"),
                value: at(datacenterModel, "url"),
                required: true,
                trim: true
            }],
            ["aps/TextBox", {
                id: "login",
                label: _("Login"),
                value: at(datacenterModel, "login"),
                required: true,
                trim: true
            }],
            ["aps/Password", {
                id: "password",
                label: _("Password"),
                value: placeholder,
                required: false,
                showShowPasswordButton: false,
                showStrengthIndicator: false,
                showResetButton: false,
                confirmation: false,
                autoSize: true,
                size: 8
            }]
        ]]
    ]];
    load(widgets).then(function () {
        var password = registry.byId('password');
        password.inputNode.on('focus', function () {
            if (password.get('value') === placeholder) {
                password.set('value', '');
            }
        });
        password.inputNode.on('blur', function () {
            var value = password.get('value');
            if (value === '') {
                password.set('value', placeholder);
            }
            datacenterModel.set('password', value);
        });

        aps.app.onSubmit = function () {
            var page = registry.byId("apsPageContainer");
            if (!page.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }
            if (/^(http|https):\/\//.exec(datacenterModel.url) == null) {
                datacenterModel.set('url', 'https://' + datacenterModel.url);
            }
            var data = getPlainValue(datacenterModel);
            if (data.password === '') {
                delete data.password;
            }
            datacenterStore.put(data).then(function () {
                aps.apsc.gotoView(common.getRelatedViewId('view'), datacenter.aps.id);
            }, messages.displayError);
        };
    }, messages.displayError);

    aps.app.onCancel = function () {
        aps.apsc.gotoView(common.getRelatedViewId('view'), datacenter.aps.id);
    };
});
