/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "aps/load",
    "aps/Memory",
    "aps/xhr",
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getPlainValue",
    "dojox/mvc/getStateful",
    "./js/common.js",
    "./js/messages.js",
    "./js/wizardData.js"
], function (
    load,
    Memory,
    xhr,
    registry,
    at,
    getPlainValue,
    getStateful,
    common,
    messages,
    wizardData
) {
    var tenant = aps.context.vars.tenant,
        enableOptions = [
            {value: "1", label: _("Enabled")},
            {value: "0", label: _("Disabled")}
        ],
        datacenterModel = getStateful({
            id: null
        }),
        adminModel = getStateful({
            login: "",
            email: "",
            firstName: "",
            lastName: "",
            notificationsBackup: common.NOTIFICATION_LEVEL.MORE,
            notificationsManagement: 0
        });


    function goToList() {
        aps.apsc.gotoView(common.getRelatedViewId('list'));
    }

    function getSelectDatacenterOptions(datacenters) {
        return datacenters.map(function (datacenter) {
            return {
                value: datacenter.id,
                label: datacenter.name
            };
        });
    }

    console.log(aps.context.vars);


    xhr.get("/aps/2/resources/" + tenant.aps.id + "/getDatacenters").then(function (datacenters) {
        var showDatacenters = datacenters.length != 1;
        datacenterModel.set('id', datacenters.length ? datacenters[0].id : null);
        load(["aps/PageContainer", {id: "apsPageContainer"}, [
            ["aps/FieldSet", {title: _("General")}, [
                ["aps/Select", {
                    id: "datacenterId",
                    label: _("Datacenter"),
                    required: true,
                    visible: showDatacenters,
                    store: new Memory({data: getSelectDatacenterOptions(datacenters), idProperty: "value"}),
                    value: at(datacenterModel, "id")
                }],
                ["aps/TextBox", {
                    id: "login",
                    label: _("Login"),
                    required: true,
                    trim: true,
                    value: at(adminModel, "login")
                }],
                ["aps/TextBox", {
                    id: "email",
                    label: _("Email"),
                    required: true,
                    trim: true,
                    pattern: '^[^\\s@]+@[^\\s@]+\\.[^\\s@.]+$',
                    invalidMessage: _('Please enter a valid email.'),
                    value: at(adminModel, "email")
                }],
                ["aps/TextBox", {
                    id: "firstName",
                    label: _("First name"),
                    required: false,
                    trim: true,
                    value: at(adminModel, "firstName")
                }],
                ["aps/TextBox", {
                    id: "lastName",
                    label: _("Last name"),
                    required: false,
                    trim: true,
                    value: at(adminModel, "lastName")
                }]
            ]],
            ["aps/FieldSet", {title: _("Notifications")}, [
                ["aps/Select", {
                    id: "notificationSelect",
                    label: _("Backup notifications"),
                    store: new Memory({data: common.getBackupNotificationOptions(), idProperty: "value"}),
                    value: at(adminModel, "notificationsBackup")
                }],
                ["aps/Select", {
                    id: "quotaSelect",
                    label: _("Business notifications"),
                    store: new Memory({data: enableOptions, idProperty: "value"}),
                    value: at(adminModel, "notificationsManagement")
                }]
            ]],
            ["aps/Output", {
                value: _("Note: The administrator receives notifications from all customers.")
            }]
        ]]).then(function () {
            aps.app.onSubmit = function () {
                var page = registry.byId("apsPageContainer");
                if (!page.validate()) {
                    aps.apsc.cancelProcessing();
                    return;
                }
                if (datacenterModel.id === "") {
                    messages.displayWarning(_("Please select a datacenter."));
                    aps.apsc.cancelProcessing();
                    return;
                }

                xhr.get("/aps/2/resources/" + tenant.aps.id + "/loginExists", {
                    query: {
                        login: adminModel.login,
                        datacenterId: datacenterModel.id
                    }
                }).then(function (existing) {
                    if (existing) {
                        messages.displayWarning(_("This login already exists."));
                        aps.apsc.cancelProcessing();
                        return;
                    }
                    xhr("/aps/2/resources/" + tenant.aps.id + "/administrators/", {
                        method: "POST",
                        query: {
                            datacenterId: datacenterModel.id
                        },
                        data: JSON.stringify(getPlainValue(adminModel)),
                        headers: {"Content-Type": "application/json"}
                    }).then(function () {
                        wizardData.setWizardData({newAdminEmail: adminModel.email}, common.getRelatedViewId('new'));
                        goToList();
                    }, messages.displayError);
                }, messages.displayError);
            };
        });
    }, messages.displayError);

    aps.app.onCancel = goToList;
});
