/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

/**
 * Renders view user page controls.
 */
define([
    "aps/i18n",
    "aps/xhr",
    "app/common",
    "app/quotas"
], function (
    _,
    xhr,
    common,
    quotas
) {
    function getOverviewValue(counterField) {
        return counterField == common.QUOTA_UNLIMITED_VALUE ? _('Unlimited') : counterField;
    }

    function getStorageOverview(storageLimit) {
        return storageLimit == common.QUOTA_UNLIMITED_VALUE ? _('Unlimited') : storageLimit / common.BYTES_IN_GB;
    }

    function printUserStatus(status) {
        switch (status) {
            case common.USER_STATUS.ACTIVE:
                return _('Enabled');
            case common.USER_STATUS.IN_PROGRESS:
                return _('In progress');
            case common.USER_STATUS.INACTIVE:
                return _('Disabled');
            case common.USER_STATUS.NOT_ACTIVATED:
                return _('Not activated');
            default:
                return _('Disabled');
        }
    }

    function printNotification(value) {
        return value ? _('Yes') : _('No');
    }

    function renderControls(account, user, showBack, userQuota, tenant) {
        var showQuotas = quotas.canShowQuotas(userQuota, account);
        return ["aps/PageContainer", {id: "apsPageContainer"}, [
            ["aps/FieldSet", {
                title: _("Main")
            }, [
                ["aps/Output", {
                    label: _("User name"),
                    value: common.escapeAngularBrackets(account.displayName + ' (' + account.endUserLogin + ')')
                }],
                ["aps/Output", {
                    label: _('Administrator privileges'),
                    value: printNotification(account.privileged)
                }]
            ]],
            ["aps/FieldSet", {
                escapeHTML: false,
                title: _('Account status')
            }, [
                ["aps/Output", {
                    label: _('Status'),
                    value: printUserStatus(account.userstatus)
                }]
            ]],
            ["aps/FieldSet", {
                escapeHTML: false,
                title: _('Notifications')
            }, [
                ["aps/Output", {
                    label: _('Management notifications'),
                    value: printNotification(account.notificationsManagement)
                }],
                ["aps/Output", {
                    label: _('Backup notifications'),
                    value: common.getBackupNotificationText(account.notifyBackup)
                }]
            ]],
            ["aps/FieldSet", {
                escapeHTML: false,
                title: _('Quotas'),
                visible: showQuotas
            }, (!showQuotas ? [] : [
                ["aps/Output", {
                    label: _('Cloud Storage (GB)'),
                    value: getStorageOverview(account.userStorageUsage.limit)
                }],
                ["aps/Output", {
                    label: _('Physical workstations'),
                    visible: Boolean(tenant.WorkstationCount && tenant.WorkstationCount.limit),
                    value: getOverviewValue(account.UserWorkstationCount.limit)
                }],
                ["aps/Output", {
                    label: _('Physical servers'),
                    visible: Boolean(tenant.ServerCount && tenant.ServerCount.limit),
                    value: getOverviewValue(account.userServerCount.limit)
                }],
                ["aps/Output", {
                    label: _('Virtual machines'),
                    visible: Boolean(tenant.VmCount && tenant.VmCount.limit),
                    value: getOverviewValue(account.UserVmCount.limit)
                }],
                ["aps/Output", {
                    label: _('Mobile devices'),
                    visible: Boolean(tenant.MobileDeviceCount && tenant.MobileDeviceCount.limit),
                    value: account.UserMobileDeviceCount ? getOverviewValue(account.UserMobileDeviceCount.limit) : null
                }],
                ["aps/Output", {
                    label: _('Web Hosting Servers'),
                    visible: Boolean(tenant.WebHostingServerCount && tenant.WebHostingServerCount.limit),
                    value: account.userWebHostingServerCount ? getOverviewValue(account.userWebHostingServerCount.limit) : null
                }],
                ["aps/Output", {
                    label: _('Websites'),
                    visible: Boolean(tenant.WebsiteCount && tenant.WebsiteCount.limit),
                    value: account.UserWebsiteCount ? getOverviewValue(account.UserWebsiteCount.limit) : null
                }]
            ])],
            ["aps/FieldSet", {title: true}],
            ["aps/Container", {id: "buttons"}, [
                ["aps/Button", {
                    title: _('Edit'),
                    disabled: user.disabled,
                    onClick: function () {
                        var viewId = common.getCurrentView() === 'ccp--suservice--view' ?
                            'ccp--user--edit' :
                            common.getRelatedViewId('edit');
                        aps.apsc.gotoView(viewId, account.aps.id);
                    }
                }],
                ["aps/Button", {
                    title: _('Back to user list'),
                    visible: showBack,
                    onClick: function () {
                        aps.apsc.gotoView(common.getRelatedViewId('list'));
                    }
                }],
                ["aps/Button", {
                    title: _('Change Password'),
                    visible: account.userstatus == common.USER_STATUS.ACTIVE && !account.activateDuringCreation,
                    onClick: function () {
                        aps.apsc.gotoView(common.getRelatedViewId('change-password'), account.aps.id);
                    }
                }]
            ]]
        ]];
    }

    return {
        renderControls: renderControls
    };
});
