#!/usr/bin/env bash

set -e

CRONTASK_ACTION="$1";
CRONTASK_USER="$2";
CRONTASK_COMMAND="$3";
CRONTASK_NAME="$4";
CRONTASK_LOG="$5";
CRONTASK_PERIOD="$6";

function _log_msg {
    echo $1 >> ${CRONTASK_LOG}
}

function _add_cron_task {
    crontab -u ${CRONTASK_USER} -l 2> /dev/null | { cat; echo "${CRONTASK_PERIOD} ${CRONTASK_COMMAND} >/dev/null 2>&1"; } | crontab -u ${CRONTASK_USER} - >> "${CRONTASK_LOG}" 2>&1
}

function _check_and_add_cron_task {
    _log_msg "Adding '${CRONTASK_NAME}' task to cron..."
    local job_line=$(crontab -u ${CRONTASK_USER} -l 2> /dev/null | grep "${CRONTASK_COMMAND}")
    if [ ! -z "$job_line" ]; then
        _log_msg "Task '${CRONTASK_NAME}' already exist in cron."
        return
    fi
    _add_cron_task
    _log_msg "Task '${CRONTASK_NAME}' has been added to cron."
}

function _delete_cron_task {
    crontab -u ${CRONTASK_USER} -l 2> /dev/null | grep -v "${CRONTASK_COMMAND}"  | crontab -u ${CRONTASK_USER} - >> ${CRONTASK_LOG} 2>&1
}

function _check_and_delete_cron_task {
    _log_msg "Removing '${CRONTASK_NAME}' task from cron..."
    local job_line=$(crontab -u ${CRONTASK_USER} -l  2> /dev/null | grep "${CRONTASK_COMMAND}")
    if [ -z "$job_line" ]; then
        _log_msg "Cannot find '${CRONTASK_NAME}' task in cron."
        return
    fi
    _delete_cron_task
    _log_msg "Task '${CRONTASK_NAME}' has been removed from cron."
}

case $CRONTASK_ACTION in
    add)
        _check_and_add_cron_task
        ;;
    delete)
        _check_and_delete_cron_task
        ;;
    *)
        _log_msg "Unknown action $CRONTASK_ACTION."
        ;;
esac