import xmlrpclib
import json
import httplib
import urlparse
from optparse import OptionParser

ACRONIS_BACKUP_GLOBALS_ID = "http://acronis.com/aps/cloud-backup-v1/globals/9.2"


def check_rpc_response(response):
    if response['status'] != 0:
        raise Exception(response['error_message'])


def get_application_instances(public_api, application_id):
    response = public_api.pem.APS.getApplicationInstances({"app_id": application_id})
    check_rpc_response(response)
    return response['result']


def set_minimal_commitment(public_api, instance_id, enable):
    print "%s minimum commitment for instance:" % ('Enable' if enable else 'Disable'), instance_id

    ret = public_api.pem.APS.getApplicationInstanceToken({"application_instance_id": instance_id})
    check_rpc_response(ret)

    aps_token = ret['result']['aps_token']
    aps_uri = ret['result']['controller_uri']

    url = urlparse.urlparse(aps_uri)
    aps = httplib.HTTPSConnection(url.netloc)
    aps.connect()

    aps.request("GET", "/aps/2/resources?implementing(%s)" % ACRONIS_BACKUP_GLOBALS_ID, None, {
        "APS-Token": aps_token
    })
    res = aps.getresponse()
    if res.status != 200:
        print "Can't get resources for application instance {0}".format(instance_id)
        exit(-1)

    applications = json.loads(res.read())
    aps.close()

    body = json.dumps({
        "minimalCommitment": enable
    })

    for application in applications:
        aps.connect()
        global_id = application["aps"]["id"]
        aps.request("PUT", "/aps/2/application/globals/{0}".format(global_id), body, {
            "APS-Token": aps_token
        })
        res = aps.getresponse()
        if res.status != 200:
            print "Can't change minimum commitment flag for {0}".format(global_id)
            exit(-1)
        aps.close()


def main():
    parser = OptionParser()
    parser.add_option('-u', '--api-url', dest='api_url',
                      help='Public API access point, required, i.e., "https://localhost:8440/RPC2"')
    parser.add_option('-a', '--application-id', dest='application_id', type="int",
                      help='Application ID, required')
    parser.add_option('-d', '--disable', dest='enable', action='store_false', default=True,
                      help='Disable minimum commitment for an application')
    opts, args = parser.parse_args()

    if not (opts.api_url and opts.application_id):
        parser.print_help()
        return

    print "Public API access point: {0}".format(opts.api_url)
    print "Application ID: {0}".format(opts.application_id)

    try:
        public_api = xmlrpclib.ServerProxy(opts.api_url)
        application_id = opts.application_id
        enable = opts.enable

        print "Getting instances for the application..."
        instances = get_application_instances(public_api, application_id)
        if not len(instances):
            print "The application does not have instances"
            return

        for instance in instances:
            application_instance_id = instance['application_instance_id']
            set_minimal_commitment(public_api, application_instance_id, enable)

        print "Minimum commitment has been successfully %s for the application:" % ("enabled" if enable else "disabled"), application_id

    except Exception as e:
        print 'Error:', e


if __name__ == '__main__':
    main()
