import sys
import time
import xmlrpclib
import os.path
from common import *

if len(sys.argv) < 4:
    print "Use the following command format: pa-resources-upgrade.sh <POA host> <APS directory> <APS package name>"
    exit(-1)

POA_HOST = sys.argv[1]
APS_APACHE_DIRECTORY_NAME = sys.argv[2]
APS_PACKAGE_NAME = sys.argv[3]

ENDPOINT_PORT = '443'

if os.path.exists('/etc/httpd/conf.d/acronis.conf'):
    ENDPOINT_PORT = '10443'

HTTPS = 'https'
HTTP = 'http'

def openApiUrl():
    return "://%s:8440/RPC2" % POA_HOST

def importPkg(http_protocol=HTTPS):
    url = "%s://%s:%s/%s/%s" % (http_protocol, getLanIp(), ENDPOINT_PORT, APS_APACHE_DIRECTORY_NAME, APS_PACKAGE_NAME)
    tries = 10
    delay = 5 # sec
    iteration = 0
    while True:
        iteration += 1
        print "Importing APS package from %s, try #%s" % (url, iteration)
        if _check_url_exists(url):
            break

        if iteration < tries:
            time.sleep(delay)
        else:
            print "APS package URL location is not available. Exiting..."
            sys.exit(-1)

    return poa.pem.APS.importPackage({"package_url": url})

def _check_url_exists(url):
    try:
        # python 2.x
        import httplib
        conn = httplib.HTTPConnection(str(getLanIp()), str(ENDPOINT_PORT))
        conn.request("HEAD", "/%s/%s" % (APS_APACHE_DIRECTORY_NAME, APS_PACKAGE_NAME))
        response = conn.getresponse()
    except ImportError:
        # python 3+
        import httplib2
        h = httplib2.Http()
        response, _ = h.request(str(url))

    return 200 == response.status

poa = xmlrpclib.ServerProxy("http{0}".format(openApiUrl()))
print "Upgrade APS application..."

try:
    poa.pem.getAccountInfo({'account_id': 1})
except:
    print "Warning: OpenAPI does not accept http requests, trying https"
    poa = xmlrpclib.ServerProxy("https{0}".format(openApiUrl()))

try:
    ret = importPkg()
except:
    print "Warning: APS package not available throw https, trying http"
    ret = importPkg(HTTP)

checkRpcResponse(ret)

application_id = ret['result'].get('application_id', 'Application Id undefined')
print "Acronis Backup Cloud imported, id: {0}".format(application_id)

print "Acronis Backup Cloud Application was successfully upgraded."
