import sys
import time
import xmlrpclib
import json
import httplib
import urlparse
import urllib2
import os.path
from common import *
from billing import *
import gettext
_ = gettext.gettext


if len(sys.argv) < 7:
    print "Use the following command format: pa-resources.sh <POA host> <APS directory> <APS package name> <APS package type> <app directory name> <APS package version> <APS runtime version>"
    exit(-1)

POA_HOST = sys.argv[1]
APS_APACHE_DIRECTORY_NAME = sys.argv[2]
APS_PACKAGE_NAME = sys.argv[3]
PKG_TYPE = sys.argv[4]
APP_DIRECTORY_NAME = sys.argv[5]
APS_PKG_VERSION = sys.argv[6]
RUNTIME_VERSION = sys.argv[7]

ENDPOINT_PORT = '443'

if os.path.exists('/etc/httpd/conf.d/acronis.conf'):
    ENDPOINT_PORT = '10443'

HTTPS = 'https'
HTTP = 'http'


def openApiUrl():
    return "://%s:8440/RPC2" % POA_HOST


def importPkg(http_protocol=HTTPS):
    url = "%s://%s:%s/%s/%s" % (http_protocol, getLanIp(), ENDPOINT_PORT, APS_APACHE_DIRECTORY_NAME, APS_PACKAGE_NAME)
    tries = 10
    delay = 5  # sec
    iteration = 0
    while True:
        iteration += 1
        print "Importing APS package from %s, try #%s" % (url, iteration)
        if _check_url_exists(url):
            break

        if iteration < tries:
            time.sleep(delay)
        else:
            print "APS package URL location is not available. Exiting..."
            sys.exit(-1)

    return poa.pem.APS.importPackage({"package_url": url})


def addService(name, apsName, autoprovide):
    ret = poa.pem.addResourceType({"act_params": [{"var_name": "service_id", "var_value": apsName}
        , {"var_name": "app_id", "var_value": str(application_id)}
        , {"var_name": "autoprovide_service", "var_value": str(autoprovide)}]
        , "name": name, "resclass_name": "rc.saas.service"})
    checkRpcResponse(ret)
    print "'{0}' resource type has been created with id: {1}".format(name, ret['result']['resource_type_id'])
    return ret


def addCounter(name, apsName, rtClass):
    ret = poa.pem.addResourceType({"act_params": [{"var_name": "resource_id", "var_value": apsName}
        , {"var_name": "service_id", "var_value": "tenant"}
        , {"var_name": "app_id", "var_value": str(application_id)}]
        , "name": name, "resclass_name": rtClass})
    checkRpcResponse(ret)
    print "'{0}' resource type has been created with id: {1}".format(name, ret['result']['resource_type_id'])
    return ret


def activateServiceTemplate(st_id):
    ret = poa.pem.activateST({"st_id": st_id})
    checkRpcResponse(ret)


def resourceExist(name, rtClass):
    ret = poa.pem.getResourceTypesByClass({"resclass_name": rtClass})
    checkRpcResponse(ret)
    for item in ret['result']:
        if item['resource_type_name'] == name:
            return True
    return False


def _check_url_exists(url):
    try:
        return 200 == urllib2.urlopen(url).getcode()

    except BaseException as e:
        # url is not available
        print(e)

    return False


def addBackupProfile(backup_profile_name):
    ret = poa.pem.APS.getUserToken({"user_id": 1})
    checkRpcResponse(ret)

    aps_token = ret['result']['aps_token']
    aps_uri = ret['result']['controller_uri']

    url = urlparse.urlparse(aps_uri)
    aps = httplib.HTTPSConnection(url.netloc)
    aps.connect()

    if PKG_TYPE == '2':
        body = json.dumps({
            "aps": {
                "type": "http://acronis.com/aps/cloud-backup-v1/backup-profile/2.5"
            },
            "name": backup_profile_name,
            "backupNotification": 3,
            "managementNotification": 0,
            "storageId": "default",
            "userQuota": 1,
            "mspGroupType": 0
        })
    else:
        body = json.dumps({
            "aps": {
                "type": "http://acronis.com/aps/cloud-backup-v1/backup-profile/2.5"
            },
            "name": backup_profile_name,
            "backupNotification": 3,
            "managementNotification": 0,
            "storageId": "default",
            "userQuota": 1
        })

    aps.request("POST", "/aps/2/resources/{0}/storages".format(application_resource_id), body, {
        "APS-Token": aps_token
    })
    res = aps.getresponse()
    if res.status != 200 and res.status != 202:
        print "Can't create {0}".format(backup_profile_name)
        exit(-1)

    bp = json.loads(res.read())
    aps.close()

    bp_id = bp['aps']['id']
    print "'{0}' resource has been created with id: {1}".format(backup_profile_name, bp_id)

    bp_status = bp['aps']['status']
    while bp_status == 'aps:provisioning':
        time.sleep(1)
        aps.connect()
        aps.request("GET", "/aps/2/resources/{0}".format(bp_id), None, {"APS-Token": aps_token})
        res = aps.getresponse()
        if res.status != 200:
            print "Can't create {0} resource type".format(backup_profile_name)
            exit(-1)

        bp = json.loads(res.read())
        aps.close()
        bp_status = bp['aps']['status']

    profile_resource_id = bp['resourceTypeId']
    print "'{0}' resource type has been created with id: {1}".format(backup_profile_name, profile_resource_id)
    return profile_resource_id


poa = xmlrpclib.ServerProxy("http{0}".format(openApiUrl()))
print "Install APS application..."

try:
    poa.pem.getAccountInfo({'account_id': 1})
except:
    print "Warning: OpenAPI does not accept http requests, trying https"
    poa = xmlrpclib.ServerProxy("https{0}".format(openApiUrl()))

try:
    ret = importPkg()
except:
    print "Warning: APS package not available throw https, trying http"
    ret = importPkg(HTTP)
checkRpcResponse(ret)

application_id = ret['result']['application_id']
print "Acronis Backup Cloud imported, id:", application_id

ret = poa.pem.APS.getApplicationInstances({"app_id": application_id})
checkRpcResponse(ret)

if len(ret['result']):
    if PKG_TYPE != '1':
        print "Add additional resources..."
        resource_name = _("Backup Administrators (Acronis)")
        if not resourceExist(resource_name, "rc.saas.service"):
            addService(resource_name, "admin", 0)

        resource_name = _("Reseller's Cloud Storage (Acronis)")
        if not resourceExist(resource_name, "rc.saas.resource"):
            addCounter(resource_name, "AcronisHostedStorage", "rc.saas.resource")

        resource_name = _("Service Provider's Cloud Storage (Acronis)")
        if not resourceExist(resource_name, "rc.saas.resource"):
            addCounter(resource_name, "ServiceProviderHostedStorage", "rc.saas.resource")

        resource_name = _("Protected Mobile Devices (Acronis)")
        if not resourceExist(resource_name, "rc.saas.resource.unit"):
            addCounter(resource_name, "MobileDeviceCount", "rc.saas.resource.unit")

        resource_name = _("Windows Server Essentials (Acronis)")
        if not resourceExist(resource_name, "rc.saas.resource.unit"):
            addCounter(resource_name, "WindowsServerEssentialsCount", "rc.saas.resource.unit")

        resource_name = _("Virtual Hosts (Acronis)")
        if not resourceExist(resource_name, "rc.saas.resource.unit"):
            addCounter(resource_name, "VirtualHostsCount", "rc.saas.resource.unit")

        resource_name = _("Mailbox (Acronis)")
        if not resourceExist(resource_name, "rc.saas.resource.unit"):
            addCounter(resource_name, "MailboxCount", "rc.saas.resource.unit")

    if RUNTIME_VERSION != '2.0':
        for instance in ret['result']:
            application_instance_id = instance['application_instance_id']

            print "Upgrade application instance:", application_instance_id
            ret = poa.pem.APS.upgradeApplicationInstance({
                'application_instance_id': application_instance_id,
                'package_version': APS_PKG_VERSION
            })
            checkRpcResponse(ret)

    print "Acronis Backup Cloud Application was successfully upgraded."
    sys.exit(0)

ret = poa.pem.APS.provideApplicationInstance({"subscription_id": 0, "rt_id": 0, "app_id": application_id,
                                              "url_path": "https://%s:10443/%s" % (
                                              getLanIp(), APS_APACHE_DIRECTORY_NAME)})
checkRpcResponse(ret)

application_instance_id = ret['result']['application_instance_id']
application_resource_id = ret['result']['application_resource_id']

print "Acronis Backup Cloud instantiated :", application_instance_id, application_resource_id

customer_resource_types = []
reseller_resource_types = []

ret = poa.pem.addResourceType({"act_params": [{"var_name": "resource_uid", "var_value": application_resource_id}
    , {"var_name": "app_id", "var_value": str(application_id)}]
    , "name": _("Backup Service (Acronis)"), "resclass_name": "rc.saas.service.link"})
checkRpcResponse(ret)

backup_service_app_rt_id = ret['result']['resource_type_id']
customer_resource_types.append(backup_service_app_rt_id)
reseller_resource_types.append(backup_service_app_rt_id)
print "'Backup Service Application' resource type has been created with id: ", backup_service_app_rt_id

ret = addService(_("Backup Tenant (Acronis)"), "tenant", 1)
tenant_rt_id = ret['result']['resource_type_id']
customer_resource_types.append(tenant_rt_id)
reseller_resource_types.append(tenant_rt_id)

ret = addService(_("Backup Users (Acronis)"), "users", 0)
customer_resource_types.append(ret['result']['resource_type_id'])

storage_resource_name = _("Cloud Storage (Acronis)") if PKG_TYPE != '1' else _("Cloud Storage")
ret = addCounter(storage_resource_name, "StorageUsage", "rc.saas.resource")
storage_resource_id = ret['result']['resource_type_id']
customer_resource_types.append(storage_resource_id)
reseller_resource_types.append(storage_resource_id)

servers_resource_name = _("Protected Servers (Acronis)") if PKG_TYPE != '1' else _("Servers")
ret = addCounter(servers_resource_name, "ServerCount", "rc.saas.resource.unit")
servers_resource_id = ret['result']['resource_type_id']
customer_resource_types.append(servers_resource_id)
reseller_resource_types.append(servers_resource_id)

machines_resource_name = _("Protected Virtual Machines (Acronis)") if PKG_TYPE != '1' else _("Virtual Machines")
ret = addCounter(machines_resource_name, "VmCount", "rc.saas.resource.unit")
machines_resource_id = ret['result']['resource_type_id']
customer_resource_types.append(machines_resource_id)
reseller_resource_types.append(machines_resource_id)

workstations_resource_name = _("Protected Workstations (Acronis)") if PKG_TYPE != '1' else _("Workstations")
ret = addCounter(workstations_resource_name, "WorkstationCount", "rc.saas.resource.unit")
workstations_resource_id = ret['result']['resource_type_id']
customer_resource_types.append(workstations_resource_id)
reseller_resource_types.append(workstations_resource_id)

if PKG_TYPE != '1':
    mobile_devices_resource_name = _("Protected Mobile Devices (Acronis)") if PKG_TYPE != '1' else _("Mobile Devices")
    ret = addCounter(mobile_devices_resource_name, "MobileDeviceCount", "rc.saas.resource.unit")
    mobile_devices_resource_id = ret['result']['resource_type_id']
    customer_resource_types.append(mobile_devices_resource_id)
    reseller_resource_types.append(mobile_devices_resource_id)

    wse_resource_name = _("Windows Server Essentials (Acronis)") if PKG_TYPE != '1' else _("Windows Server Essentials")
    ret = addCounter(wse_resource_name, "WindowsServerEssentialsCount", "rc.saas.resource.unit")
    wse_resource_id = ret['result']['resource_type_id']
    customer_resource_types.append(wse_resource_id)
    reseller_resource_types.append(wse_resource_id)

    virtual_hosts_resource_name = _("Virtual Hosts (Acronis)") if PKG_TYPE != '1' else _("Virtual Hosts")
    ret = addCounter(virtual_hosts_resource_name, "VirtualHostsCount", "rc.saas.resource.unit")
    virtual_hosts_resource_id = ret['result']['resource_type_id']
    customer_resource_types.append(virtual_hosts_resource_id)
    reseller_resource_types.append(virtual_hosts_resource_id)

    mailbox_resource_name = _("Mailbox (Acronis)") if PKG_TYPE != '1' else _("Mailbox")
    ret = addCounter(mailbox_resource_name, "MailboxCount", "rc.saas.resource.unit")
    mailbox_resource_id = ret['result']['resource_type_id']
    customer_resource_types.append(mailbox_resource_id)
    reseller_resource_types.append(mailbox_resource_id)

backup_profile_resource_name = _("Default Backup Profile (Acronis)") if PKG_TYPE != '1' else _("Default Backup Profile")
backup_profile_id = addBackupProfile(backup_profile_resource_name)
customer_resource_types.append(backup_profile_id)

if PKG_TYPE != '1':
    ret = addService(_("Backup Administrators (Acronis)"), "admin", 0)
    admin_rt_id = ret['result']['resource_type_id']
    reseller_resource_types.append(admin_rt_id)

    ret = addService(_("Reseller Branding (Acronis)"), "reseller", 1)
    branding_rt_id = ret['result']['resource_type_id']
    reseller_resource_types.append(branding_rt_id)

    ret = addCounter(_("Reseller's Cloud Storage (Acronis)"), "AcronisHostedStorage", "rc.saas.resource")
    acronis_hosted_storage_resource_id = ret['result']['resource_type_id']
    reseller_resource_types.append(acronis_hosted_storage_resource_id)

    ret = addCounter(_("Service Provider's Cloud Storage (Acronis)"), "ServiceProviderHostedStorage", "rc.saas.resource")
    provider_hosted_storage_resource_id = ret['result']['resource_type_id']
    reseller_resource_types.append(provider_hosted_storage_resource_id)

    if PKG_TYPE == '2':
            customer_resource_types.append(acronis_hosted_storage_resource_id)
            customer_resource_types.append(provider_hosted_storage_resource_id)

    ret = poa.pem.addServiceTemplate({"owner_id": 1, "name": _("Acronis Backup Cloud for Reseller Branding"),
                                      "resources": [{'resource_type_id': i} for i in reseller_resource_types]})
    checkRpcResponse(ret)

    reseller_st_id = ret['result'].get('st_id', 'Id undefined')
    print "'Acronis Backup Cloud Reseller Branding' service template has been created with id: ", reseller_st_id

    limited = [backup_service_app_rt_id, tenant_rt_id, branding_rt_id]
    unlimited = list(set(reseller_resource_types) - set(limited))
    unlimits = [{"resource_id": i, "resource_limit": -1} for i in unlimited]
    limits = unlimits + [{"resource_id": i, "resource_limit": 1} for i in limited]
    st_limits = {"st_id": reseller_st_id, "limits": limits}
    ret = poa.pem.setSTRTLimits(st_limits)
    checkRpcResponse(ret)

    activateServiceTemplate(reseller_st_id)

if PKG_TYPE == '1':
    ret = poa.pem.addServiceTemplate({"owner_id": 1, "name": _("Acronis Backup Service"),
                                      "resources": [{'resource_type_id': i} for i in customer_resource_types],
                                      "category": "http://www.acronis.com/aps/cloud-backup-v1#acronis-service-template"})
else:
    ret = poa.pem.addServiceTemplate({"owner_id": 1, "name": _("Acronis Backup Service"),
                                      "resources": [{'resource_type_id': i} for i in customer_resource_types]})

checkRpcResponse(ret)

user_st_id = ret['result'].get('st_id', 'Id undefined')
print "'Acronis Backup Cloud' service template has been created with id: ", user_st_id

limited = [backup_service_app_rt_id, tenant_rt_id, backup_profile_id]
unlimited = list(set(customer_resource_types) - set(limited))
unlimits = [{"resource_id": i, "resource_limit": -1} for i in unlimited]
limits = unlimits + [{"resource_id": i, "resource_limit": 1} for i in limited]
st_limits = {"st_id": user_st_id, "limits": limits}
ret = poa.pem.setSTRTLimits(st_limits)
checkRpcResponse(ret)

activateServiceTemplate(user_st_id)

if PKG_TYPE == '1':
    addBackupServicePlans(poa, user_st_id,
                          storage_resource_id, servers_resource_id, machines_resource_id, workstations_resource_id,
                          APP_DIRECTORY_NAME)

print "Acronis Backup Cloud Application was successfully installed."
