<?php
/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\CloudApi;

use Swagger\Client\Model\Applications\Application;
use Swagger\Client\Model\Infra\Infra;
use Swagger\Client\Model\OfferingItems\OfferingItemOutput;
use Swagger\Client\Model\Reports\Report;
use Swagger\Client\Model\Reports\Stored\StoredReportParamsItems;
use Swagger\Client\Model\Tenants\OfferingItem;
use Swagger\Client\Model\Tenants\Tenant;
use Swagger\Client\Model\Usages\UsageOutput;
use Swagger\Client\Model\Users\User;

interface ApiInterface
{
    const APPLICATION_TYPE_BACKUP = 'backup';
    const APPLICATION_TYPE_FILES_CLOUD = 'files_cloud';
    const APPLICATION_TYPE_HYPER_CONVERGED_INFRASTRUCTURE = 'hci';
    const APPLICATION_TYPE_NOTARY = 'notary';
    const APPLICATION_TYPE_PHYSICAL_DATA_SHIPPING = 'physical_data_shipping';
    const APPLICATION_TYPE_PLATFORM = 'platform';

    const OFFERING_ITEM_TYPE_COUNT = 'count';
    const OFFERING_ITEM_TYPE_INFRA = 'infra';
    const OFFERING_ITEM_TYPE_FEATURE = 'feature';

    // Backup
    const OFFERING_ITEM_STORAGE = 'storage';
    const OFFERING_ITEM_LOCAL_STORAGE = 'local_storage';
    const OFFERING_ITEM_CHILD_STORAGES = 'child_storages';
    const OFFERING_ITEM_WORKSTATIONS = 'workstations';
    const OFFERING_ITEM_SERVERS = 'servers';
    const OFFERING_ITEM_WEB_HOSTING_SERVERS = 'web_hosting_servers';
    const OFFERING_ITEM_VMS = 'vms';
    const OFFERING_ITEM_MOBILES = 'mobiles';
    const OFFERING_ITEM_WEBSITES = 'websites';
    const OFFERING_ITEM_MAILBOXES = 'mailboxes';
    const OFFERING_ITEM_MAILBOX_INSTANCE_SIZE = 'mailbox_instance_size';
    const OFFERING_ITEM_O365_MAILBOXES = 'o365_mailboxes';
    const OFFERING_ITEM_O365_ONEDRIVE = 'o365_onedrive';
    const OFFERING_ITEM_O365_SHAREPOINT_SITES = 'o365_sharepoint_sites';

    // Disaster Recovery
    const OFFERING_ITEM_DR_STORAGE = 'dr_storage';
    const OFFERING_ITEM_COMPUTE_POINTS = 'compute_points';
    const OFFERING_ITEM_PUBLIC_IPS = 'public_ips';
    const OFFERING_ITEM_DR_CLOUD_SERVERS = 'dr_cloud_servers';
    const OFFERING_ITEM_DR_INTERNET_ACCESS = 'dr_internet_access';
    const OFFERING_ITEM_DR_CHILD_STORAGES = 'dr_child_storages';

    const OFFERING_ITEM_STATUS_ACTIVE = 1;
    const OFFERING_ITEM_STATUS_INACTIVE = 0;

    const UNLIMITED_OFFERING_ITEM_VALUE = null;

    const CUSTOMER_OFFERING_ITEMS = [
        self::OFFERING_ITEM_STORAGE,
        self::OFFERING_ITEM_LOCAL_STORAGE,
        self::OFFERING_ITEM_WORKSTATIONS,
        self::OFFERING_ITEM_SERVERS,
        self::OFFERING_ITEM_WEB_HOSTING_SERVERS,
        self::OFFERING_ITEM_VMS,
        self::OFFERING_ITEM_MOBILES,
        self::OFFERING_ITEM_WEBSITES,
        self::OFFERING_ITEM_MAILBOXES,
        self::OFFERING_ITEM_MAILBOX_INSTANCE_SIZE,
        self::OFFERING_ITEM_O365_MAILBOXES,
        self::OFFERING_ITEM_O365_ONEDRIVE,
        self::OFFERING_ITEM_O365_SHAREPOINT_SITES,
        self::OFFERING_ITEM_DR_STORAGE,
        self::OFFERING_ITEM_COMPUTE_POINTS,
        self::OFFERING_ITEM_PUBLIC_IPS,
        self::OFFERING_ITEM_DR_CLOUD_SERVERS,
        self::OFFERING_ITEM_DR_INTERNET_ACCESS,
    ];

    const PARTNER_OFFERING_ITEMS = [
        self::OFFERING_ITEM_STORAGE,
        self::OFFERING_ITEM_LOCAL_STORAGE,
        self::OFFERING_ITEM_WORKSTATIONS,
        self::OFFERING_ITEM_SERVERS,
        self::OFFERING_ITEM_WEB_HOSTING_SERVERS,
        self::OFFERING_ITEM_VMS,
        self::OFFERING_ITEM_MOBILES,
        self::OFFERING_ITEM_WEBSITES,
        self::OFFERING_ITEM_MAILBOXES,
        self::OFFERING_ITEM_MAILBOX_INSTANCE_SIZE,
        self::OFFERING_ITEM_O365_MAILBOXES,
        self::OFFERING_ITEM_O365_ONEDRIVE,
        self::OFFERING_ITEM_O365_SHAREPOINT_SITES,
        self::OFFERING_ITEM_DR_STORAGE,
        self::OFFERING_ITEM_COMPUTE_POINTS,
        self::OFFERING_ITEM_PUBLIC_IPS,
        self::OFFERING_ITEM_DR_CLOUD_SERVERS,
        self::OFFERING_ITEM_DR_INTERNET_ACCESS,

        self::OFFERING_ITEM_CHILD_STORAGES,
        self::OFFERING_ITEM_DR_CHILD_STORAGES,
    ];

    const USER_OFFERING_ITEMS = [
        self::OFFERING_ITEM_STORAGE,
        self::OFFERING_ITEM_LOCAL_STORAGE,
        self::OFFERING_ITEM_WORKSTATIONS,
        self::OFFERING_ITEM_SERVERS,
        self::OFFERING_ITEM_WEB_HOSTING_SERVERS,
        self::OFFERING_ITEM_VMS,
        self::OFFERING_ITEM_MOBILES,
        self::OFFERING_ITEM_WEBSITES,
        self::OFFERING_ITEM_MAILBOXES,
    ];

    /**
     * @return string
     */
    public function getUrl();

    /**
     * @return string
     */
    public function getLogin();

    /**
     * @return string
     */
    public function getPassword();

    /* ====== Users API ====== */

    /**
     * @return User
     */
    public function getMe();

    /**
     * Returns True if the login exists otherwise False
     *
     * @param string $login
     * @return bool
     */
    public function checkLogin($login);

    /* ====== Methods to manage tenants ====== */

    /**
     * @param string $tenantId
     * @return Tenant
     */
    public function getTenant($tenantId);

    /**
     * @param $rootTenantId
     * @param $leafTenantId
     * @return string[] a list of UUIDs
     */
    public function resolveTenantPath($rootTenantId, $leafTenantId);

    /**
     * @param string $tenantId
     * @return UsageOutput[]
     */
    public function getTenantUsages($tenantId);

    /* ====== Methods to manage tenant's applications ====== */

    /**
     * @return Application[]
     */
    public function getApplications();

    /**
     * @param string $type
     * @return Application|null
     */
    public function getApplicationByType($type);

    /**
     * @param string $tenantId
     * @return string[] a list of UUIDs
     */
    public function getTenantApplicationsUuids($tenantId);

    /**
     * @param string $tenantId
     * @param string $applicationId
     * @return bool
     */
    public function isApplicationEnabledForTenant($tenantId, $applicationId);

    /**
     * @param string $tenantId
     * @param string $applicationId
     */
    public function enableApplicationForTenant($tenantId, $applicationId);

    /**
     * @param string $tenantId
     * @param string $applicationId
     */
    public function disableApplicationForTenant($tenantId, $applicationId);

    /* ====== Methods to manage offering items ====== */

    /**
     * @param string $tenantId
     * @param string $applicationId
     * @return OfferingItemOutput[]
     */
    public function getTenantApplicationOfferingItems($tenantId, $applicationId);

    /**
     * @param string $tenantId
     * @param OfferingItem[] $offeringItems
     * @return OfferingItemOutput[]
     */
    public function updateTenantApplicationOfferingItems($tenantId, $offeringItems);

    /**
     * Recursive enable an application for a hierarchy of tenants except root tenant.
     *
     * @param string $rootTenantId
     * @param string $leafTenantId
     * @param string $applicationId
     */
    public function enableApplicationForTenantsChain($rootTenantId, $leafTenantId, $applicationId);

    /**
     * Recursive enable specified offering items with unlimited quotas for a hierarchy of tenants except root tenant.
     * Note: Enabled or not specified offering items won't be affected
     *
     * @param string $rootTenantId
     * @param string $leafTenantId
     * @param string $applicationId
     * @param OfferingItem[] $offeringItems
     */
    public function enableApplicationOfferingItemsForTenantsChain(
        $rootTenantId,
        $leafTenantId,
        $applicationId,
        array $offeringItems
    );

    /**
     * Update specified only offering items for $leafTenantId.
     * Recursive enable offering items for tenant's parent (see method enableUnlimitedOfferingItemsForTenantsChain)
     *
     * @param string $rootTenantId
     * @param string $leafTenantId
     * @param string $applicationId
     * @param OfferingItem[] $offeringItems
     */
    public function updateApplicationOfferingItemsForTenantsChain(
        $rootTenantId,
        $leafTenantId,
        $applicationId,
        array $offeringItems
    );

    /**
     * @param string $offeringItemName
     * @return bool
     */
    public function isOfferingItemForCustomer($offeringItemName);

    /**
     * @param string $offeringItemName
     * @return bool
     */
    public function isOfferingItemForPartner($offeringItemName);

    /**
     * @param string $offeringItemName
     * @return bool
     */
    public function isOfferingItemForUser($offeringItemName);

    /* ====== Methods to manage reports ====== */

    /**
     * @param string $reportId
     * @return StoredReportParamsItems[]
     */
    public function getStoredUsageReports($reportId);

    /**
     * @param string $reportId
     * @param string $storedReportId
     * @param string $destinationFile
     * @return void
     */
    public function downloadStoredUsageReportArchive($reportId, $storedReportId, $destinationFile);

    /**
     * @param string $tenantId
     * @return Report
     */
    public function orderCurrentUsageReportForAccounts($tenantId);

    /* ====== Methods to manage infra objects ====== */

    /**
     * @param string $tenantId
     * @return string[] a list of UUIDs
     */
    public function getTenantInfrasUuids($tenantId);

    /**
     * @param string[] $uuids
     * @return Infra[]
     */
    public function fetchInfras($uuids);

    /**
     * @param string $tenantId
     * @return Infra[]
     */
    public function fetchTenantInfras($tenantId);

    /**
     * @param $rootTenantId
     * @param $leafTenantId
     * @return string[] a list of UUIDs
     */
    public function getInfrasUuidsForTenantsChain($rootTenantId, $leafTenantId);

    /**
     * @param $rootTenantId
     * @param $leafTenantId
     * @return Infra[]
     */
    public function fetchInfrasForTenantsChain($rootTenantId, $leafTenantId);
}