<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Console\Command;

use Acronis\App\Database\MigrationInterface;
use Acronis\App\Database\Migrations\InitialMigration;
use Acronis\App\Services\Database\DatabaseAwareTrait;
use Acronis\App\Services\Logger\AppLoggerAwareTrait;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Performs only up sql
 * @package Acronis\App\Console\Command
 */
final class DatabaseMigrate extends Loggable
{
    use DatabaseAwareTrait;
    use AppLoggerAwareTrait;

    /** @var MigrationInterface[] */
    private $migrations = [];

    public function __construct()
    {
        parent::__construct();
        // Add list of sql for current update below:
        $this->migrations[] = new InitialMigration();
    }

    protected function getCommandName()
    {
        return 'database:migrate';
    }

    protected function getCommandHelp()
    {
        return '<info>Migrates database schema.</info>';
    }

    protected function getCommandDescription()
    {
        return 'Migrates database schema.';
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        foreach ($this->migrations as $migration) {
            $this->getLog()->notice(
                'Applying migration "%s"...',
                $migration->getDescription()
            );
            $migration->up($this->getDatabaseConnection()->getPdo());
            $this->getLog()->notice('Done.');
        }
    }
}