<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Console\Command;

use Acronis\App\Services\Redis\PureRedisAwareTrait;
use Acronis\App\Services\Redis\RedisClient;
use LogInterface;
use Str;
use Symfony\Component\Lock\Factory;
use Symfony\Component\Lock\Store\RedisStore;

/**
 * Implement method getCommandName, getLog or use this trait only in console commands extending Loggable
 * @method string getCommandName()
 * @method LogInterface getLog()
 * @package Acronis\App\Console\Command
 */
trait LockableTrait
{
    static protected $lockPrefix = 'lock';

    use PureRedisAwareTrait;

    /**
     * @param callable $method
     * @param string $lockId
     */
    public function executeWithLock(callable $method, $arguments)
    {
        $commandName = $this->getCommandName();
        $lockId = $this->generateLockId($commandName, $arguments);
        $store = new RedisStore($this->getPureRedis());
        $factory = new Factory($store);
        $lock = $factory->createLock($lockId);

        if (!$lock->acquire()) {
            $this->getLog()->warning(
                'Unable to acquire lock "%s", another process acquired the lock.'
            );
            return;
        }

        try {
            $method();
        } finally {
            $lock->release();
        }
    }

    public function releaseAllLocks($commandName)
    {
        $client = new RedisClient($this->getPureRedis());
        $client->deleteAll($this->generateLockId($commandName));
    }

    /**
     * @param $commandName
     * @param array $arguments
     * @return string
     */
    private function generateLockId($commandName, $arguments = [])
    {
        return Str::format(
            '%s_%s|%s',
            static::$lockPrefix, $commandName, implode(',', $arguments)
        );
    }
}