<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Console\Command;

use Acronis\App\Services\Logger\AppLoggerAwareTrait;
use Acronis\App\Services\Logger\LoggerAwareTrait;
use Log;
use Monolog\Formatter\LineFormatter;
use Monolog\Handler\StreamHandler;
use Monolog\Logger;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

abstract class Loggable extends Command
{
    use LoggerAwareTrait;
    use AppLoggerAwareTrait;

    protected static $verbosityLevelMap = array(
        OutputInterface::VERBOSITY_NORMAL => Logger::NOTICE,
        OutputInterface::VERBOSITY_VERBOSE => Logger::INFO,
        OutputInterface::VERBOSITY_VERY_VERBOSE => Logger::DEBUG,
        OutputInterface::VERBOSITY_DEBUG => Logger::DEBUG,
    );

    protected function configure()
    {
        $this
            ->setName($this->getCommandName())
            ->setDescription($this->getCommandDescription())
            ->setHelp($this->getCommandHelp());

        parent::configure();
    }

    protected function initialize(InputInterface $input, OutputInterface $output)
    {
        parent::initialize($input, $output);

        $verbosity = $output->getVerbosity();
        $level = isset(static::$verbosityLevelMap[$verbosity])
            ? static::$verbosityLevelMap[$verbosity]
            : Logger::WARNING;

        $stream = new StreamHandler('php://stdout', $level);

        $format = Log::FIELD_TIME . ' ' . Log::FIELD_REQUEST . ' ' . Log::FIELD_TYPE . ' ' . Log::FIELD_MESSAGE . PHP_EOL;
        $formatter = new LineFormatter($format, null, true);

        $stream->setFormatter($formatter);
        $this->getLogger()->pushHandler($stream);
    }

    /**
     * @return string
     */
    abstract protected function getCommandName();

    /**
     * @return string
     */
    abstract protected function getCommandHelp();

    /**
     * @return string
     */
    abstract protected function getCommandDescription();
}
