<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Console\Command\UsageReport;

use Acronis\App\Console\Command\Loggable;
use Acronis\App\Services\Apsc\ApscInstanceFactoryTrait;
use Acronis\App\Services\Logger\AppLoggerAwareTrait;
use APSResource;
use RqlBuilder;
use Str;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

class Master extends Loggable
{
    use ApscInstanceFactoryTrait;

    const COMMAND_NAME = 'usage-report:master';

    protected function getCommandName()
    {
        return static::COMMAND_NAME;
    }

    protected function getCommandHelp()
    {
        return <<<TEXT
<info>
Usage report aggregation master command. It should be registered as cron job and run successfully at least one time
a day for all datacetners and application instances in order to be able to synchronize usage via OSA periodic task.
</info>
TEXT;
    }

    protected function getCommandDescription()
    {
        return 'Usage report aggregation master command.';
    }

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @return int|null|void
     * @throws \Exception
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        foreach (\APS\ControllerProxy::listInstances() as $apsInstanceId) {
            $datacenters = $this->fetchDatacentersForApsInstance($apsInstanceId);
            $manageCommandPath = realpath(DIR_BIN . DS . 'manage.php');

            $childPids = [];

            /** @var \datacenter $datacenter */
            foreach ($datacenters as $datacenter) {
                $pid = pcntl_fork();
                if ($pid == -1) {
                    $this->getLog()->error(
                        'Unable to fork child process for managing usage report aggregation with APS instance ID "%s" and datacenter ID "%s."',
                        $apsInstanceId, $datacenter->aps->id
                    );
                    exit(1);
                } else if ($pid) {
                    $childPids[] = $pid;
                    continue;
                } else {
                    pcntl_exec(
                        '/usr/bin/env',
                        [
                            'php',
                            $manageCommandPath,
                            Process::COMMAND_NAME,
                            $apsInstanceId,
                            $datacenter->aps->id
                        ]
                    );

                    return;
                }
            }

            foreach ($childPids as $pid) {
                pcntl_waitpid($pid, $status);
            }
        }
    }

    /**
     * @param $apsInstanceId
     * @return array
     * @throws \Exception
     */
    private function fetchDatacentersForApsInstance($apsInstanceId)
    {
        $apsc = $this->createApscInstance($apsInstanceId);
        $rql = RqlBuilder::createInstance()
            ->implementing(APSResource::CLOUD_BACKUP_DATACENTER_TYPE)
            ->getQuery();

        return $apsc->getResources($rql);
    }
}