<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Database\Model;

use Illuminate\Database\Eloquent\Model;
use Str;

/**
 * Class ReportEntry
 * @package  Acronis\App\Database\Model
 * @property int $id
 * @property string $date
 * @property string $file_path
 * @property string $datacenter_id
 * @property string $aps_instance_id
 * @property string $report_id
 * @property string $stored_report_id
 * @property integer $status
 */
class ReportEntry extends Model
{
    const TABLE_NAME = 'reports';

    public $timestamps = true;

    protected $table = self::TABLE_NAME;

    /**
     * Statuses are ordered by
     * corresponding steps of usage processing
     */
    const REPORT_STATUS_ERROR = -1;
    const REPORT_STATUS_NEW = 1;
    const REPORT_STATUS_ORDERED = 2;
    const REPORT_STATUS_READY = 3;
    const REPORT_STATUS_DOWNLOADED = 4;
    const REPORT_STATUS_AGGREGATED = 5;
    const REPORT_STATUS_ERASED = 6;

    const REPORT_STATUS_UNKNOWN_DESCRIPTION = 'Unknown';

    private static $allowedStatuses = [
        self::REPORT_STATUS_ERROR => 'Error',
        self::REPORT_STATUS_NEW => 'New',
        self::REPORT_STATUS_ORDERED => 'Ordered',
        self::REPORT_STATUS_READY => 'Ready for downloading',
        self::REPORT_STATUS_DOWNLOADED => 'Downloaded and saved',
        self::REPORT_STATUS_AGGREGATED => 'Aggregated',
        self::REPORT_STATUS_ERASED => 'Erased',
    ];

    public static function getStatusDescription($status)
    {
        return isset(self::$allowedStatuses[$status])
            ? self::$allowedStatuses[$status]
            : self::REPORT_STATUS_UNKNOWN_DESCRIPTION;
    }

    /**
     * @param $status
     * @throws \InvalidReportStatus
     */
    public function checkReportStatusIsAllowed($status)
    {
        if (!isset(self::$allowedStatuses[$status])) {
            throw new \InvalidReportStatus(Str::format(
                'Report status "%s" not allowed.',
                $status
            ));
        }
    }
}