<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Database\Model;

use Acronis\App\Services\Database\DatabaseAwareTrait;
use Illuminate\Database\Eloquent\Collection;

class ReportEntryHelper
{
    use DatabaseAwareTrait;

    public function __construct()
    {
        // Booting Eloquent for the first time
        $this->getDatabaseConnection();
    }

    /**
     * @return ReportEntry[]|Collection
     */
    public function getAllReports()
    {
        return ReportEntry::all();
    }

    /**
     * @param string $datacenterId
     * @param string $date
     * @param string $apsInstanceId
     * @return ReportEntry
     */
    public function getOrCreateReport($datacenterId, $date, $apsInstanceId)
    {
        $currentReports = $this->getReports($datacenterId, $date);

        if (count($currentReports)) {
            return $currentReports[0];
        }

        return $this->createReport($datacenterId, $date, $apsInstanceId);
    }

    /**
     * @param string $datacenterId
     * @param string $date
     * @param string $apsInstanceId
     * @return ReportEntry
     */
    public function createReport($datacenterId, $date, $apsInstanceId)
    {
        $reportEntry = new ReportEntry();
        $reportEntry->status = ReportEntry::REPORT_STATUS_NEW;
        $reportEntry->datacenter_id = $datacenterId;
        $reportEntry->date = $date;
        $reportEntry->aps_instance_id = $apsInstanceId;
        $reportEntry->save();

        return $reportEntry;
    }

    /**
     * @param string $datacenterId
     * @param string $date
     * @param bool $fetchErased
     * @return ReportEntry[]
     */
    public function getReports($datacenterId, $date, $fetchErased = false)
    {
        $statement = ReportEntry::where('datacenter_id', '=', $datacenterId)
            ->where('date', '=', $date);

        if (!$fetchErased) {
            $statement = $statement->where('status', '!=', ReportEntry::REPORT_STATUS_ERASED);
        }

        return $statement->get();
    }

    /**
     * @param $date
     * @return ReportEntry[]|Collection
     */
    public function getReportsTillDate($date, $fetchErased = false)
    {
        $statement = ReportEntry::where('date', '<=', $date);

        if (!$fetchErased) {
            $statement = $statement->where('status', '!=', ReportEntry::REPORT_STATUS_ERASED);
        }

        return $statement->get();
    }
}