<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Model_ConnectionChecker implements Model_ConnectionCheckerInterface
{
    const VERIFY_ENDPOINT = 0;
    const VERIFY_BAAS_ACRONIS_COM = 1;
    const VERIFY_BAAS_CREDENTIALS = 4;
    const VERIFY_POA_HOST = 5;
    const VERIFY_OPEN_API = 6;
    const SSL_PORT = 443;
    const OPEN_API_PORT = 8440;
    const UNDEFINED_TYPE = 'Undefined check type %s';

    /** @var string */
    private $_partnerLogin;
    /** @var string */
    private $_partnerPassword;
    /** @var string */
    private $_baasUrl;
    /** @var string */
    private $_controllerHost;
    /** @var FacadeInterface */
    private $_facade;

    /**
     * @param string $partnerLogin
     * @param string $partnerPassword
     * @param string $baasUrl
     * @param string $controllerHost
     * @param FacadeInterface $facade
     */
    public function __construct(
        $partnerLogin,
        $partnerPassword,
        $baasUrl,
        $controllerHost,
        FacadeInterface $facade = NULL
    )
    {
        $this->_partnerLogin = $partnerLogin;
        $this->_partnerPassword = $partnerPassword;
        $this->_baasUrl = $baasUrl;
        $this->_controllerHost = $controllerHost;
        $this->_facade = is_null($facade) ? new Facade() : $facade;
    }

    /**
     * @param string $type
     * @throws Exception
     * @return Model_ConnectionChecker_CheckerInterface
     */
    public function getCheckerByType($type)
    {
        switch($type)
        {
            case self::VERIFY_ENDPOINT:
                return new Model_ConnectionChecker_EndpointConnection();
            case self::VERIFY_BAAS_ACRONIS_COM:
                return new Model_ConnectionChecker_HostConnection($this->_baasUrl, self::SSL_PORT);
            case self::VERIFY_BAAS_CREDENTIALS:
                return new Model_ConnectionChecker_BaasCredentials(
                    $this->_partnerLogin,
                    $this->_partnerPassword,
                    $this->_baasUrl,
                    $this->_facade->getMspApi($this->_baasUrl)
                );
            case self::VERIFY_POA_HOST:
                return new Model_ConnectionChecker_OpenApiHostConnection(
                    $this->_controllerHost,
                    self::OPEN_API_PORT
                );
            case self::VERIFY_OPEN_API:
                return new Model_ConnectionChecker_OpenApiRequest(
                    $this->_facade->getOpenApi($this->_controllerHost)
                );
            default:
                throw new Exception(sprintf(self::UNDEFINED_TYPE, $type));
        }
    }

    /**
     * @return string
     */
    public function getControllerHost()
    {
        return $this->_controllerHost;
    }

    /**
     * @return FacadeInterface
     */
    public function getFacade()
    {
        return $this->_facade;
    }

    /**
     * @return string
     */
    public function getPartnerLogin()
    {
        return $this->_partnerLogin;
    }

    /**
     * @return string
     */
    public function getPartnerPassword()
    {
        return $this->_partnerPassword;
    }

    /**
     * @return string
     */
    public function getBaasUrl()
    {
        return $this->_baasUrl;
    }
}
