<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Model_ConnectionChecker_CurlConnector implements Model_ConnectionChecker_CurlConnectorInterface
{
    const TIMEOUT = 30;
    const CONNECTION_TIMEOUT = 10;
    const HOST_WITH_PORT = '{host}:{port}';

    /** @var  CurlInterface */
    private $_curl;

    /**
     * @param CurlInterface $curl
     */
    public function __construct(CurlInterface $curl = NULL)
    {
        $this->_curl = is_null($curl) ? new Curl() : $curl;
    }

    /**
     * @param string $host
     * @param string $port
     * @return bool
     * @throws Exception
     */
    public function tryConnectToHost($host, $port)
    {
        $uri = strtr(self::HOST_WITH_PORT, array('{host}' => $host, '{port}' => $port));
        $this->_curl->setOption(CURLOPT_URL, $uri);
        $this->_curl->setOption(CURLOPT_CONNECTTIMEOUT, self::CONNECTION_TIMEOUT);
        $this->_curl->setOption(CURLOPT_TIMEOUT, self::TIMEOUT);
        $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
        $this->_curl->setOption(CURLOPT_SSL_VERIFYHOST, false);
        $this->_curl->setOption(CURLOPT_SSL_VERIFYPEER, false);

        $response = $this->_curl->execute();
        if ($response === false)
        {
            throw new Exception($this->_curl->getError(), $this->_curl->getErrorNo());
        }
        return true;
    }
} 