<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

/**
 * Class Model_Resource_Tenant_CustomerCCPv2
 */
class Model_Resource_Tenant_CustomerCCPv2 extends Model_Resource_Tenant_Customer
{
    public function onTenantAvailable($event)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->tenant->aps->id, $this->tenant->accountId, $this->tenant->subscriptionId, __METHOD__
        ));

        parent::onTenantAvailable($event);

        if ($this->tenant->ismigration) {
            return;
        }

        $applicationConfig = $this->getApplicationConfig();
        if (!$applicationConfig->service_auto_assignation) {
            return;
        }
        $admin = $this->getApsApi()->getAdmin($this->tenant);
        if (is_null($admin)) {
            return;
        }
        $this->provisionDefaultUser($admin);
    }

    protected function createDefaultUserResource($tenant, $serviceUser)
    {
        $userQuota = Service_APSApi_Api::PROFILE_USER_QUOTA_LIMITS;
        $backupNotification = Enum_NotificationsBackup::DISABLE_NOTIFICATIONS;
        $managementNotification = false;

        $profile = $tenant->profile;
        if (!$profile->isEmpty()) {
            $userQuota = $profile->userQuota;
            $backupNotification = $profile->backupNotification;
            $managementNotification = (bool)$profile->managementNotification;
        }

        $user = \APS\TypeLibrary::newResourceByTypeId(\APS\TypeLibrary::getTypeByServiceId('users'));
        $user->user->set($serviceUser);
        $user->tenant->set($tenant);
        $user->notifyBackup = $backupNotification;
        $user->notificationsManagement = $managementNotification;

        $user->userStorageUsage = new Counter();
        $user->userLocalStorageUsage = new Counter();
        $user->userServerCount = new Counter();
        $user->UserVmCount = new Counter();
        $user->UserWorkstationCount = new Counter();
        $user->UserMobileDeviceCount = new Counter();
        $user->UserMailboxCount = new Counter();
        $user->UserWebsiteCount = new Counter();

        if ($userQuota) {
            $user->userStorageUsage->limit = $this->getResourceLimit($tenant->StorageUsage->limit, $userQuota);
            $user->userLocalStorageUsage->limit = $this->getResourceLimit($tenant->LocalStorageUsage->limit, $userQuota);
            $user->userServerCount->limit = $this->getResourceLimit($tenant->ServerCount->limit, $userQuota);
            $user->UserVmCount->limit = $this->getResourceLimit($tenant->VmCount->limit, $userQuota);
            $user->UserWorkstationCount->limit = $this->getResourceLimit($tenant->WorkstationCount->limit, $userQuota);
            $user->UserMobileDeviceCount->limit = $this->getResourceLimit($tenant->MobileDeviceCount->limit, $userQuota);
            $user->UserMailboxCount->limit = $this->getResourceLimit($tenant->MailboxCount->limit, $userQuota);
            $user->UserWebsiteCount->limit = $this->getResourceLimit($tenant->WebsiteCount->limit, $userQuota);
        } else {
            $user->userStorageUsage->limit = self::UNLIMITED_VALUE;
            $user->userLocalStorageUsage->limit = self::UNLIMITED_VALUE;
            $user->userServerCount->limit = self::UNLIMITED_VALUE;
            $user->UserVmCount->limit = self::UNLIMITED_VALUE;
            $user->UserWorkstationCount->limit = self::UNLIMITED_VALUE;
            $user->UserMobileDeviceCount->limit = self::UNLIMITED_VALUE;
            $user->UserMailboxCount->limit = self::UNLIMITED_VALUE;
            $user->UserWebsiteCount->limit = self::UNLIMITED_VALUE;
        }

        $user->privileged = users::ADMINISTRATOR_PRIVILEGES_YES;
        $user->activateDuringCreation = false;
        return $user;
    }

    /**
     * @param $serviceUser
     */
    protected function provisionDefaultUser($serviceUser)
    {
        $tenant = $this->tenant;

        $user = $this->createDefaultUserResource($tenant, $serviceUser);

        $apsc = $this->getApsc()->impersonate($tenant);
        $apsc->resetSession();
        $apsc->provisionResource($user);
    }

    /**
     * @param $limit
     * @param $userQuota
     * @return int
     */
    private function getResourceLimit($limit, $userQuota)
    {
        if (is_null($limit)) {
            return 0;
        }
        if ($limit == 0) {
            return 0;
        }
        if ($userQuota === Service_APSApi_Api::PROFILE_USER_QUOTA_ANY ||
            $userQuota === Service_APSApi_Api::PROFILE_USER_QUOTA_OFF
        ) {
            return self::UNLIMITED_VALUE;
        }
        return $limit; // Service_APSApi_Api::PROFILE_USER_QUOTA_LIMITS
    }

    /**
     * @return object
     */
    protected function getApplicationConfig()
    {
        return Settings::getInstance()->getApplicationConfig();
    }
}
