<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Model_ScreenNotification_Struct_MessageConverter implements Model_ScreenNotification_Struct_MessageConverterInterface
{
    const FIELD_ID = "id";
    const FIELD_TYPE = "type";
    const FIELD_STATUS = "status";
    const FIELD_MESSAGE = "message";
    const FIELD_DETAILS = "details";
    const FIELD_FULL_DETAILS = "fullDetails";
    const FIELD_LINK = "link";
    const FIELD_LINK_MORE = "linkMore";
    const FIELD_LINK_MORE_TEXT = "linkMoreText";
    const FIELD_PACKAGE_ID = "packageId";
    const FIELD_UPDATED = "updated";
    const FIELD_RESOLVED = "resolved";
    const FIELD_CREATED = "created";
    const FIELD_MODIFIED = "modified";
    const FIELD_ACCOUNT_ID = "accountId";
    const FIELD_USER_ID = "userId";
    const FIELD_CONTEXT = "context";
    const FIELD_INITIATOR_ID = "initiatorId";
    const FIELD_INITIATOR = "initiator";
    const FIELD_SERVICE = "service";
    const FIELD_REQUIRED = "required";
    const FIELD_PARENT_ID = "parentId";
    const FIELD_IP_ADDRESS = "ipAddress";

    const PREFIX_GET = 'get';
    const PREFIX_SET = 'set';

    /**
     * @param array $array
     * @param $object
     * @param $field
     */
    private static function _fromArrayToObject(array &$array, $object, $field)
    {
        if (isset($array[$field]))
        {
            $value = $array[$field];
            $methodReflection = new ReflectionMethod($object, self::PREFIX_SET . ucfirst($field));
            $methodReflection->invoke($object, $value);
        }
    }

    /**
     * @param array $array
     * @param $object
     * @param $field
     */
    private static function _toArrayFromObject(array &$array, $object, $field)
    {
        $methodReflection = new ReflectionMethod($object, self::PREFIX_GET . ucfirst($field));
        $value = $methodReflection->invoke($object);
        if (isset($value))
        {
            $array[$field] = $value;
        }
    }

    /**
     * @param array $array
     * @param $object
     * @param $field
     */
    private static function _fromArrayToObjectLocalizedMessage(array &$array, $object, $field)
    {
        if (isset($array[$field]))
        {
            $value = $array[$field];
            if (is_array($value))
            {
                $value = Model_ScreenNotification_Struct_LocalizedMessageConverter::fromArray($value);
            }
            elseif (is_object($value) && $value instanceof stdClass)
            {
                $value = Model_ScreenNotification_Struct_LocalizedMessageConverter::fromArray((array)$value);
            }
            
            $methodReflection = new ReflectionMethod($object, self::PREFIX_SET . ucfirst($field));
            $methodReflection->invoke($object, $value);
        }
    }

    /**
     * @param array $array
     * @param $object
     * @param $field
     */
    private static function _toArrayFromObjectLocalizedMessage(array &$array, $object, $field)
    {
        $methodReflection = new ReflectionMethod($object, self::PREFIX_GET . ucfirst($field));
        $value = $methodReflection->invoke($object);
        if (isset($value))
        {
            $array[$field] = Model_ScreenNotification_Struct_LocalizedMessageConverter::toArray($value);
        }
    }

    /**
     * @param array $array
     *
     * @return Model_ScreenNotification_Struct_MessageInterface
     */
    public static function fromArray(array $array)
    {
        $object = new Model_ScreenNotification_Struct_Message();
        self::_fromArrayToObject($array, $object, self::FIELD_ID);
        self::_fromArrayToObject($array, $object, self::FIELD_TYPE);
        self::_fromArrayToObject($array, $object, self::FIELD_STATUS);
        self::_fromArrayToObjectLocalizedMessage($array, $object, self::FIELD_MESSAGE);
        self::_fromArrayToObjectLocalizedMessage($array, $object, self::FIELD_DETAILS);
        self::_fromArrayToObjectLocalizedMessage($array, $object, self::FIELD_FULL_DETAILS);
        self::_fromArrayToObject($array, $object, self::FIELD_LINK);
        self::_fromArrayToObject($array, $object, self::FIELD_LINK_MORE);
        self::_fromArrayToObjectLocalizedMessage($array, $object, self::FIELD_LINK_MORE_TEXT);
        self::_fromArrayToObject($array, $object, self::FIELD_PACKAGE_ID);
        self::_fromArrayToObject($array, $object, self::FIELD_UPDATED);
        self::_fromArrayToObject($array, $object, self::FIELD_RESOLVED);
        self::_fromArrayToObject($array, $object, self::FIELD_CREATED);
        self::_fromArrayToObject($array, $object, self::FIELD_MODIFIED);
        self::_fromArrayToObject($array, $object, self::FIELD_ACCOUNT_ID);
        self::_fromArrayToObject($array, $object, self::FIELD_USER_ID);
        self::_fromArrayToObject($array, $object, self::FIELD_CONTEXT);
        self::_fromArrayToObject($array, $object, self::FIELD_INITIATOR_ID);
        self::_fromArrayToObject($array, $object, self::FIELD_INITIATOR);
        self::_fromArrayToObject($array, $object, self::FIELD_SERVICE);
        self::_fromArrayToObject($array, $object, self::FIELD_REQUIRED);
        self::_fromArrayToObject($array, $object, self::FIELD_PARENT_ID);
        self::_fromArrayToObject($array, $object, self::FIELD_IP_ADDRESS);

        return $object;
    }

    /**
     * @param Model_ScreenNotification_Struct_MessageInterface $object
     * @return array
     */
    public static function toArray(Model_ScreenNotification_Struct_MessageInterface $object)
    {
        $array = array();

        self::_toArrayFromObject($array, $object, self::FIELD_ID);
        self::_toArrayFromObject($array, $object, self::FIELD_TYPE);
        self::_toArrayFromObject($array, $object, self::FIELD_STATUS);
        self::_toArrayFromObjectLocalizedMessage($array, $object, self::FIELD_MESSAGE);
        self::_toArrayFromObjectLocalizedMessage($array, $object, self::FIELD_DETAILS);
        self::_toArrayFromObjectLocalizedMessage($array, $object, self::FIELD_FULL_DETAILS);
        self::_toArrayFromObject($array, $object, self::FIELD_LINK);
        self::_toArrayFromObject($array, $object, self::FIELD_LINK_MORE);
        self::_toArrayFromObjectLocalizedMessage($array, $object, self::FIELD_LINK_MORE_TEXT);
        self::_toArrayFromObject($array, $object, self::FIELD_PACKAGE_ID);
        self::_toArrayFromObject($array, $object, self::FIELD_UPDATED);
        self::_toArrayFromObject($array, $object, self::FIELD_RESOLVED);
        self::_toArrayFromObject($array, $object, self::FIELD_CREATED);
        self::_toArrayFromObject($array, $object, self::FIELD_MODIFIED);
        self::_toArrayFromObject($array, $object, self::FIELD_ACCOUNT_ID);
        self::_toArrayFromObject($array, $object, self::FIELD_USER_ID);
        self::_toArrayFromObject($array, $object, self::FIELD_CONTEXT);
        self::_toArrayFromObject($array, $object, self::FIELD_INITIATOR_ID);
        self::_toArrayFromObject($array, $object, self::FIELD_INITIATOR);
        self::_toArrayFromObject($array, $object, self::FIELD_SERVICE);
        self::_toArrayFromObject($array, $object, self::FIELD_REQUIRED);
        self::_toArrayFromObject($array, $object, self::FIELD_PARENT_ID);
        self::_toArrayFromObject($array, $object, self::FIELD_IP_ADDRESS);

        return $array;
    }
} 