<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Model_TenantManager implements Model_TenantManagerInterface
{
    const STATUS_ACTIVE = 1;
    const STATUS_INACTIVE = 0;
    /** @var  Service_MspApiClient_Request_Struct_Factory */
    private $groupInfoFactory;
    /** @var  Service_MspApiClient_ApiInterface */
    private $mspApi;

    /**
     * @param Service_MspApiClient_ApiInterface $mspApi
     * @param Service_MspApiClient_Request_Struct_Factory $groupInfoFactory
     */
    public function __construct(
        Service_MspApiClient_ApiInterface $mspApi,
        Service_MspApiClient_Request_Struct_Factory $groupInfoFactory = null
    ) {
        $this->mspApi = $mspApi;
        $this->groupInfoFactory = is_null($groupInfoFactory) ?
            new Service_MspApiClient_Request_Struct_Factory() :
            $groupInfoFactory;
    }

    /**
     * @param int $mspGroupId
     * @param int $version
     * @return int
     */
    public function enableGroup($mspGroupId, $version = null)
    {
        return $this->updateGroupStatus($mspGroupId, self::STATUS_ACTIVE, $version);
    }

    /**
     * @param int $mspGroupId
     * @param int $version
     * @return int
     */
    public function disableGroup($mspGroupId, $version = null)
    {
        return $this->updateGroupStatus($mspGroupId, self::STATUS_INACTIVE, $version);
    }

    /**
     * @param int $groupId
     * @param Service_MspApiClient_Request_Struct_UpdateGroupInterface $groupInfo
     * @param int $version
     * @return int
     */
    public function updateGroup(
        $groupId,
        Service_MspApiClient_Request_Struct_UpdateGroupInterface $groupInfo,
        $version = null
    ) {
        $tries = 10;
        do {
            if (is_null($version)) {
                $oldGroupInfo = $this->mspApi->getGroupById($groupId);
                if (is_null($oldGroupInfo)) {
                    return null;
                }
                $version = $oldGroupInfo->getVersion();
            }

            try {
                return $this->mspApi->updateGroup($groupId, $groupInfo, $version);
            } catch (Service_BaseClient_HTTPException $e) {
                if ($e->getCode() != Service_BaseClient_HTTPException::HTTP_CODE_VERSION_EXPIRED) {
                    throw new $e;
                }
            }
            $version = null;
            $tries--;
            sleep(1);
        } while ($tries);
        throw new $e;
    }

    /**
     * @param int $mspGroupId
     * @param int $status
     * @param int $version
     * @return int
     */
    private function updateGroupStatus($mspGroupId, $status, $version = null)
    {
        $updatedGroup = $this->groupInfoFactory->getUpdateGroup();
        $updatedGroup->setStatus($status);
        return $this->updateGroup($mspGroupId, $updatedGroup, $version);
    }
}
