<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

abstract class Service_BaseClient_Response_Base implements Service_BaseClient_Response_IResponse
{
    const PROPERTY_NOT_FOUND = 'Property with name %s not found';

    /** @var mixed */
    private $_data = null;

    /**
     * @param string|null $responseBody
     */
    public function __construct($responseBody = null)
    {
        $responseData = json_decode($responseBody, true);
        $this->_data = is_array($responseData) ? $this->_init($responseData) : null;
    }

    /**
     * @param array $responseData
     */
    abstract protected function _init(array $responseData);

    /**
     * @return mixed
     */
    public function getData()
    {
        return $this->_data;
    }

    /**
     * @param string $property
     * @param array $data
     * @param bool $throwException
     * @return mixed
     * @throws Exception
     */
    protected function _getProperty($property, array $data, $throwException = true)
    {
        if (array_key_exists($property, $data))
        {
            return $data[$property];
        }
        $errorMessage = sprintf(self::PROPERTY_NOT_FOUND, $property);
        if ($throwException)
        {
            throw new Exception($errorMessage);
        }
        Log::getInstance()->warning($errorMessage);
        return null;
    }

    /**
     * @param array $nestedProperties
     * @param array $data
     * @param bool $throwException
     * @return mixed
     */
    protected function _getNestedProperty(array $nestedProperties, array $data, $throwException = true)
    {
        foreach($nestedProperties as $property)
        {
            $data = $this->_getProperty($property, $data, $throwException);
        }
        return $data;
    }
}
