<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_BaseClient_RestClient implements Service_BaseClient_IRestClient
{
    const REQUEST_FAILED = '%s request (%s) failed: %s';

    /** @var Service_BaseClient_RestTransport */
    private $_rpcTransport;
    /** @var string */
    private $_apiUri = null;

    /**
     * @param Service_BaseClient_IRestTransport $rpcTransport
     */
    public function __construct(Service_BaseClient_IRestTransport $rpcTransport = NULL)
    {
        $this->_rpcTransport = is_null($rpcTransport) ? new Service_BaseClient_RestTransport() : $rpcTransport;
    }

    public function refresh()
    {
        $this->_rpcTransport->refresh();
    }

    /**
     * @param Service_BaseClient_Request_IRequest $request
     * @param string $authHeaders
     * @throws Exception
     * @return Service_BaseClient_Response_IResponse|NULL
     */
    public function execute(Service_BaseClient_Request_IRequest $request, $authHeaders = NULL)
    {
        $this->_rpcTransport->setSendingStrategy($request->getSendingStrategy());

        $headers = $request->getHeaders();
        if (is_array($authHeaders))
        {
            $headers = is_array($headers) ? array_merge($headers, $authHeaders) : $authHeaders;
        }

        $url = $this->_apiUri . $request->getResource();
        list($code, $responseBody) = $this->_rpcTransport->execute(
            $url,
            $request->getMethod(),
            $request->getRequestBody(),
            $headers
        );
        $response = NULL;
        try
        {
            $response = $request->getExpectedResponse($code, $responseBody);
        }
        catch (Exception $exception)
        {
            $message = $exception->getMessage();
            Log::getInstance()->error(sprintf(self::REQUEST_FAILED, $request->getMethod(), $url, $message));
            throw new Exception($message);
        }
        return $response;
    }

    /**
     * @param string $apiUri
     */
    public function setApiUri($apiUri)
    {
        $this->_apiUri = $apiUri;
    }

    /**
     * @return string
     */
    public function getApiUri()
    {
        return $this->_apiUri;
    }

    /**
     * @param string $cookieFile
     */
    public function setCookieFile($cookieFile)
    {
        $this->_rpcTransport->setCookieFile($cookieFile);
    }

    /**
     * @return bool
     */
    public function hasCookieFile()
    {
        return $this->_rpcTransport->hasCookieFile();
    }

    public function invalidateCookies()
    {
        $this->_rpcTransport->invalidateCookies();
    }
}
