<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

interface Service_FileInterface
{
    /**
     * @return string
     */
    public function getPath();

    /**
     * @return string
     */
    public function getName();

    /**
     * @return bool
     */
    public function isExists();

    /**
     * @return bool
     */
    public function isReadable();

    /**
     * @return bool
     */
    public function isWritable();

    /**
     * @param string $mode
     * @return bool|resource
     */
    public function open($mode);

    /**
     * @return bool
     */
    public function close();

    /**
     * @return bool
     */
    public function delete();

    /**
     * @param string $dataToWrite
     * @param int $length
     * @return bool false|int
     */
    public function write($dataToWrite, $length = NULL);

    /**
     * @param string $group
     * @return bool
     */
    public function chgrp($group);

    /**
     * @param integer $mode octal number that represents file rights
     * @return bool
     */
    public function chmod($mode);

    /**
     * @param string $owner
     * @return bool
     */
    public function chown($owner);

    /**
     * @param int $createTime
     * @param int $accessTime
     *
     * @return void
     */
    public function create($createTime = NULL, $accessTime = NULL);

    /**
     * @param array $dataToWrite
     * @param string $delimiter
     * @param string $enclosure
     * @return bool false|int Number of bytes have been written
     */
    public function fputcsv(array $dataToWrite, $delimiter = ',', $enclosure = '"');

    /**
     * @param string $content
     * @param int $flags
     * @param resource $context
     * @return int
     */
    public function putContents($content, $flags = NULL, $context = NULL);

    /**
     * @param int $flags
     * @param resource $context
     * @param int $offset
     * @param int $maxlen
     * @return string
     */
    public function getContents($flags = NULL, $context = NULL, $offset = NULL, $maxlen = NULL);

    /**
     * @param          $destinationPath
     * @param resource $context
     * @return bool
     */
    public function copy($destinationPath, $context = NULL);

    /**
     * @param int $operation
     * @return bool
     */
    public function lock($operation);

    /**
     * @return string
     */
    public function mime();

    /**
     * @return int Unix timestamp
     */
    public function lastAccessTime();

    /**
     * @return int Unix timestamp
     */
    public function createdOn();

    /**
     * @return int
     */
    public function getSize();
}
