<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

interface Service_MspApiClient_ApiInterface
{
    /**
     * @param string $login
     * @param string $baasUrl
     * @param bool $force
     * @return Service_MspApiClient_Response_Struct_Account
     */
    public function getAccountInfo($login, $baasUrl, $force = false);
    /**
     * @param int $adminId
     * @return string
     */
    public function getImpersonationToken($adminId);

    /**
     * @param int $userId
     * @return string
     */
    public function getImpersonationUserToken($userId);

    /**
     * @param string $login
     * @param string $password
     * @param string $url
     * @param bool $force
     * @throws Exception
     */
    public function setCredentials($login, $password, $url, $force = false);

    /**
     * @return Service_MspApiClient_Response_Struct_Login
     * @internal param bool $force
     */
    public function login();

    /**
     * @param Service_MspApiClient_Request_Struct_UserInfo $userInfo
     * @param int $groupId
     * @return Service_MspApiClient_Response_Struct_CreateGroup
     */
    public function createUser(Service_MspApiClient_Request_Struct_UserInfo $userInfo, $groupId);

    /**
     * @param Service_MspApiClient_Request_Struct_AdministratorInfo $userInfo
     * @param int $groupId
     * @return Service_MspApiClient_Response_Struct_CreateGroup
     */
    public function createAdministrator(Service_MspApiClient_Request_Struct_AdministratorInfo $userInfo, $groupId);

    /**
     * @param int $adminId
     * @param int $version
     * @return Service_MspApiClient_Response_Version
     */
    public function deleteAdministrator($adminId, $version);

    /**
     * @param int $adminId
     * @return Service_MspApiClient_Response_Struct_AdministratorInfo
     */
    public function getAdministratorById($adminId);

    /**
     * @param Service_MspApiClient_Request_Struct_AdministratorUpdate $adminInfo
     * @param int $adminId
     * @param int $version
     * @return int
     */
    public function updateAdministrator(Service_MspApiClient_Request_Struct_AdministratorUpdate $adminInfo, $adminId, $version);

    /**
     * @param int $groupId
     * @return array
     */
    public function getAdministrators($groupId);

    /**
     * @param int $groupId
     * @return Service_MspApiClient_Response_Struct_UserItem[]
     */
    public function getUsers($groupId);

    /**
     * @param int $id
     * @return Service_MspApiClient_Response_Struct_UserInfo
     */
    public function getUserById($id);

    /**
     * @param int $groupId
     * @return Service_MspApiClient_Response_Struct_GroupInfo
     */
    public function getGroupById($groupId);

    /**
     * @param int $groupId
     * @return Service_MspApiClient_Response_Struct_Storage[]
     */
    public function getStoragesByGroupId($groupId);

    /**
     * @param int $userId
     * @return NULL
     */
    public function activateServiceUser($userId);

    /**
     * @param Service_MspApiClient_Request_Struct_CreateGroupInterface $groupInfo
     * @param int|null $parentGroupId
     * @return Service_MspApiClient_Response_Struct_CreateGroup
     */
    public function createGroup(
        Service_MspApiClient_Request_Struct_CreateGroupInterface $groupInfo,
        $parentGroupId = null
    );

    /**
     * @param int $userId
     * @param Service_MspApiClient_Request_Struct_UpdateUserInfo $userInfo
     * @param int $version
     * @return int
     * @throws Service_BaseClient_HTTPException
     */
    public function updateUser($userId, Service_MspApiClient_Request_Struct_UpdateUserInfo $userInfo, $version);

    /**
     * @param int $userId
     * @param int $version
     * @return NULL
     */
    public function deleteUser($userId, $version);

    /**
     * @param int|null $parentGroupId
     * @return Service_MspApiClient_Response_Struct_GroupItem[]
     */
    public function getGroups($parentGroupId = null);

    /**
     * @param string $prefix
     * @return Service_MspApiClient_Response_Struct_SearchItem[]
     */
    public function search($prefix);

    /**
     * @param int $groupId
     * @param int $version
     * @return NULL
     */
    public function deleteGroup($groupId, $version);

    /**
     * @param int $groupId
     * @param string $language
     * @return Service_MspApiClient_Response_Struct_BackupConsole
     */
    public function getBackupConsole($groupId, $language = NULL);

    /**
     * @param string $token
     * @return Service_MspApiClient_Response_Struct_ActivateServiceUser
     */
    public function activate($token);

    /**
     * @param string $token
     * @param string $password
     * @return Service_MspApiClient_Response_Struct_ActivateServiceUser
     */
    public function activateAccountWithPassword($token, $password);

    /**
     * @param string $login
     * @param string $email
     * @return NULL
     * @throws Service_BaseClient_HTTPException
     */
    public function mailUpdate($login, $email);
        
    /**
     * @param $login
     * @return Service_MspApiClient_Response_Struct_ActivateServiceUser
     */
    public function mailActivate($login);

    /**
     * @param Service_MspApiClient_Request_Struct_PostMailActivateInterface $mailActivate
     * @return Service_MspApiClient_Response_Empty
     */
    public function postMailActivate(Service_MspApiClient_Request_Struct_PostMailActivateInterface $mailActivate);

    /**
     * @param string $mail
     * @return string
     */
    public function mailReset($mail);

    /**
     * @param string $token
     * @param string $password
     * @return Service_MspApiClient_Response_Struct_Account
     */
    public function resetPassword($token, $password);

    /**
     * @param int $groupId
     * @param Service_MspApiClient_Request_Struct_UpdateGroupInterface $groupInfo
     * @param int $version
     * @return int
     */
    public function updateGroup($groupId, Service_MspApiClient_Request_Struct_UpdateGroupInterface $groupInfo, $version);

    /**
     * @param int|null $groupId
     * @return array
     */
    public function getStorages($groupId = null);

    /**
     * @param int $domainController
     * @param int $externalGroupId
     * @return Service_MspApiClient_Response_MigrationGroup
     */
    public function getMigrationGroup($domainController, $externalGroupId);

    /**
     * @param int $domainController
     * @param int $externalUserId
     * @return Service_MspApiClient_Response_MigrationUser
     */
    public function getMigrationUser($domainController, $externalUserId);

    /**
     * @param $groupId
     * @return NULL
     */
    public function lockGroup($groupId);

    /**
     * @param $groupId
     * @return NULL
     */
    public function unlockGroup($groupId);

    /**
     * @return null|string
     */
    public function getAccountServerUrl();
    
    /**
     * @return int
     * @throws Exception
     * @throws Service_BaseClient_HTTPException
     */
    public function getAccountGroupId();

    /**
     * @return array
     */
    public function getAccountPrivileges();

    /**
     * @return void
     */
    public function resetCountersCache();

    /**
     * @return Service_MspApiClient_Response_Struct_LogicVersions
     */
    public function getLogicVersions();

    /**
     * @return bool | NULL
     */
    public function getApiMajorVersion();

    /**
     * @param Service_MspApiClient_Request_Struct_RequestUsageReport $reportAttributes
     * @param $groupId
     * @return mixed
     */
    public function requestUsageReport(Service_MspApiClient_Request_Struct_RequestUsageReport $reportAttributes, $groupId);

    /**
     * @param Service_MspApiClient_Request_Struct_UsageReport $reportAttributes
     * @param $groupId
     * @return mixed
     */
    public function getUsageReport(Service_MspApiClient_Request_Struct_UsageReport $reportAttributes, $groupId);

    /**
     * @param int $groupId
     * @return Service_MspApiClient_Response_Struct_Brand
     */
    public function getBrand($groupId);
}
