<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_MspApiClient_Request_CreateGroup extends Service_BaseClient_Request_Json
{
    const MESSAGE_FOR_HTTP_CODE_400 = 'One of mandatory fields is not specified or business rule error';
    const MESSAGE_FOR_HTTP_CODE_401 = 'Unauthorized';
    const MESSAGE_FOR_HTTP_CODE_403 = 'Forbidden';
    const MESSAGE_FOR_HTTP_CODE_404 = 'Parent group is not found or storage is not found';
    const MESSAGE_FOR_HTTP_CODE_409 = 'There is another group with same name on same level';
    const MESSAGE_FOR_HTTP_CODE_503 = 'Service Unavailable';

    const HTTP_CODE_SUCCESS = 201;
    const HTTP_CODE_INTERNAL_ERROR = 400;
    const HTTP_CODE_UNAUTHORIZED = 401;
    const HTTP_CODE_FORBIDDEN = 403;
    const HTTP_CODE_NOT_FOUND = 404;
    const HTTP_CODE_EXISTING_GROUP = 409;
    const HTTP_CODE_SERVICE_UNAVAILABLE = 503;

    const REQUEST_URL_FORMAT = '/groups/%s/children/';
    const FORMAT_UNDEFINED_RESPONSE = 'Undefined response code (%s) with body: %s';

    /**
     * @param Service_BaseClient_Request_Struct_IArrayRepresentation $requestObject
     * @param int $parentGroupId
     */
    public function __construct(
        Service_BaseClient_Request_Struct_IArrayRepresentation $requestObject,
        $parentGroupId
    ) {
        $this->_requestObject = $requestObject;
        $this->_resource = sprintf(self::REQUEST_URL_FORMAT, $parentGroupId);
        $this->_method = self::METHOD_POST;
    }

    /**
     * @param string $httpCode
     * @param string $responseBody
     * @throws Exception
     * @return Service_MspApiClient_Response_CreateNewUser
     */
    public function getExpectedResponse($httpCode = NULL, $responseBody = NULL)
    {
        switch ($httpCode)
        {
            case self::HTTP_CODE_SUCCESS:
                return new Service_MspApiClient_Response_CreateNewUser($responseBody);
            case self::HTTP_CODE_INTERNAL_ERROR:
                throw new Exception(self::MESSAGE_FOR_HTTP_CODE_400);
            case self::HTTP_CODE_UNAUTHORIZED:
                throw new Exception(self::MESSAGE_FOR_HTTP_CODE_401);
            case self::HTTP_CODE_FORBIDDEN:
                throw new Exception(self::MESSAGE_FOR_HTTP_CODE_403);
            case self::HTTP_CODE_NOT_FOUND:
                throw new Exception(self::MESSAGE_FOR_HTTP_CODE_404);
            case self::HTTP_CODE_EXISTING_GROUP:
                throw new Exception(self::MESSAGE_FOR_HTTP_CODE_409);
            case self::HTTP_CODE_SERVICE_UNAVAILABLE:
                throw new Exception(self::MESSAGE_FOR_HTTP_CODE_503);
            default:
                throw new Exception(sprintf(self::FORMAT_UNDEFINED_RESPONSE, $httpCode, $responseBody));
        }
    }
}
