<?php
/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

class Service_MspApiClient_Request_Struct_UserInfo implements
    Service_MspApiClient_Request_Struct_UserInfoInterface,
    Service_BaseClient_Request_Struct_IArrayRepresentation
{
    /** @var string */
    private $_language;
    /** @var string */
    private $_login;
    /** @var string */
    private $_email;
    /** @var string */
    private $_firstName;
    /** @var string */
    private $_lastName;
    /** @var Enum_NotificationsBackup */
    private $_notificationsBackup;
    /** @var int */
    private $_notificationsManagement = 0;
    /** @var Service_MspApiClient_Request_Struct_UserPrivileges */
    private $_privileges;
    /** @var integer */
    private $_privileged;
    /** @var int */
    private $_roles;

    /**
     * @param array $updateMap
     * @param Service_MspApiClient_Request_Struct_UserPrivileges $privileges
     */
    public function __construct(array $updateMap = [],
                                Service_MspApiClient_Request_Struct_UserPrivileges $privileges = NULL)
    {
        $this->_privileges = is_null($privileges)
            ? new Service_MspApiClient_Request_Struct_UserPrivileges()
            : $privileges;
    }

    /**
     * @return array
     */
    public function toArray()
    {
        $properties = [
            static::FIELD_LOGIN => $this->getLogin(),
            static::FIELD_EMAIL => $this->getEmail(),
            static::FIELD_FIRSTNAME => $this->getFirstName(),
            static::FIELD_LASTNAME => $this->getLastName(),
            static::FIELD_NOTIFICATIONS_BACKUP => is_null($this->getNotificationsBackup()) ? NULL: $this->getNotificationsBackup()->getValue(),
            static::FIELD_NOTIFICATIONS_MANAGEMENT => $this->getNotificationsManagement(),
            static::FIELD_PRIVILEGES => $this->getPrivileges()->toArray(),
            static::FIELD_PRIVILEGED => $this->getPrivileged()
        ];
        if (!is_null($this->getLanguage())) {
            $properties[static::FIELD_LANGUAGE] = $this->getLanguage();
        }
        if (!is_null($this->getRoles())) {
            $properties[static::FIELD_ROLES] = $this->getRoles();
        }
        return $properties;
    }

    /**
     * @param string $login
     */
    public function setLogin($login)
    {
        $this->_login = $login;
    }
    
    /**
     * @return string
     */
    public function getLogin()
    {
        return $this->_login;
    }
    
    /**
     * @param string $email
     */
    public function setEmail($email)
    {
        $this->_email = $email;
    }

    /**
     * @return string
     */
    public function getEmail()
    {
        return $this->_email;
    }

    /**
     * @param string $firstName
     */
    public function setFirstName($firstName)
    {
        $this->_firstName = $firstName;
    }

    /**
     * @return string
     */
    public function getFirstName()
    {
        return $this->_firstName;
    }

    /**
     * @param string $lastName
     */
    public function setLastName($lastName)
    {
        $this->_lastName = $lastName;
    }

    /**
     * @return string
     */
    public function getLastName()
    {
        return $this->_lastName;
    }

    /**
     * @param Enum_NotificationsBackup $notificationsBackup
     */
    public function setNotificationsBackup(Enum_NotificationsBackup $notificationsBackup)
    {
        $this->_notificationsBackup = $notificationsBackup;
    }

    /**
     * @return Enum_NotificationsBackup
     */
    public function getNotificationsBackup()
    {
        return $this->_notificationsBackup;
    }

    /**
     * @param int $notificationsManagement
     */
    public function setNotificationsManagement($notificationsManagement)
    {
        $this->_notificationsManagement = $notificationsManagement;
    }

    /**
     * @return int
     */
    public function getNotificationsManagement()
    {
        return $this->_notificationsManagement;
    }

    /**
     * @param Service_MspApiClient_Request_Struct_UserPrivileges $privileges
     */
    public function setPrivileges(Service_MspApiClient_Request_Struct_UserPrivileges $privileges)
    {
        $this->_privileges = $privileges;
    }

    /**
     * @return Service_MspApiClient_Request_Struct_UserPrivileges
     */
    public function getPrivileges()
    {
        return $this->_privileges;
    }

    /**
     * @return int
     */
    public function getPrivileged()
    {
        return $this->_privileged;
    }

    /**
     * @param int $privileged
     */
    public function setPrivileged($privileged)
    {
        $this->_privileged = $privileged;
    }

    /**
     * @return int
     */
    public function getRoles()
    {
        return $this->_roles;
    }

    /**
     * @param int $roles
     */
    public function setRoles($roles)
    {
        $this->_roles = $roles;
    }

    /**
     * @return string
     */
    public function getLanguage()
    {
        return $this->_language;
    }

    /**
     * @param string $language
     */
    public function setLanguage($language)
    {
        $this->_language = $language;
    }
}
