<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_MspApiClient_Response_GetUsers extends Service_BaseClient_Response_Base
{
    const PROPERTY_ITEMS = 'items';
    const PROPERTY_EMAIL = 'email';
    const PROPERTY_FIRSTNAME = 'firstname';
    const PROPERTY_LASTNAME = 'lastname';
    const PROPERTY_STATUS = 'status';
    const PROPERTY_ID = 'id';
    const PROPERTY_USAGE = 'usage';
    const PROPERTY_VM_COUNT = 'vm_count';
    const PROPERTY_SERVER_COUNT = 'server_count';
    const PROPERTY_WORKSTATION_COUNT = 'workstation_count';
    const PROPERTY_MOBILE_COUNT = 'mobile_count';
    const PROPERTY_MAILBOX_COUNT = 'mailbox_count';
    const PROPERTY_WEBSITE_COUNT = 'website_count';
    const PROPERTY_STORAGE_SIZE = 'storage_size';
    const PROPERTY_LOCAL_STORAGE_SIZE = 'local_storage_size';
    const PROPERTY_PRIVILEGES = 'privileges';
    const PROPERTY_PRIVILEGED = 'privileged';
    const PROPERTY_LOGIN = 'login';

    /** @var Service_MspApiClient_Response_Struct_FactoryInterface */
    private $_factory;

    /**
     * @param string|null $responseBody
     * @param Service_MspApiClient_Response_Struct_FactoryInterface $factory
     */
    public function __construct($responseBody = NULL, Service_MspApiClient_Response_Struct_FactoryInterface $factory = NULL)
    {
        $this->_factory = is_null($factory) ? new Service_MspApiClient_Response_Struct_Factory() : $factory;
        parent::__construct($responseBody);
    }

    /**
     * @param array $responseData
     * @return array of Service_MspApiClient_Response_Struct_UserItem
     */
    protected function _init(array $responseData)
    {
        $users = array();
        $items = $this->_getProperty(self::PROPERTY_ITEMS, $responseData);
        foreach ($items as $itemData)
        {
            $userItem = $this->_factory->getUserItem();
            $userItem->setEmail($this->_getProperty(self::PROPERTY_EMAIL, $itemData));
            $userItem->setFirstName($this->_getProperty(self::PROPERTY_FIRSTNAME, $itemData));
            $userItem->setLastName($this->_getProperty(self::PROPERTY_LASTNAME, $itemData));
            $userItem->setStatus($this->_getProperty(self::PROPERTY_STATUS, $itemData));
            $userItem->setId($this->_getProperty(self::PROPERTY_ID, $itemData));
            $userItem->setLogin($this->_getProperty(self::PROPERTY_LOGIN, $itemData));
            $userItem->setPrivileged($this->_getProperty(self::PROPERTY_PRIVILEGED, $itemData));

            $usageData = $this->_getProperty(self::PROPERTY_USAGE, $itemData);
            $usage = $userItem->getUsage();
            $usage->setVmCount($this->_getProperty(self::PROPERTY_VM_COUNT, $usageData));
            $usage->setServerCount($this->_getProperty(self::PROPERTY_SERVER_COUNT, $usageData));
            $usage->setWorkstationCount($this->_getProperty(self::PROPERTY_WORKSTATION_COUNT, $usageData));
            $usage->setMobileCount($this->_getProperty(self::PROPERTY_MOBILE_COUNT, $usageData, false));
            $usage->setMailboxCount($this->_getProperty(self::PROPERTY_MAILBOX_COUNT, $usageData, false));
            $usage->setWebsiteCount($this->_getProperty(self::PROPERTY_WEBSITE_COUNT, $usageData, false));
            $usage->setStorageSize($this->_getProperty(self::PROPERTY_STORAGE_SIZE, $usageData));
            $usage->setLocalStorageSize($this->_getProperty(self::PROPERTY_LOCAL_STORAGE_SIZE, $usageData, false));

            $privilegesData = $this->_getProperty(self::PROPERTY_PRIVILEGES, $itemData);
            $privileges = $userItem->getPrivileges();
            $privileges->setVmCount($this->_getProperty(self::PROPERTY_VM_COUNT, $privilegesData));
            $privileges->setServerCount($this->_getProperty(self::PROPERTY_SERVER_COUNT, $privilegesData));
            $privileges->setWorkstationCount($this->_getProperty(self::PROPERTY_WORKSTATION_COUNT, $privilegesData));
            $privileges->setMobileCount($this->_getProperty(self::PROPERTY_MOBILE_COUNT, $privilegesData, false));
            $privileges->setMailboxCount($this->_getProperty(self::PROPERTY_MAILBOX_COUNT, $privilegesData, false));
            $privileges->setWebsiteCount($this->_getProperty(self::PROPERTY_WEBSITE_COUNT, $privilegesData, false));
            $privileges->setStorageSize($this->_getProperty(self::PROPERTY_STORAGE_SIZE, $privilegesData));
            $privileges->setLocalStorageSize($this->_getProperty(self::PROPERTY_LOCAL_STORAGE_SIZE, $privilegesData, false));

            $users[] = $userItem;
        }
        return $users;
    }
}