<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

interface Service_OpenApi_ApiInterface
{
    const ROOT_ACCOUNT_ID = 1;

    const ACCOUNT_TYPE_PROVIDER = 'A';
    const ACCOUNT_TYPE_RESELLER = 'R';
    const ACCOUNT_TYPE_CUSTOMER = 'C';

    /**
     * @param int $accountId
     * @return Service_OpenApi_Response_AccountInfo
     */
    public function getAccountInfo($accountId);

    /**
     * @param int $accountId
     * @return array
     */
    public function getAccountSubscription($accountId);

    /**
     * @param int $subscriptionId
     * @param bool $getResources
     * @return SubscriptionInfoInterface
     */
    public function getSubscription($subscriptionId, $getResources = true);

    /**
     * @param string $apsApplicationId
     * @throws Exception
     * @return string
     */
    public function apsGetApplicationId($apsApplicationId);

    /**
     * @param string $resClassName
     * @param string $name
     * @param string $description
     * @param array $attributes
     * @param array $parameters
     * @return string
     */
    public function addResourceType($resClassName, $name, $description = null, array $attributes = null, array $parameters = null);

    /**
     * @param string $name
     * @param int $applicationId
     * @param string $resourceId
     * @param string $description
     * @param array $attributes
     * @return string
     */
    public function addRTServiceLink($name, $applicationId, $resourceId, $description = null, array $attributes = null);

    /**
     * @param int $resourceTypeId
     */
    public function removeResourceType($resourceTypeId);

    /**
     * @param int $providerId
     * @return array of customer IDs
     */
    public function getCustomerIds($providerId);

     /**
     * @param int $providerId
     * @return array of templates
     */
    public function getServiceTemplateList($providerId);

    /**
     * @param int $accountId
     * @param int $subscriptionId
     * @return array aaccount token
     */
    public function getAccountToken($accountId, $subscriptionId);

    /**
     * @param int $applicationInstanceId
     * @return array application instance token
     */
    public function getApplicationInstanceToken($applicationInstanceId);

    /**
     * @param int $applicationInstanceId
     * @return array
     */
    public function getApplicationInstanceSettings($applicationInstanceId);

    /**
     * @param int $applicationId
     * @return array
     */
    public function getApplicationInstances($applicationId);

    /**
     * @param int $subscriptionId
     * @return array
     */
    public function getSubscriptionApplicationInstances($subscriptionId);

    /**
     * @param $applicationInstanceId
     * @param $serviceId
     * @return array
     */
    public function getServiceInstances($applicationInstanceId, $serviceId);

    /**
     * @param $applicationInstanceId
     * @param $serviceId
     * @return array
     */
    public function getApplicationInstanceServiceResourceTypes($applicationInstanceId, $serviceId = null);

    /**
     * @param $applicationId
     * @param $endpointUrl
     * @param int $subscriptionId
     * @param int $rtId
     * @return mixed
     */
    public function provideApplicationInstance($applicationId, $endpointUrl, $subscriptionId = 0, $rtId = 0);

    /**
     * @param $name
     * @param $resources
     * @param null $category
     * @param int $ownerId
     * @param bool $autoProvisioning
     * @return array
     */
    public function addServiceTemplate($name, $resources, $category = null, $ownerId = 1, $autoProvisioning = false);

    /**
     * @param $serviceTemplateId
     * @param $limits
     * @return array
     */
    public function setServiceTemplateLimits($serviceTemplateId, $limits);

    /**
     * @param $serviceTemplateId
     * @return mixed
     */
    public function activateServiceTemplate($serviceTemplateId);


    /**
     * @param $applicationName
     * @param $applicationResourceId
     * @param $applicationId
     * @return string
     */
    public function addApplicationInstance($applicationName, $applicationResourceId, $applicationId);

    /**
     * @param $name
     * @param $resources
     * @param null $category
     * @return mixed
     */
    public function addActivatedServiceTemplate($name, $resources, $category = null);

    /**
     * @param $applicationId
     * @param $name
     * @param $type
     * @param $autoprovide
     * @return string
     */
    public function addServiceType($applicationId, $name, $type, $autoprovide);

    /**
     * @param $resourceId
     * @param $unit
     * @return array
     */
    public function changeMeasurementUnit($resourceId, $unit);

    /**
     * @param $name
     * @return array
     */
    public function addResourceCategory($name);

    /**
     * @param $id
     * @param $storeTemplateId
     * @param $title
     * @param $storeScreenParams
     * @param $salesCategories
     * @return array
     */
    public function addStoreScreen($id, $storeTemplateId, $title, $storeScreenParams, $salesCategories);

    /**
     * @param $id
     * @param $name
     * @param $orderNumber
     * @param $iconPath
     * @param $storeScreens
     * @return array
     */
    public function addPurchaseScenario($id, $name, $orderNumber, $iconPath, $storeScreens);

    /**
     * @param $name
     * @param $serviceTemplateId
     * @param $shortDescription
     * @param $longDescription
     * @param $salesCategory
     * @param $imageFile
     * @param $published
     * @param $notificationTemplate
     * @param $billPeriod
     * @param $billPeriodType
     * @param $chargingType
     * @param $autoRenewType
     * @param $renewOrderInterval
     * @param $resourceRates
     * @param $planPeriods
     * @param $listingPriority
     * @return array
     */
    public function addServicePlan(
        $name,
        $serviceTemplateId,
        $shortDescription,
        $longDescription,
        $salesCategory,
        $imageFile,
        $published,
        $notificationTemplate,
        $billPeriod,
        $billPeriodType,
        $chargingType,
        $autoRenewType,
        $renewOrderInterval,
        $resourceRates,
        $planPeriods,
        $listingPriority
    );

    /**
     * @param $fromPlanId
     * @param $toPlanId
     * @return array
     */
    public function addPlanUpgrade($fromPlanId, $toPlanId);
}
