<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_PbaApi_Api implements Service_PbaApi_ApiInterface
{
    const PEMGATE = 'PEMGATE';

    const SUBSCRIPTION_STATUS_TRIAL = 15;
    const SUBSCRIPTION_STATUS_ACTIVE = 30;
    const SUBSCRIPTION_STATUS_EXPIRED = 50;

    const SUBSCRIPTION_SERVICE_STATUS_STOPPED = 30;
    const SUBSCRIPTION_SERVICE_STATUS_RUNNING = 50;

    const DEFAULT_REASON_ID = 1;
    const DEFAULT_EXTEND_TRIAL_DESCRIPTION = 'ExtendTrial';


    /** @var Service_PbaApi_ClientInterface */
    private $_client;

    /**
     * @param Service_PbaApi_ClientInterface $client
     */
    public function __construct(Service_PbaApi_ClientInterface $client)
    {
        $this->_client = $client;
    }

    public function startTransaction()
    {
        $this->_client->startTransaction();
    }

    public function commitTransaction()
    {
        $this->_client->commitTransaction();
    }

    public function rollbackTransaction()
    {
        $this->_client->rollbackTransaction();
    }

    /**
     * @param int $accountId
     * @param int $discountId
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function addAccountInDiscount($accountId, $discountId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_AddAccountInDiscount($accountId, $discountId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_ADD_ACCOUNT_DISCOUNT_MESSAGE, $discountId, $accountId),
                Service_PbaApi_Api_Exception::CAN_NOT_ADD_ACCOUNT_DISCOUNT_CODE,
                $e
            );
        }
    }

    /**
     * @param int $accountId
     * @param int $discountId
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function moveToArchiveAccountInDiscount($accountId, $discountId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_MoveToArchiveAccountInDiscount($accountId, $discountId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_MOVE_TO_ARCHIVE_ACCOUNT_DISCOUNT_MESSAGE, $discountId, $accountId),
                Service_PbaApi_Api_Exception::CAN_NOT_MOVE_TO_ARCHIVE_ACCOUNT_DISCOUNT_CODE,
                $e
            );
        }
    }

    /**
     * @param int $subscriptionId
     * @param string $parameterId
     * @return mixed
     * @throws Exception
     */
    public function getSubscriptionParameterValue($subscriptionId, $parameterId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetSubscriptionParameterValue($subscriptionId, $parameterId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_RECEIVE_PARAMETER_MESSAGE, $parameterId, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_RECEIVE_PARAMETER_CODE,
                $e
            );
        }
    }

    /**
     * @param int $objectType
     * @param int $objectId
     * @return Service_PbaApi_Api_Response_Struct_ObjectAttributesListItemInterface[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getObjectAttributesList($objectType, $objectId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetObjectAttributesList($objectType, $objectId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_RECEIVE_LIST_OF_ATTRIBUTES_MESSAGE, $objectId, $objectType),
                Service_PbaApi_Api_Exception::CAN_NOT_RECEIVE_LIST_OF_ATTRIBUTES_CODE,
                $e
            );
        }
    }

    /**
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_AttributeTypeListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getAttributesTypeList($sortNo = 1)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetAttributesTypeList($sortNo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                Service_PbaApi_Api_Exception::CAN_NOT_GET_ATTRIBUTES_TYPE_LIST_MESSAGE,
                Service_PbaApi_Api_Exception::CAN_NOT_GET_ATTRIBUTES_TYPE_LIST_CODE,
                $e
            );
        }
    }

    /**
     * Method returns the list of published service plans of specified plan category.
     *
     * @param int $accountID account ID of plan category owner
     * @param string $stType name of service gate service plans are provisioned through: PEMGATE, DOMAINGATE, DUMMYGATE, etc.
     * @param int $planCategoryID plan category ID
     * @param int $sortNo parameter defines how output data is sorted:
     *    "1" - the output is sorted by plan ID in ascending order,
     *    "2" - by plan name in ascending order, etc. Negative value makes output sorted in descending order.
     * @return Service_PbaApi_Api_Response_Struct_AvailablePlanListItemInterface[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getAvailablePlanList($accountID, $stType, $planCategoryID, $sortNo = 1)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetAvailablePlanList($accountID, $stType, $planCategoryID, $sortNo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_AVAILABLE_PLAN_LIST_MESSAGE, $accountID),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_AVAILABLE_PLAN_LIST_CODE,
                $e
            );
        }
    }

    /**
     * @param int $planID
     * @return Service_PbaApi_Api_Response_Struct_PlanInterface
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlan($planID)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetPlan($planID);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_MESSAGE, $planID),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_CODE,
                $e
            );
        }
    }

    /**
     * @param int $planId
     * @return Service_PbaApi_Api_Response_Struct_PlanDetails
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanDetails($planId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetPlanDetails($planId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_DETAILS_MESSAGE, $planId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_DETAILS_CODE,
                $e
            );
        }
    }

    /**
     * Method returns the list of plan categories for specified vendor account.
     *
     * @param int $accountId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_PlanCategoryListItemInterface[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanCategoryList($accountId, $sortNo = 1)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetPlanCategoryList($accountId, $sortNo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_CATEGORIES_MESSAGE, $accountId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_CATEGORIES_CODE,
                $e
            );
        }
    }

    /**
     * @param int $accountId
     * @return Service_PbaApi_Api_Response_Struct_PlanInterface[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getServicePlans($accountId)
    {
        $plans = array();
        $planCategories = $this->getPlanCategoryList($accountId);
        foreach ($planCategories as $category) {
            $availablePlans = $this->getAvailablePlanList($accountId, self::PEMGATE, $category->getPlanCategoryId());
            foreach ($availablePlans as $availablePlan) {
                $planId = $availablePlan->getPlanId();
                $plan = $this->getPlan($planId);
                $plans[$planId] = $plan;
            }
        }
        return $plans;
    }

    /**
     * @param int $accountId
     * @param int $planId
     * @return Service_PbaApi_Api_Response_Struct_PlanInterface
     * @throws Service_PbaApi_Api_Exception
     */
    public function getServicePlan($accountId, $planId)
    {
        $planCategories = $this->getPlanCategoryList($accountId);
        foreach ($planCategories as $category) {
            $availablePlans = $this->getAvailablePlanList($accountId, self::PEMGATE, $category->getPlanCategoryId());
            foreach ($availablePlans as $availablePlan) {
                $availablePlanId = $availablePlan->getPlanId();
                if ($availablePlanId == $planId) {
                    return $this->getPlan($planId);
                }
            }
        }
        return NULL;
    }

    /**
     * @param int $planId
     * @param int $serviceTemplateID
     * @param Service_PbaApi_Api_Request_Struct_ServiceTemplateParameter[] $parameters
     * @return Service_PbaApi_Api_Response_Struct_GetAndValidatePlanParams[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getAndValidatePlanParameters($planId, $serviceTemplateID, array $parameters = array())
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetAndValidatePlanParams($planId, $serviceTemplateID, $parameters);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_AND_VALIDATE_PLAN_PARAMETERS_MESSAGE, $planId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_AND_VALIDATE_PLAN_PARAMETERS_CODE,
                $e
            );
        }
    }

    /**
     * This method returns the list of subscriptions belonging to a particular customer.
     *
     * @param int $accountId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_CustomerSubscriptionListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getCustomerSubscriptionList($accountId, $sortNo = 1)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetCustomerSubscriptionList($accountId, $sortNo);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_CUSTOMER_SUBSCRIPTION_LIST_MESSAGE, $accountId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_CUSTOMER_SUBSCRIPTION_LIST_CODE,
                $e
            );
        }
    }

    /**
     * @param integer $subscriptionId
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionDetailsInterface
     * @throws Service_PbaApi_Api_Exception
     */
    public function getSubscriptionDetails($subscriptionId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetSubscriptionDetails($subscriptionId);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_SUBSCRIPTION_DETAILS_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_SUBSCRIPTION_DETAILS_CODE,
                $e
            );
        }
    }

    /**
     * @param integer $subscriptionId
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionDetailsEx
     * @throws Service_PbaApi_Api_Exception
     */
    public function getSubscriptionDetailsEx($subscriptionId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetSubscriptionDetailsEx($subscriptionId);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_SUBSCRIPTION_DETAILS_EX_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_SUBSCRIPTION_DETAILS_EX_CODE,
                $e
            );
        }
    }

    /**
     * @param $orderId
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function getSubscriptionsListByOrder($orderId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetSubscriptionsListByOrder($orderId);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_SUBSCRIPTION_LIST_BY_ORDER_MESSAGE, $orderId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_SUBSCRIPTION_LIST_BY_ORDER_CODE,
                $e
            );
        }
    }

    /**
     * @param int $subscriptionId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionResourcesList[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getSubscriptionResourcesList($subscriptionId, $sortNo = 0)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetSubscriptionResourcesList($subscriptionId, $sortNo);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_SUBSCRIPTION_RESOURCES_LIST_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_SUBSCRIPTION_RESOURCES_LIST_CODE,
                $e
            );
        }
    }

    /**
     * @param int $subscriptionId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_GetSubscriptionSalesOrdersListInterface[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getSubscriptionSalesOrdersList($subscriptionId, $sortNo = 0)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetSubscriptionSalesOrdersList($subscriptionId, $sortNo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_SUBSCRIPTION_SALES_ORDERS_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_SUBSCRIPTION_SALES_ORDERS_CODE,
                $e
            );
        }
    }

    /**
     * @param int $subscriptionId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionAvailablePlanListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getSubscriptionAvailablePlanList($subscriptionId, $sortNo = 0)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetSubscriptionAvailablePlanList($subscriptionId, $sortNo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_SUBSCRIPTION_AVAILABLE_PLAN_LIST_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_SUBSCRIPTION_AVAILABLE_PLAN_LIST_CODE,
                $e
            );
        }
    }

    /**
     * @param integer $subscriptionId
     * @param integer $status
     * @param integer $serviceStatus
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionDetailsInterface
     * @throws Service_PbaApi_Api_Exception
     */
    public function updateSubscriptionStatus($subscriptionId, $status, $serviceStatus)
    {
        try {
            $request = new Service_PbaApi_Api_Request_UpdateSubscriptionStatus($subscriptionId, $status, $serviceStatus);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_UPDATE_SUBSCRIPTION_STATUS_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_UPDATE_SUBSCRIPTION_STATUS_CODE,
                $e
            );
        }
    }

    /**
     * @param integer $subscriptionId
     * @param int $expirationDate
     * @param string $reason
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionDetailsInterface
     * @throws Service_PbaApi_Api_Exception
     */
    public function extendTrialPeriod($subscriptionId, $expirationDate, $reason)
    {
        $subscriptionDetails = $this->getSubscriptionDetails($subscriptionId);
        $subscriptionStatus = $subscriptionDetails->getStatus();
        $subscriptionServiceStatus = $subscriptionDetails->getServStatus();

        try {
            switch ($subscriptionStatus) {
                case self::SUBSCRIPTION_STATUS_TRIAL:
                    $this->updateSubscriptionStatus(
                        $subscriptionId,
                        self::SUBSCRIPTION_STATUS_ACTIVE,
                        self::SUBSCRIPTION_SERVICE_STATUS_RUNNING
                    );
                    $this->changeExpiration(
                        $subscriptionId,
                        $expirationDate,
                        $reason
                    );
                    $response = $this->updateSubscriptionStatus(
                        $subscriptionId,
                        self::SUBSCRIPTION_STATUS_TRIAL,
                        self::SUBSCRIPTION_SERVICE_STATUS_RUNNING
                    );
                    break;
                case self::SUBSCRIPTION_STATUS_EXPIRED:
                    $this->changeExpiration($subscriptionId, $expirationDate, $reason);
                    $this->updateSubscriptionStatus(
                        $subscriptionId,
                        self::SUBSCRIPTION_STATUS_TRIAL,
                        self::SUBSCRIPTION_SERVICE_STATUS_STOPPED
                    );
                    $response = $this->startSubscription($subscriptionId);
                    break;
                default:
                    throw new Service_PbaApi_Api_Exception(
                        Service_PbaApi_Api_Exception::CAN_NOT_UPDATE_SUBSCRIPTION_WITH_INCORRECT_STATUS_MESSAGE,
                        Service_PbaApi_Api_Exception::CAN_NOT_UPDATE_SUBSCRIPTION_WITH_INCORRECT_STATUS_CODE
                    );
                    break;
            }
            return $response;
        } catch (Exception $e) {
            $this->updateSubscriptionStatus(
                $subscriptionId,
                $subscriptionStatus,
                $subscriptionServiceStatus
            );

            Log::getInstance()->event($e->getMessage());
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_UPDATE_SUBSCRIPTION_STATUS_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_UPDATE_SUBSCRIPTION_STATUS_CODE,
                $e
            );
        }
    }

    /**
     * @param integer $subscriptionId
     * @param integer $newExpData
     * @param string $comment
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionDetailsInterface
     * @throws Service_PbaApi_Api_Exception
     */
    public function changeExpiration($subscriptionId, $newExpData, $comment)
    {
        try {
            $request = new Service_PbaApi_Api_Request_ChangeExpiration($subscriptionId, $newExpData, $comment);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_CHANGE_EXPIRATION_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_CHANGE_EXPIRATION_CODE,
                $e
            );
        }
    }

    /**
     * @param $subscriptionId
     * @param $usePayTool
     * @param $useDefaultPayTool
     * @param $userId
     * @param $fromIp
     * @param Service_PbaApi_Api_Request_Struct_ResourceUpgradeOrder[] $resourceUpgradeOrders
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function placeResourceUpgradeOrder($subscriptionId, $usePayTool, $useDefaultPayTool, $userId, $fromIp, $resourceUpgradeOrders)
    {
        try {
            $request = new Service_PbaApi_Api_Request_PlaceResourceUpgradeOrder(
                $subscriptionId,
                $usePayTool,
                $useDefaultPayTool,
                $userId,
                $fromIp,
                $resourceUpgradeOrders
            );
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_PLACE_RESOURCE_UPGRADE_ORDER_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_PLACE_RESOURCE_UPGRADE_ORDER_CODE,
                $e
            );
        }
    }

    /**
     * @param int $orderId
     * @return Service_PbaApi_Api_Response_Struct_GetOrder
     * @throws Service_PbaApi_Api_Exception
     */
    public function getOrder($orderId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetOrder($orderId);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_ORDER_MESSAGE, $orderId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_ORDER_CODE,
                $e
            );
        }
    }

    /**
     * @param int $orderId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_GetOrderDetails[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getOrderDetails($orderId, $sortNo = 0)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetOrderDetails($orderId, $sortNo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_ORDER_DETAILS_MESSAGE, $orderId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_ORDER_DETAILS_CODE,
                $e
            );
        }
    }

    /**
     * @param int $orderId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_OrderProvisioningItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getOrderProvisioningItemsList($orderId, $sortNo = 0)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetOrderProvisioningItemsList($orderId, $sortNo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_ORDER_PROVISIONING_ITEMS_MESSAGE, $orderId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_ORDER_PROVISIONING_ITEMS_CODE,
                $e
            );
        }
    }

    /**
     * @param int $orderId
     * @param string $status
     * @param string $signature
     * @return Service_PbaApi_Api_Response_Struct_Status
     * @throws Service_PbaApi_Api_Exception
     */
    public function changeOrderStatus($orderId, $status, $signature)
    {
        try {
            $request = new Service_PbaApi_Api_Request_ChangeOrderStatus($orderId, $status, $signature);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_CHANGE_ORDER_STATUS_MESSAGE, $orderId),
                Service_PbaApi_Api_Exception::CAN_NOT_CHANGE_ORDER_STATUS_CODE,
                $e
            );
        }
    }

    /**
     * @param $attributeId
     * @param $attributeValue
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_GetAccountListByCustomAttribute[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getAccountListByCustomAttribute($attributeId, $attributeValue, $sortNo = 1)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetAccountListByCustomAttribute(
                $attributeId,
                $attributeValue,
                $sortNo
            );

            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_ACCOUNT_LIST_BY_CUSTOM_ATTRIBUTE_MESSAGE, $attributeId, $attributeValue),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_ACCOUNT_LIST_CODE,
                $e
            );
        }
    }

    /**
     * @param $vendorAccountID
     * @return Service_PbaApi_Api_Response_Struct_GetVendorCurrency
     * @throws Service_PbaApi_Api_Exception
     */
    public function getVendorCurrency($vendorAccountID)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetVendorCurrency($vendorAccountID);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_VENDOR_CURRENCY_MESSAGE, $vendorAccountID),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_VENDOR_CURRENCY_CODE,
                $e
            );
        }
    }

    /**
     * @param $planId
     * @param int $sort
     * @return Service_PbaApi_Api_Response_Struct_PlanPeriodListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanPeriodList($planId, $sort = 1)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetPlanPeriodList($planId, $sort);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_PERIOD_LIST_MESSAGE, $planId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_PERIOD_LIST_CODE,
                $e
            );
        }
    }

    /**
     * @param $planId
     * @param int $sort
     * @return Service_PbaApi_Api_Response_Struct_PlanRateListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanRateList($planId, $sort = 1)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetPlanRateList($planId, $sort);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_RATE_LIST_MESSAGE, $planId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_RATE_LIST_CODE,
                $e
            );
        }
    }

    /**
     * @param $planId
     * @param int $sort
     * @return Service_PbaApi_Api_Response_Struct_PlanRateListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanRateListFull($planId, $sort = 1)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetPlanRateListFull($planId, $sort);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_RATE_LIST_FULL_MESSAGE, $planId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_RATE_LIST_FULL_CODE,
                $e
            );
        }
    }

    /**
     * @param $planPeriodId
     * @param int $sort
     * @return Service_PbaApi_Api_Response_Struct_PlanRateListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanRateListForPeriod($planPeriodId, $sort = 1)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetPlanRateListForPeriod($planPeriodId, $sort);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_RATE_LIST_FOR_PERIOD_MESSAGE, $planPeriodId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_RATE_LIST_FOR_PERIOD_CODE,
                $e
            );
        }
    }

    /**
     * @param $planId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_PlanRate[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanRates($planId, $sortNo = 1)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetPlanRates($planId, $sortNo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_RATES_FOR_PLAN_MESSAGE, $planId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_RATES_FOR_PLAN_CODE,
                $e
            );
        }
    }

    /**
     * @param $vendorAccountId
     * @param $parameters
     * @param $language
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function placeOrderAndAuthorize($vendorAccountId, $parameters, $language = null)
    {
        try {
            $request = new Service_PbaApi_Api_Request_PlaceOrderAndAuthorize($vendorAccountId, $parameters, $language);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_PLACE_ORDER_AND_AUTHORIZE_MESSAGE),
                Service_PbaApi_Api_Exception::CAN_NOT_PLACE_ORDER_AND_AUTHORIZE_CODE,
                $e
            );
        }
    }

    /**
     * @param int $subscriptionId subscription ID;
     * @param int $planId ID of destination plan for upgrades / downgrades from one plan to another;
     * ID of current subscription service plan for switching between subscription periods of same plan;
     * @param int $periodId ID of subscription period of destination service plan or ID of another period of same service plan;
     * @param int $upgradeStartType defines when new subscription period will be calculated from:
     * "10" - from the start date of current subscription period,
     * "20" - from the date of upgrade;
     * @param int $applyPromo this parameter is optional;
     * if it is sent with the value 1 then a promo code already tired to the subscription is applied.
     * (If it is sent with 0 or empty value then a promo code is not applied)
     *
     * @return Service_PbaApi_Api_Response_Struct_PlacePlanPeriodSwitchOrder
     * @throws Service_PbaApi_Api_Exception
     */
    public function placePlanPeriodSwitchOrder($subscriptionId, $planId, $periodId, $upgradeStartType, $applyPromo = 0)
    {
        try {
            $request = new Service_PbaApi_Api_Request_PlacePlanPeriodSwitchOrder($subscriptionId, $planId, $periodId, $upgradeStartType, $applyPromo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_PLACE_PLAN_PERIOD_SWITCH_ORDER_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_PLACE_PLAN_PERIOD_SWITCH_ORDER_CODE,
                $e
            );
        }
    }

    /**
     * @param int $subscriptionId ID of subscription to be cancelled.
     * @param int $cancelType cancellation type:
     * 10 – customer is refunded. Corresponds to the behavior of subscription, for which all checkboxes of
     * Subscription Cancellation Options (Control Panel) are selected. The refund, overuse and cancellation fees are taken.
     * 20 – customer account is credited (credit memo is created). The refund, overuse and cancellation fees are taken.
     * 30 – customer is not refunded / credited. Corresponds to the behavior of subscription, for which all checkboxes of
     * Subscription Cancellation Options (Control Panel) are deselected. No refund, overuse or cancellation fees are taken.
     * @param int $reasonId ID of system reason code to be stated as reason of subscription cancellation.
     * @param string $description optional comment on the reason subscription is cancelled.
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function placeSubscriptionCancellationOrder($subscriptionId, $cancelType, $reasonId, $description = '')
    {
        try {
            $request = new Service_PbaApi_Api_Request_PlaceSubscriptionCancellationOrder($subscriptionId, $cancelType, $reasonId, $description);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_PLACE_SUBSCRIPTION_CANCELLATION_ORDER_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_PLACE_SUBSCRIPTION_CANCELLATION_ORDER_CODE,
                $e
            );
        }
    }

    /**
     * @param Service_PbaApi_Api_Request_Struct_OrderAttributes $orderAttributes
     * @param Service_PbaApi_Api_Request_Struct_OrderDetailAttributes[] $orderDetails
     * @param Service_PbaApi_Api_Request_Struct_TaxDetailAttributes[] $taxDetails
     * @return int Order ID
     * @throws Service_PbaApi_Api_Exception
     */
    public function createManualOrder(Service_PbaApi_Api_Request_Struct_OrderAttributes $orderAttributes, array $orderDetails, array $taxDetails = array())
    {
        try {
            $request = new Service_PbaApi_Api_Request_CreateManualOrder($orderAttributes, $orderDetails, $taxDetails);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_CREATE_MANUAL_ORDER_MESSAGE),
                Service_PbaApi_Api_Exception::CAN_NOT_CREATE_MANUAL_ORDER_CODE,
                $e
            );
        }
    }

    /**
     * @param $attributeType
     * @param $objId
     * @param array Service_PbaApi_Api_Request_Struct_UpdateObjAttrItem[] $attributeToUpdate
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function updateObjAttrList($attributeType, $objId, array $attributeToUpdate)
    {
        try {
            $request = new Service_PbaApi_Api_Request_UpdateObjAttrList($attributeType, $objId, $attributeToUpdate);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_UPDATE_OBJ_ATTR_LIST_MESSAGE),
                Service_PbaApi_Api_Exception::CAN_NOT_UPDATE_OBJ_ATTR_LIST_CODE,
                $e
            );
        }
    }

    /**
     * Don't use if not sure
     * Returns some information about the resource
     * @param $resourceId
     * @return Service_PbaApi_Api_Response_Struct_GetResource
     * @throws Service_PbaApi_Api_Exception
     */
    public function getResource($resourceId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetResource($resourceId);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_RESOURCE_MESSAGE, $resourceId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_RESOURCE_CODE,
                $e
            );
        }
    }

    /**
     * Don't use if not sure
     * Returns some information about the included resource
     * @param $resourceId
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function getIncludedResources($resourceId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetIncludedResources($resourceId);
            $response = $this->_client->execute($request);

            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_INCLUDED_RESOURCES_MESSAGE, $resourceId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_INCLUDED_RESOURCES_CODE,
                $e
            );
        }
    }

    /**
     * @param $accountId
     * @return Service_PbaApi_Api_Response_Struct_Account
     * @throws Service_PbaApi_Api_Exception
     */
    public function getAccount($accountId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetAccount($accountId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_ACCOUNT_MESSAGE, $accountId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_ACCOUNT_CODE,
                $e
            );
        }
    }

    /**
     * @param int $discountId
     * @return Service_PbaApi_Api_Response_Struct_Discount
     * @throws Service_PbaApi_Api_Exception
     */
    public function getDiscountById($discountId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetDiscountById($discountId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_DISCOUNT_BY_ID_MESSAGE, $discountId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_DISCOUNT_BY_ID,
                $e
            );
        }
    }

    /**
     * @param integer $subscriptionId
     * @param string $parameterId
     * @param string $parameterValue
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function updateSubscriptionParameterValue($subscriptionId, $parameterId, $parameterValue)
    {
        try {
            $request = new Service_PbaApi_Api_Request_UpdateSubscriptionParameterValue($subscriptionId, $parameterId, $parameterValue);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_UPDATE_SUBSCRIPTION_PARAMETER_BY_ID_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_UPDATE_SUBSCRIPTION_PARAMETER_BY_ID_CODE,
                $e
            );
        }
    }

    /**
     * @param $subscriptionId
     * @param $reasonId
     * @param $description
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function startSubscription($subscriptionId, $reasonId = self::DEFAULT_REASON_ID, $description = self::DEFAULT_EXTEND_TRIAL_DESCRIPTION)
    {
        try {
            $request = new Service_PbaApi_Api_Request_StartSubscription($subscriptionId, $reasonId, $description);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_START_SUBSCRIPTION_BY_ID_MESSAGE, $subscriptionId),
                Service_PbaApi_Api_Exception::CAN_NOT_START_SUBSCRIPTION_BY_ID,
                $e
            );
        }
    }

    /**
     * @param integer $splitDate
     * @param integer $vendorId
     * @return Service_PbaApi_Api_Response_Struct_Status
     * @throws Service_PbaApi_Api_Exception
     */
    public function splitDeliverLinesForVendorByDate($splitDate, $vendorId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_SplitDeliverLinesForVendorByDate($splitDate, $vendorId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_SPLIT_DELIVER_LINES_FOR_VENDOR_BY_DATE_MESSAGE, $splitDate, $vendorId),
                Service_PbaApi_Api_Exception::CAN_NOT_SPLIT_DELIVER_LINES_FOR_VENDOR_BY_DATE_CODE,
                $e
            );
        }
    }

    /**
     * @param int $fromDate
     * @param int $toDate
     * @param int $vendorId
     * @return Service_PbaApi_Api_Response_Struct_Status
     * @throws Service_PbaApi_Api_Exception
     */
    public function mergeDeliverLinesForPeriodByVendor($fromDate, $toDate, $vendorId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_MergeDeliverLinesForPeriodByVendor($fromDate, $toDate, $vendorId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_MERGE_DELIVER_LINES_FOR_PERIOD_BY_VENDOR_MESSAGE, $fromDate, $toDate, $vendorId),
                Service_PbaApi_Api_Exception::CAN_NOT_MERGE_DELIVER_LINES_FOR_PERIOD_BY_VENDOR_CODE,
                $e
            );
        }
    }

    /**
     * @param int $fromDate
     * @param int $toDate
     * @param int $vendorId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_DeliverLine[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getDeliverLinesForPeriodByVendorList($fromDate, $toDate, $vendorId, $sortNo = 0)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetDeliverLinesForPeriodByVendorList($fromDate, $toDate, $vendorId, $sortNo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_DELIVER_LINES_FOR_PERIOD_BY_VENDOR_LIST_MESSAGE, $fromDate, $toDate, $vendorId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_DELIVER_LINES_FOR_PERIOD_BY_VENDOR_LIST_CODE,
                $e
            );
        }
    }

    /**
     * @param int $planId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_PlanHistoryListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanHistoryList($planId, $sortNo = 0)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetPlanHistoryList($planId, $sortNo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_HISTORY_LIST_MESSAGE, $planId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_HISTORY_LIST_CODE,
                $e
            );
        }
    }

    /**
     * @param int $planId
     * @param int $userId
     * @param int $date
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanFromArchive($planId, $userId, $date)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetPlanFromArchive($planId, $userId, $date);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_FROM_ARCHIVE_MESSAGE, $planId, $userId, $date),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_FROM_ARCHIVE_CODE,
                $e
            );
        }
    }

    /**
     * @param int $planId
     * @return Service_PbaApi_Api_Response_Struct_PlanPublicationStatus
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanPublicationStatusForUpdate($planId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetPlanPublicationStatusForUpdate($planId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_PUBLICATION_STATUS_FOR_PLAN_MESSAGE, $planId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_PUBLICATION_STATUS_FOR_PLAN_CODE,
                $e
            );
        }
    }

    /**
     * @param int $planId
     * @param int $published
     * @param int $showPriority
     * @param int $defaultPlanPeriodId
     * @return Service_PbaApi_Api_Response_Struct_Status
     * @throws Service_PbaApi_Api_Exception
     */
    public function updatePlanPublicationStatus($planId, $published, $showPriority, $defaultPlanPeriodId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_UpdatePlanPublicationStatus($planId, $published, $showPriority, $defaultPlanPeriodId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_UPDATE_PUBLICATION_STATUS_FOR_PLAN_MESSAGE, $planId),
                Service_PbaApi_Api_Exception::CAN_NOT_UPDATE_PUBLICATION_STATUS_FOR_PLAN_CODE,
                $e
            );
        }
    }

    /**
     * @param int $discountId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_DiscountedResourceListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getDiscountedResourceList($discountId, $sortNo = 0)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetDiscountedResourceList($discountId, $sortNo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_DISCOUNTED_RESOURCES_BY_DISCOUNT_MESSAGE, $discountId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_DISCOUNTED_RESOURCES_BY_DISCOUNT_CODE,
                $e
            );
        }
    }

    /**
     * @param int $discountId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_DiscountedPlanListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getDiscountedPlanList($discountId, $sortNo = 0)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetDiscountedPlanList($discountId, $sortNo);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_DISCOUNTED_PLANS_BY_DISCOUNT_MESSAGE, $discountId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_DISCOUNTED_PLANS_BY_DISCOUNT_CODE,
                $e
            );
        }
    }

    /**
     * @param $detailId
     * @return Service_PbaApi_Api_Response_Struct_OrderDetail
     * @throws Service_PbaApi_Api_Exception
     */
    public function getOrderDetail($detailId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetOrderDetail($detailId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_ORDER_DETAIL_MESSAGE, $detailId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_ORDER_DETAIL_CODE,
                $e
            );
        }
    }

    /**
     * @param $periodId
     * @return Service_PbaApi_Api_Response_Struct_PlanPeriod
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanPeriod($periodId)
    {
        try {
            $request = new Service_PbaApi_Api_Request_GetPlanPeriod($periodId);
            $response = $this->_client->execute($request);
            return $response->getData();
        } catch (Exception $e) {
            throw new Service_PbaApi_Api_Exception(
                sprintf(Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_PERIOD_MESSAGE, $periodId),
                Service_PbaApi_Api_Exception::CAN_NOT_GET_PLAN_PERIOD_CODE,
                $e
            );
        }
    }
}
