<?php

/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */
class Service_PbaApi_Api_Request_Struct_OrderDetailAttributes
{
    /**
     * the detail line amount;
     * @var float
     */
    private $extendedPrice;

    /**
     * the detail line discount;
     * @var float
     */
    private $discountAmount;

    /**
     * taxes total amount of the detail line;
     * @var float
     */
    private $taxAmount;

    /**
     * taxes amount included in to the detail extended price;
     * @var float
     */
    private $taxIncludedAmount;

    /**
     * signifies how taxes for this order detail are calculated.
     * Available options are:
     * "0" - taxes are to be calculated by Business Automation ,
     * "10" - Business Automation will set taxes specified in the passed tax details;
     * @var int
     */
    private $taxAlgorithm;

    /**
     * a unit price. Passed value is not used for any calculations.
     * It is displayed in the Unit Price column of the order details list;
     * @var float
     */
    private $unitPrice;

    /**
     * units quantity. Passed value is not used for any calculations.
     * It is displayed in the Quantity column of the order details list;
     * @var int
     */
    private $serviceQuantity;

    /**
     * unit of measure. It is displayed in the Unit of Measure column of the order details list;
     * @var string
     */
    private $serviceUnitMeasure;

    /**
     * billing period type. Available options are:
     * "3" - year(s),
     * "4" - month(s).
     * It is displayed in the Duration column of the order details list;
     * @var int
     */
    private $durationBillPeriodType;

    /**
     * billing period duration.
     * It is displayed in the Duration column of the order details list;
     * The parameter is optional and can be skipped, recommended value is "1".;
     * @var int
     */
    private $durationBillPeriod;

    /**
     * duration. It is displayed in the Duration column of the order details list.
     * The parameter is optional and can be skipped;
     * @var int
     */
    private $duration;

    /**
     * ID of an existing tax category applied to the order detail.
     * It is displayed in the Tax Category column of the order details list;
     * @var int
     */
    private $taxCategoryId;

    /**
     * order detail description;
     * @var string
     */
    private $description;

    /**
     * ID of a resource category applied to the order detail.
     * The parameter is optional and can be skipped;
     * @var int
     */
    private $resourceCategoryId;

    /**
     * ID of a plan category applied to the order detail.
     * The parameter is optional and can be skipped;
     * @var int
     */
    private $planCategoryId;

    /**
     * ID of the customer existing subscription.
     * The parameter is optional and can be skipped;
     * @var int
     */
    private $subscriptionId;

    /**
     * ID of a subscription period applied to the order detail.
     * The parameter is optional and can be skipped;
     * @var int
     */
    private $planPeriodId;

    /**
     * ID of a promotion applied to the order detail.
     * The parameter is optional and can be skipped;
     * @var int
     */
    private $promoId;

    /**
     * ID of a discount applied to the order detail.
     * The parameter is optional and can be skipped;
     * @var int
     */
    private $discountId;

    /**
     * ID of a resource. The parameter is optional and can be skipped;
     * @var int
     */
    private $resourceId;

    /**
     * @return float
     */
    public function getExtendedPrice()
    {
        return $this->extendedPrice;
    }

    /**
     * @param float $extendedPrice
     */
    public function setExtendedPrice($extendedPrice)
    {
        $this->extendedPrice = $extendedPrice;
    }

    /**
     * @return float
     */
    public function getDiscountAmount()
    {
        return $this->discountAmount;
    }

    /**
     * @param float $discountAmount
     */
    public function setDiscountAmount($discountAmount)
    {
        $this->discountAmount = $discountAmount;
    }

    /**
     * @return float
     */
    public function getTaxAmount()
    {
        return $this->taxAmount;
    }

    /**
     * @param float $taxAmount
     */
    public function setTaxAmount($taxAmount)
    {
        $this->taxAmount = $taxAmount;
    }

    /**
     * @return float
     */
    public function getTaxIncludedAmount()
    {
        return $this->taxIncludedAmount;
    }

    /**
     * @param float $taxIncludedAmount
     */
    public function setTaxIncludedAmount($taxIncludedAmount)
    {
        $this->taxIncludedAmount = $taxIncludedAmount;
    }

    /**
     * @return int
     */
    public function getTaxAlgorithm()
    {
        return $this->taxAlgorithm;
    }

    /**
     * @param int $taxAlgorithm
     */
    public function setTaxAlgorithm($taxAlgorithm)
    {
        $this->taxAlgorithm = $taxAlgorithm;
    }

    /**
     * @return float
     */
    public function getUnitPrice()
    {
        return $this->unitPrice;
    }

    /**
     * @param float $unitPrice
     */
    public function setUnitPrice($unitPrice)
    {
        $this->unitPrice = $unitPrice;
    }

    /**
     * @return int
     */
    public function getServiceQuantity()
    {
        return $this->serviceQuantity;
    }

    /**
     * @param int $serviceQuantity
     */
    public function setServiceQuantity($serviceQuantity)
    {
        $this->serviceQuantity = $serviceQuantity;
    }

    /**
     * @return string
     */
    public function getServiceUnitMeasure()
    {
        return $this->serviceUnitMeasure;
    }

    /**
     * @param string $serviceUnitMeasure
     */
    public function setServiceUnitMeasure($serviceUnitMeasure)
    {
        $this->serviceUnitMeasure = $serviceUnitMeasure;
    }

    /**
     * @return int
     */
    public function getDurationBillPeriodType()
    {
        return $this->durationBillPeriodType;
    }

    /**
     * @param int $durationBillPeriodType
     */
    public function setDurationBillPeriodType($durationBillPeriodType)
    {
        $this->durationBillPeriodType = $durationBillPeriodType;
    }

    /**
     * @return int
     */
    public function getDurationBillPeriod()
    {
        return $this->durationBillPeriod;
    }

    /**
     * @param int $durationBillPeriod
     */
    public function setDurationBillPeriod($durationBillPeriod)
    {
        $this->durationBillPeriod = $durationBillPeriod;
    }

    /**
     * @return int
     */
    public function getDuration()
    {
        return $this->duration;
    }

    /**
     * @param int $duration
     */
    public function setDuration($duration)
    {
        $this->duration = $duration;
    }

    /**
     * @return int
     */
    public function getTaxCategoryId()
    {
        return $this->taxCategoryId;
    }

    /**
     * @param int $taxCategoryId
     */
    public function setTaxCategoryId($taxCategoryId)
    {
        $this->taxCategoryId = $taxCategoryId;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return int
     */
    public function getResourceCategoryId()
    {
        return $this->resourceCategoryId;
    }

    /**
     * @param int $resourceCategoryId
     */
    public function setResourceCategoryId($resourceCategoryId)
    {
        $this->resourceCategoryId = $resourceCategoryId;
    }

    /**
     * @return int
     */
    public function getPlanCategoryId()
    {
        return $this->planCategoryId;
    }

    /**
     * @param int $planCategoryId
     */
    public function setPlanCategoryId($planCategoryId)
    {
        $this->planCategoryId = $planCategoryId;
    }

    /**
     * @return int
     */
    public function getSubscriptionId()
    {
        return $this->subscriptionId;
    }

    /**
     * @param int $subscriptionId
     */
    public function setSubscriptionId($subscriptionId)
    {
        $this->subscriptionId = $subscriptionId;
    }

    /**
     * @return int
     */
    public function getPlanPeriodId()
    {
        return $this->planPeriodId;
    }

    /**
     * @param int $planPeriodId
     */
    public function setPlanPeriodId($planPeriodId)
    {
        $this->planPeriodId = $planPeriodId;
    }

    /**
     * @return int
     */
    public function getPromoId()
    {
        return $this->promoId;
    }

    /**
     * @param int $promoId
     */
    public function setPromoId($promoId)
    {
        $this->promoId = $promoId;
    }

    /**
     * @return int
     */
    public function getDiscountId()
    {
        return $this->discountId;
    }

    /**
     * @param int $discountId
     */
    public function setDiscountId($discountId)
    {
        $this->discountId = $discountId;
    }

    /**
     * @return int
     */
    public function getResourceId()
    {
        return $this->resourceId;
    }

    /**
     * @param int $resourceId
     */
    public function setResourceId($resourceId)
    {
        $this->resourceId = $resourceId;
    }

    public function __toString()
    {
        $attributes = array(
            'ExtendedPrice' => $this->extendedPrice,
            'TaxAmt' => $this->taxAmount,
            'TaxInclAmt' => $this->taxIncludedAmount,
            'DiscountAmt' => $this->discountAmount,
            'TaxAlg' => $this->taxAlgorithm,
            'UnitPrice' => $this->unitPrice,
            'ServQty' => $this->serviceQuantity,
            'ServUnitMeasure' => $this->serviceUnitMeasure,
            'Duration' => $this->duration,
            'DurBillPeriod' => $this->durationBillPeriod,
            'DurBillPeriodType' => $this->durationBillPeriodType,
            'TaxCatID' => $this->taxCategoryId,
            'ResCatID' => $this->resourceCategoryId,
            'planCategoryID' => $this->planCategoryId,
            'subscriptionID' => $this->subscriptionId,
            'PlanPeriodID' => $this->planPeriodId,
            'PromoID' => $this->promoId,
            'DiscID' => $this->discountId,
            'resourceID' => $this->resourceId,
            'Descr' => $this->description,
        );

        $result = 'OrdDet:';
        foreach ($attributes as $k => $v) {
            if (is_null($v)) {
                continue;
            }
            $result .= $k . '=' . $v . ';';
        }
        return $result;
    }
}
