<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_PbaApi_Api_Response_Struct_SubscriptionDetailsEx implements Service_PbaApi_Api_Response_Struct_SubscriptionDetailsExInterface
{
    /**
     * subscription identifier;
     * @var int
     */
    private $subscriptionId;

    /**
     * subscription name;
     * @var str
     */
    private $subscriptionName;

    /**
     * identifier of the account to which the subscription belongs;
     * @var int
     */
    private $accountId;

    /**
     * identifier of the service plan with which the subscription is associated;
     * @var int
     */
    private $planId;

    /**
     * name of the subscription service plan.
     * Starting from Business Automation 5.0, a service plan name is a multi-language field.
     * To get a plan name in a particular language, you need to specify this language in the method call.
     * If a language is not specified, the method returns values in English (default) language.
     * For details, refer to the Specifying Locale section;
     * @var str
     */
    private $planName;

    /**
     * subscription status (
     * the possible status values are:
     * 10 – "Ordered",
     * 15 – "Trial",
     * 30 – "Active",
     * 40 – "Graced",
     * 50 – "Expired",
     * 60 – "Terminated",
     * 70 – "Canceled",
     * 80 – "Administrative Hold",
     * 85 – "Credit Hold",
     * 89 – "Credit and Administrative Hold"
     * );
     * @var int
     */
    private $status;

    /**
     * subscription service current status (
     * the possible status values are:
     * 10 – Not Provisioned,
     * 20 – Provisioning,
     * 30 – Stopped,
     * 40 – Starting,
     * 50 – Running,
     * 60 – Stopping,
     * 70 – Removing,
     * 80 – Changing Plan,
     * 90 – Removed
     * );
     * @var int
     */
    private $servStatus;

    /**
     * start date of the subscription period (in Unix format);
     * @var int
     */
    private $startDate;

    /**
     * expiration date of the subscription period (in Unix format);
     * @var int
     */
    private $expirationDate;

    /**
     * date of when the subscription was billed the last time (Unix format);
     * @var int
     */
    private $lastBillDate;

    /**
     * date of when the subscription is to be billed the next time (Unix format);
     * @var int
     */
    private $nextBillDate;

    /**
     * subscription billing period type (
     * 4 – monthly on statement cycle date,
     * 2 – fixed number of months,
     * 3 – fixed number of years
     * );
     * @var int
     */
    private $billingPeriodType;

    /**
     * subscription billing period duration
     * (if BillingPeriodType=2 and BillingPeriod=1, subscription billing period is 1 month);
     * @var int
     */
    private $billingPeriod;

    /**
     * indicates whether the subscription period is trial: 0 – No, 1 – Yes;
     * @var int
     */
    private $trial;

    /**
     * the stopping date of the service from the subscription.
     * It is equal to ExpirationDate + Grace Period set in Service Terms;
     * @var int
     */
    private $shutdownDate;

    /**
     * the date of destroying of the service from the subscription. It is equal to ShutdownDate + Hold Period set in Service Terms;
     * @var int
     */
    private $terminateDate;

    /**
     * the subscription period duration that depends on the PeriodType
     * (for example, 2 can mean 2 Days, 2 Months, or 2 Years);
     * @var int
     */
    private $period;

    /**
     * the subscription period type: 1 - Day(s), 2 - Month(s), 3 - Year(s);
     * @var int
     */
    private $periodType;

    /**
     * the freezing of prices option: 0 - No, 1 - Yes.
     * @var int
     */
    private $freezePrices;

    /**
     * Promotion code applied to the subscription.
     * @var str
     */
    private $promoCode;

    /**
     * @return int
     */
    public function getSubscriptionId()
    {
        return $this->subscriptionId;
    }

    /**
     * @param int $subscriptionId
     */
    public function setSubscriptionId($subscriptionId)
    {
        $this->subscriptionId = $subscriptionId;
    }

    /**
     * @return str
     */
    public function getSubscriptionName()
    {
        return $this->subscriptionName;
    }

    /**
     * @param str $subscriptionName
     */
    public function setSubscriptionName($subscriptionName)
    {
        $this->subscriptionName = $subscriptionName;
    }

    /**
     * @return int
     */
    public function getAccountId()
    {
        return $this->accountId;
    }

    /**
     * @param int $accountId
     */
    public function setAccountId($accountId)
    {
        $this->accountId = $accountId;
    }

    /**
     * @return int
     */
    public function getPlanId()
    {
        return $this->planId;
    }

    /**
     * @param int $planId
     */
    public function setPlanId($planId)
    {
        $this->planId = $planId;
    }

    /**
     * @return str
     */
    public function getPlanName()
    {
        return $this->planName;
    }

    /**
     * @param str $planName
     */
    public function setPlanName($planName)
    {
        $this->planName = $planName;
    }

    /**
     * @return int
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * @param int $status
     */
    public function setStatus($status)
    {
        $this->status = $status;
    }

    /**
     * @return int
     */
    public function getServStatus()
    {
        return $this->servStatus;
    }

    /**
     * @param int $servStatus
     */
    public function setServStatus($servStatus)
    {
        $this->servStatus = $servStatus;
    }

    /**
     * @return int
     */
    public function getStartDate()
    {
        return $this->startDate;
    }

    /**
     * @param int $startDate
     */
    public function setStartDate($startDate)
    {
        $this->startDate = $startDate;
    }

    /**
     * @return int
     */
    public function getExpirationDate()
    {
        return $this->expirationDate;
    }

    /**
     * @param int $expirationDate
     */
    public function setExpirationDate($expirationDate)
    {
        $this->expirationDate = $expirationDate;
    }

    /**
     * @return int
     */
    public function getLastBillDate()
    {
        return $this->lastBillDate;
    }

    /**
     * @param int $lastBillDate
     */
    public function setLastBillDate($lastBillDate)
    {
        $this->lastBillDate = $lastBillDate;
    }

    /**
     * @return int
     */
    public function getNextBillDate()
    {
        return $this->nextBillDate;
    }

    /**
     * @param int $nextBillDate
     */
    public function setNextBillDate($nextBillDate)
    {
        $this->nextBillDate = $nextBillDate;
    }

    /**
     * @return int
     */
    public function getBillingPeriodType()
    {
        return $this->billingPeriodType;
    }

    /**
     * @param int $billingPeriodType
     */
    public function setBillingPeriodType($billingPeriodType)
    {
        $this->billingPeriodType = $billingPeriodType;
    }

    /**
     * @return int
     */
    public function getBillingPeriod()
    {
        return $this->billingPeriod;
    }

    /**
     * @param int $billingPeriod
     */
    public function setBillingPeriod($billingPeriod)
    {
        $this->billingPeriod = $billingPeriod;
    }

    /**
     * @return int
     */
    public function getTrial()
    {
        return $this->trial;
    }

    /**
     * @param int $trial
     */
    public function setTrial($trial)
    {
        $this->trial = $trial;
    }

    /**
     * @return int
     */
    public function getShutdownDate()
    {
        return $this->shutdownDate;
    }

    /**
     * @param int $shutdownDate
     */
    public function setShutdownDate($shutdownDate)
    {
        $this->shutdownDate = $shutdownDate;
    }

    /**
     * @return int
     */
    public function getTerminateDate()
    {
        return $this->terminateDate;
    }

    /**
     * @param int $terminateDate
     */
    public function setTerminateDate($terminateDate)
    {
        $this->terminateDate = $terminateDate;
    }

    /**
     * @return int
     */
    public function getPeriod()
    {
        return $this->period;
    }

    /**
     * @param int $period
     */
    public function setPeriod($period)
    {
        $this->period = $period;
    }

    /**
     * @return int
     */
    public function getPeriodType()
    {
        return $this->periodType;
    }

    /**
     * @param int $periodType
     */
    public function setPeriodType($periodType)
    {
        $this->periodType = $periodType;
    }

    /**
     * @return int
     */
    public function getFreezePrices()
    {
        return $this->freezePrices;
    }

    /**
     * @param int $freezePrices
     */
    public function setFreezePrices($freezePrices)
    {
        $this->freezePrices = $freezePrices;
    }

    /**
     * @return str
     */
    public function getPromoCode()
    {
        return $this->promoCode;
    }

    /**
     * @param str $promoCode
     */
    public function setPromoCode($promoCode)
    {
        $this->promoCode = $promoCode;
    }
}
