<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Services\Cache;

use Acronis\App\Services\Redis\RedisClient;

class RedisCache implements CacheInterface
{
    /** @var RedisClient  */
    private $redisClient;

    /** @var string  */
    private $keyPrefix;

    /** @var int */
    private $ttl;

    /**
     * RedisStorage constructor.
     * @param RedisClient $redisClient
     * @param string $keyPrefix
     * @param int $ttl Expiration time in seconds
     */
    function __construct($redisClient, $keyPrefix = '', $ttl = null)
    {
        $this->redisClient = $redisClient;
        $this->keyPrefix = $keyPrefix;
        $this->ttl = $ttl;
    }

    /**
     * @param string $key
     * @param mixed $value
     * @param null|int $ttl
     * @throws \Exception
     */
    public function set($key, $value, $ttl = null)
    {
        $this->redisClient->set(
            $this->formatKey($key),
            $this->encode($value),
            is_null($ttl) ? $this->ttl : $ttl
        );
    }

    /**
     * @param string $key
     * @param string|null $default
     * @return mixed|null
     * @throws \Exception
     */
    public function get($key, $default = null)
    {
        $dataStr = $this->redisClient->get($this->formatKey($key));

        return !is_null($dataStr) ? $this->decode($dataStr) : $default;
    }

    /**
     * @param $key
     * @return bool
     */
    public function has($key)
    {
        return $this->redisClient->has($this->formatKey($key));
    }

    public function refresh($key, $ttl)
    {
        $this->redisClient->refresh($key, $ttl);
    }

    /***
     * @param mixed $data
     * @throws \Exception
     * @return false|string
     */
    protected function encode($data)
    {
        return serialize($data);
    }

    /**
     * @param string $dataStr
     * @throws \Exception
     * @return mixed
     */
    protected function decode($dataStr)
    {
        return unserialize($dataStr);
    }

    /**
     * @param string $key
     * @return string
     */
    protected function formatKey($key)
    {
        return $this->keyPrefix . $key;
    }
}