<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Services\Config;

use Str;

class ConfigAccessor
{
    use ConfigDatabaseTrait;
    use ConfigRedisTrait;
    use ConfigUsageReportTrait;
    use ConfigCacheTrait;
    use ConfigCloudApiTrait;

    protected $config;

    /**
     * ConfigAccessor constructor.
     * @param array $config
     */
    public function __construct($config)
    {
        $this->config = $config;
    }

    /**
     * @param $relativePath
     * @return string
     * @throws \Exception
     */
    public function getAbsolutePath($relativePath)
    {
        $filePath = DIR_CONFIG . ltrim($relativePath, DS);
        $dirPath = dirname($filePath);
        $fileName = basename($filePath);
        $dirRealPath = realpath($dirPath);
        if ($dirRealPath === false) {
            throw new \Exception(Str::format(
                'There is no access to folder "%s" with specified file "%s".',
                $dirRealPath, $fileName
            ));
        }

        return $dirRealPath . DS . $fileName;
    }

    /**
     * @param $sectionName
     * @return mixed
     * @throws \Exception
     */
    protected function getSection($sectionName)
    {
        if (!isset($this->config[$sectionName])) {
            throw new \Exception(Str::format(
                'Config section "%s" is not found in "%s".',
                $sectionName, APP_CONFIG_FILE
            ));
        }

        return $this->config[$sectionName];
    }
}