<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Services\Config;

use Symfony\Component\Config\Definition\Builder\NodeBuilder;
use Symfony\Component\Config\Definition\Builder\TreeBuilder;
use Symfony\Component\Config\Definition\ConfigurationInterface;

class ConfigSchema implements ConfigurationInterface
{
    const NODE_DATABASE = 'database';
    const NODE_DATABASE_DRIVER = 'driver';
    const NODE_DATABASE_PATH = 'database';
    const NODE_DATABASE_PREFIX = 'prefix';

    const NODE_REDIS = 'redis';
    const NODE_REDIS_DSN = 'dsn';
    const NODE_REDIS_PREFIX = 'prefix';

    const NODE_USAGE_REPORT = 'usage_report';
    const NODE_USAGE_REPORT_STORAGE = 'storage';
    const NODE_USAGE_REPORT_PREFIX = 'prefix';
    const NODE_USAGE_REPORT_TTL = 'ttl';
    const NODE_USAGE_REPORT_RETRY_TIMEOUT = 'retry_timeout';
    const NODE_USAGE_REPORT_RETRIES_LIMIT = 'retries_limit';
    const NODE_USAGE_REPORT_DOWNLOAD_PATH = 'download_path';
    const NODE_USAGE_REPORT_DOWNLOADED_FILES_TTL_IN_DAYS = 'downloaded_files_ttl_in_days';

    const NODE_CACHE = 'cache';
    const NODE_CACHE_PROVIDER = 'provider';
    const NODE_CACHE_PREFIX = 'prefix';
    const NODE_CACHE_TTL = 'ttl';

    const NODE_CLOUD_API = 'cloud_api';
    const NODE_CLOUD_API_VERIFY_SSL = 'verify_ssl';
    const NODE_CLOUD_API_ACCESS_TOKEN_TTL = 'access_token_ttl';

    public function getConfigTreeBuilder()
    {
        $treeBuilder = new TreeBuilder();
        $rootNode = $treeBuilder->root('application');

        $node = $rootNode->children();
        $this->initDatabaseNode($node);
        $this->initRedisNode($node);
        $this->initUsageReportNode($node);
        $this->initCacheNode($node);
        $this->initCloudApiNode($node);
        $node->end();

        return $treeBuilder;
    }

    /**
     * @param NodeBuilder $node
     */
    private function initCloudApiNode($node)
    {
        $node->arrayNode(self::NODE_CLOUD_API)
            ->children()
            ->booleanNode(self::NODE_CLOUD_API_VERIFY_SSL)->isRequired()->end()
            ->scalarNode(self::NODE_CLOUD_API_ACCESS_TOKEN_TTL)->isRequired()->end()
            ->end()
            ->end();
    }

    /**
     * @param NodeBuilder $node
     */
    private function initDatabaseNode($node)
    {
        $node->arrayNode(self::NODE_DATABASE)
            ->children()
            ->scalarNode(self::NODE_DATABASE_DRIVER)->isRequired()->end()
            ->scalarNode(self::NODE_DATABASE_PATH)->end()
            ->scalarNode(self::NODE_DATABASE_PREFIX)->end()
            ->end()
            ->end();
    }

    /**
     * @param NodeBuilder $node
     */
    private function initRedisNode($node)
    {
        $node->arrayNode(self::NODE_REDIS)
            ->children()
            ->scalarNode(self::NODE_REDIS_DSN)->end()
            ->scalarNode(self::NODE_REDIS_PREFIX)->end()
            ->end()
            ->end();
    }

    /**
     * @param NodeBuilder $node
     */
    private function initUsageReportNode($node)
    {
        $node->arrayNode(self::NODE_USAGE_REPORT)
            ->children()
            ->enumNode(self::NODE_USAGE_REPORT_STORAGE)->values(StorageProvidersEnum::getConstants())->isRequired()->end()
            ->scalarNode(self::NODE_USAGE_REPORT_PREFIX)->end()
            ->scalarNode(self::NODE_USAGE_REPORT_TTL)->end()
            ->scalarNode(self::NODE_USAGE_REPORT_DOWNLOAD_PATH)->isRequired()->end()
            ->scalarNode(self::NODE_USAGE_REPORT_DOWNLOADED_FILES_TTL_IN_DAYS)->end()
            ->integerNode(self::NODE_USAGE_REPORT_RETRIES_LIMIT)->isRequired()->end()
            ->integerNode(self::NODE_USAGE_REPORT_RETRY_TIMEOUT)->isRequired()->end()
            ->end()
            ->end();
    }

    /**
     * @param NodeBuilder $node
     */
    private function initCacheNode($node)
    {
        $node->arrayNode(self::NODE_CACHE)
            ->children()
            ->enumNode(self::NODE_CACHE_PROVIDER)->values(CacheProvidersEnum::getConstants())->isRequired()->end()
            ->scalarNode(self::NODE_CACHE_PREFIX)->end()
            ->integerNode(self::NODE_CACHE_TTL)->end()
            ->end()
            ->end();
    }
}