<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Services\CountersManager;

use Acronis\App\Base\ApplicationCounter as Counter;
use Acronis\App\CloudApi\ApiInterface;
use Acronis\App\Utils\UomConverter as Uom;
use Arr;

class CountersManager
{

    /** @var array */
    private $applications;

    /** @var Counter[] */
    private $countersHash;

    /** @var Counter[] */
    private $offeringItemsHash;

    /**
     * Counters constructor.
     *
     * @param array $countersData
     */
    public function __construct(array $countersData)
    {
        $this->countersHash = [];
        $this->offeringItemsHash = [];
        $this->applications = [];

        foreach ($countersData as $counterData) {
            $counter = new Counter($counterData);

            $this->countersHash[$counter->getName()] = $counter;
            $this->offeringItemsHash[$counter->getOfferingItemName()][] = $counter;
            $this->applications[$counter->getApplicationType()][] = $counter;
        }
    }

    /**
     * @param string $counterName
     * @return Counter|null
     */
    public function getCounterByName($counterName)
    {
        return Arr::get($this->countersHash, $counterName, null);
    }

    /**
     * @param string $offeringItemName
     * @return Counter[]
     */
    public function findCountersByOfferingItemName($offeringItemName)
    {
        return Arr::get($this->offeringItemsHash, $offeringItemName, []);
    }

    /**
     * @param string $applicationType
     * @return Counter[]
     */
    public function getApplicationCounters($applicationType)
    {
        return Arr::get($this->applications, $applicationType, []);
    }

    /**
     * @return Counter[]
     */
    public function getAllCounters()
    {
        return array_values($this->countersHash);
    }

    public function convertValueToOfferingItemUnits(Counter $counter, $counterValue)
    {
        if ($counterValue === 0) {
            return 0;
        }

        if ($counterValue === Counter::UNLIMITED_VALUE) {
            return ApiInterface::UNLIMITED_OFFERING_ITEM_VALUE;
        }

        return Uom::convert($counterValue, $counter->getUnit(), $counter->getOfferingItemUnit());
    }

    public function convertValueFromOfferingItemUnits(Counter $counter, $offeringItemValue)
    {
        if ($offeringItemValue === 0) {
            return 0;
        }

        if ($offeringItemValue === ApiInterface::UNLIMITED_OFFERING_ITEM_VALUE) {
            return Counter::UNLIMITED_VALUE;
        }

        return Uom::convert($offeringItemValue, $counter->getOfferingItemUnit(), $counter->getUnit());
    }
}