<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Services\Logger;

use LogInterface;
use Monolog\Logger;
use Psr\Log\LoggerInterface;

/**
 * Class LoggerDecorator
 * @package Acronis\App\Services\Logger
 */
class AppLoggerAdapter implements LogInterface
{
    use LoggerAwareTrait;

    public function emergency($message)
    {
        $args = func_get_args();
        $this->log(Logger::EMERGENCY, $args);
    }

    public function alert($message)
    {
        $args = func_get_args();
        $this->log(Logger::ALERT, $args);
    }

    public function critical($message)
    {
        $args = func_get_args();
        $this->log(Logger::CRITICAL, $args);
    }

    public function notice($message)
    {
        $args = func_get_args();
        $this->log(Logger::NOTICE, $args);
    }

    public function info($message)
    {
        $args = func_get_args();
        $this->log(Logger::INFO, $args);
    }

    public function debug($message)
    {
        $args = func_get_args();
        $this->log(Logger::DEBUG, $args);
    }

    public function event($message)
    {
        $args = func_get_args();
        $this->log(Logger::NOTICE, $args);
    }

    public function warning($message)
    {
        $args = func_get_args();
        $this->log(Logger::WARNING, $args);
    }

    public function error($message)
    {
        $args = func_get_args();
        $this->log(Logger::ERROR, $args);
    }

    private function log($level, $args)
    {
        $caller = debug_backtrace();
        $zeroFrame = $caller[1];
        $this->getLogger()->log(
            $level,
            call_user_func_array(['Str', 'format'], $args),
            [
                'request_id' => \Log::getRequestId(),
                'file' => $zeroFrame['file'],
                'line' => $zeroFrame['line'],
            ]
        );
    }
}