<?php
/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\UsageReport;

use Acronis\App\Database\Model\ReportEntry;
use Acronis\App\UsageReport\Aggregation\Aggregator;
use Acronis\App\UsageReport\Exception\ManagerException;
use APS\ControllerProxy;
use RuntimeException;

interface  ManagerInterface
{
    /**
     * @param ControllerProxy $apsc
     */
    public function setApsController($apsc);

    /**
     * @param string $subscriptionId
     * @param string $date
     * @param string $datacenterApsId
     * @return mixed
     * @throws ManagerException
     */
    public function getUsagesForSubscription($subscriptionId, $date, $datacenterApsId);

    /**
     * @param ReportEntry $reportEntry
     * @param $apsInstanceId
     * @return Aggregator
     * @throws ManagerException
     * @throws \Service_BaseClient_HTTPException
     * @throws \Swagger\Client\ApiException
     * @throws \Swagger\Client\HttpException
     * todo remove $apsInstanceId after adding tenantId to reseller's map
     */
    public function getAggregator($reportEntry, $apsInstanceId);

    /**
     * @param ReportEntry $reportEntry
     * @throws \Swagger\Client\ApiException
     * @throws \Swagger\Client\HttpException
     * @throws ManagerException
     */
    public function orderReport($reportEntry);

    /**
     * @param ReportEntry $reportEntry
     * @throws ManagerException
     */
    public function checkForStoredReport($reportEntry);

    /**
     * @param ReportEntry $reportEntry
     * @throws \Swagger\Client\ApiException
     * @throws \Swagger\Client\HttpException
     * @throws ManagerException
     */
    public function downloadReport($reportEntry);

    /**
     * Removes all saved reports and {date} folders from /tmp/reports/{date}/{id}.csv.gz
     */
    public function cleanReportDownloadPath();

    /**
     * After this ttl all downloaded report files should be deleted. -1 means infinite ttl.
     * @return int
     */
    public function getReportDownloadedFilesTtlInDays();

    public function eraseReports();

    /**
     * @param Aggregator $aggregator
     * @param ReportEntry $reportEntry
     * @throws \Exception
     * @throws RuntimeException
     */
    public function aggregateReport($aggregator, $reportEntry);

    /**
     * @return string
     */
    public function getRetryTimeout();

    /**
     * @return string
     */
    public function getRetriesLimit();
}