<?php
/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

use Acronis\App\Base\ApplicationCounter as Counter;
use Acronis\App\CloudApi\ApiInterface as CloudApi;
use Acronis\App\Utils\UomConverter as Uom;
use Swagger\Client\Model\Infra\InfraCap;

/**
 * Available types of a counter:
 *   feature - will be disabled/enabled if limit equals/not equals to 0
 *   limit - can be used to get limit only
 *   usage - can be used to set usage only
 *   counter - limit and usage are available
 * Available measurement unit types of a counter:
 *    Unit: u - quantity of units
 *    Storage: b, kb, mb, gb
 *    Time: s, m, h, d
 * Format:
 * return [
 *   [
 *       'application_type' => 'backup|files_cloud|notary',
 *       'counter_type' => 'feature|limit|usage|counter',
 *       'counter_name' => '<counter name>',
 *       'counter_unit' => '<counter unit of measurement>',
 *       'offering_item_name' => '<offering item name>',
 *       'offering_item_unit' => '<offering item unit of measurement>',
 *       'infra_capability' => 'backup|disaster_recovery|files_cloud',
 *       'is_infra' => true|false,
 *       'is_ranged' => true|false,
 *       'is_fake' => true|false,
 *       'tenant_types' => null|['CUSTOMER', 'PARTNER', 'RESELLER'],
 *   ],
 * // ...
 * ];
 */

return [
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::STORAGE_USAGE,
        'counter_unit' => Uom::BYTES,
        'offering_item_name' => CloudApi::OFFERING_ITEM_STORAGE,
        'offering_item_unit' => Uom::BYTES,
        'infra_capability' => InfraCap::BACKUP,
        'is_infra' => true,
        'sub_offering_items' => [
            CloudApi::OFFERING_ITEM_CHILD_STORAGES,
        ],
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::LOCAL_STORAGE_USAGE,
        'counter_unit' => Uom::BYTES,
        'offering_item_name' => CloudApi::OFFERING_ITEM_LOCAL_STORAGE,
        'offering_item_unit' => Uom::BYTES,
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::WORKSTATION_COUNT,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_WORKSTATIONS,
        'offering_item_unit' => Uom::UNITS,
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::SERVER_COUNT,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_SERVERS,
        'offering_item_unit' => Uom::UNITS,
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::VM_COUNT,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_VMS,
        'offering_item_unit' => Uom::UNITS,
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::MOBILE_DEVICE_COUNT,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_MOBILES,
        'offering_item_unit' => Uom::UNITS,
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::MAILBOX_COUNT,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_MAILBOXES,
        'offering_item_unit' => Uom::UNITS,
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_LIMIT,
        'counter_name' => tenant::MAILBOX_INSTANCE_SIZE,
        'counter_unit' => Uom::BYTES,
        'offering_item_name' => CloudApi::OFFERING_ITEM_MAILBOX_INSTANCE_SIZE,
        'offering_item_unit' => Uom::BYTES,
        'depends_on' => [
            tenant::MAILBOX_COUNT,
        ],
        'is_fake' => true,
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_FEATURE,
        'counter_name' => tenant::O365_MAILBOXES,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_O365_MAILBOXES,
        'offering_item_unit' => Uom::UNITS,
        'depends_on' => [
            tenant::MAILBOX_COUNT,
        ],
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_FEATURE,
        'counter_name' => tenant::O365_ONE_DRIVE,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_O365_ONEDRIVE,
        'offering_item_unit' => Uom::UNITS,
        'depends_on' => [
            tenant::MAILBOX_COUNT,
        ],
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::O365_SHARE_POINT_SITES_COUNT,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_O365_SHAREPOINT_SITES,
        'offering_item_unit' => Uom::UNITS,
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::WEB_HOSTING_SERVER_COUNT,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_WEB_HOSTING_SERVERS,
        'offering_item_unit' => Uom::UNITS,
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::WEBSITE_COUNT,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_WEBSITES,
        'offering_item_unit' => Uom::UNITS,
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::DR_STORAGE_USAGE,
        'counter_unit' => Uom::BYTES,
        'offering_item_name' => CloudApi::OFFERING_ITEM_DR_STORAGE,
        'offering_item_unit' => Uom::BYTES,
        'infra_capability' => InfraCap::DISASTER_RECOVERY,
        'is_infra' => true,
        'depends_on' => [
            tenant::STORAGE_USAGE,
        ],
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::DR_COMPUTE_POINTS_COUNT,
        'counter_unit' => Uom::HOURS, // units per hours
        'offering_item_name' => CloudApi::OFFERING_ITEM_COMPUTE_POINTS,
        'offering_item_unit' => Uom::SECONDS, // units per hours
        'infra_capability' => InfraCap::DISASTER_RECOVERY,
        'is_infra' => true,
        'is_ranged' => true,
        'depends_on' => [
            tenant::STORAGE_USAGE,
        ],
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::DR_PUBLIC_IPS_COUNT,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_PUBLIC_IPS,
        'offering_item_unit' => Uom::UNITS,
        'infra_capability' => InfraCap::DISASTER_RECOVERY,
        'is_infra' => true,
        'depends_on' => [
            tenant::STORAGE_USAGE,
        ],
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_COUNTER,
        'counter_name' => tenant::DR_CLOUD_SERVERS_COUNT,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_DR_CLOUD_SERVERS,
        'offering_item_unit' => Uom::UNITS,
        'infra_capability' => InfraCap::DISASTER_RECOVERY,
        'is_infra' => true,
        'depends_on' => [
            tenant::STORAGE_USAGE,
        ],
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_FEATURE,
        'counter_name' => tenant::DR_INTERNET_ACCESS,
        'counter_unit' => Uom::UNITS,
        'offering_item_name' => CloudApi::OFFERING_ITEM_DR_INTERNET_ACCESS,
        'offering_item_unit' => Uom::UNITS,
        'infra_capability' => InfraCap::DISASTER_RECOVERY,
        'is_infra' => true,
        'depends_on' => [
            tenant::STORAGE_USAGE,
        ],
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_USAGE,
        'counter_name' => tenant::ACRONIS_STORAGE_USAGE,
        'counter_unit' => Uom::BYTES,
        'offering_item_name' => CloudApi::OFFERING_ITEM_STORAGE,
        'offering_item_unit' => Uom::BYTES,
        'tenant_types' => [
            tenant::TYPE_PARTNER,
            tenant::TYPE_RESELLER,
        ],
        'aggregation_rules' => [
            'infra.affinity' => 'inherited',
            'infra.backend_type' => ['acronis', null],
        ],
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_USAGE,
        'counter_name' => tenant::PROVIDER_STORAGE_USAGE,
        'counter_unit' => Uom::BYTES,
        'offering_item_name' => CloudApi::OFFERING_ITEM_STORAGE,
        'offering_item_unit' => Uom::BYTES,
        'tenant_types' => [
            tenant::TYPE_PARTNER,
            tenant::TYPE_RESELLER,
        ],
        'aggregation_rules' => [
            'infra.affinity' => ['own', 'child'],
        ],
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_USAGE,
        'counter_name' => tenant::GOOGLE_STORAGE_USAGE,
        'counter_unit' => Uom::BYTES,
        'offering_item_name' => CloudApi::OFFERING_ITEM_STORAGE,
        'offering_item_unit' => Uom::BYTES,
        'tenant_types' => [
            tenant::TYPE_PARTNER,
            tenant::TYPE_RESELLER,
        ],
        'aggregation_rules' => [
            'infra.backend_type' => 'google',
        ],
    ],
    [
        'application_type' => CloudApi::APPLICATION_TYPE_BACKUP,
        'counter_type' => Counter::TYPE_USAGE,
        'counter_name' => tenant::AZURE_STORAGE_USAGE,
        'counter_unit' => Uom::BYTES,
        'offering_item_name' => CloudApi::OFFERING_ITEM_STORAGE,
        'offering_item_unit' => Uom::BYTES,
        'tenant_types' => [
            tenant::TYPE_PARTNER,
            tenant::TYPE_RESELLER,
        ],
        'aggregation_rules' => [
            'infra.backend_type' => 'azure',
        ],
    ],
];