<?php
/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

use Acronis\App\Services\AgentsManager\AgentsManager;

/**
 * Use this map to add new agents on user agents download view.
 *
 * # Map key  means response resulting key of certain agent.
 *
 * # Category means ui key of agent category (group of agents under one title)
 *
 * # Agent is identified by exact list of required parameters:
 *  - system
 *  - type
 *  - architecture
 *  - web
 *
 * Format:
 * return [
 *   'windows|windows_server|apple|ios|linux|etc.' => [
 *       'category' => 'workstations|servers|mobileDevices|virtualizationHosts|applications',
 *       'system' => 'windows|linux|apple|android|ios|hyperv|vmware|pcs|sql|arx|ad|office',
 *       'type' => 'consumer|enterprise',
 *       'architecture' => '32|64|' <- last possible value is empty string,
 *       'web' => 'true|null' <- boolean
 *   ],
 * // ...
 * ];
 */

return [
    'windows' => [
        'title' => 'Windows',
        'category' => AgentsManager::CATEGORY_WORKSTATIONS,
        'system' => AgentsManager::SYSTEM_WINDOWS,
        'type' => AgentsManager::TYPE_ENTERPRISE,
        'architecture' => AgentsManager::ARCHITECTURE_NOT_SPECIFIED,
    ],
    'apple' => [
        'title' => 'Mac OS',
        'system' => AgentsManager::SYSTEM_APPLE,
        'category' => AgentsManager::CATEGORY_WORKSTATIONS,
        'type' => AgentsManager::TYPE_ENTERPRISE,
        'architecture' => AgentsManager::ARCHITECTURE_64,
    ],
    'windows_server' => [
        'title' => 'Windows',
        'category' => AgentsManager::CATEGORY_SERVERS,
        'system' => AgentsManager::SYSTEM_WINDOWS_SERVER,
        'type' => AgentsManager::TYPE_ENTERPRISE,
        'architecture' => AgentsManager::ARCHITECTURE_NOT_SPECIFIED,
    ],
    'linux' => [
        'title' => 'Linux',
        'system' => AgentsManager::SYSTEM_LINUX,
        'category' => AgentsManager::CATEGORY_SERVERS,
        'type' => AgentsManager::TYPE_ENTERPRISE,
        'architecture' => AgentsManager::ARCHITECTURE_64,
    ],
    'ios' => [
        'title' => 'iOS',
        'system' => AgentsManager::SYSTEM_IOS,
        'category' => AgentsManager::CATEGORY_MOBILE_DEVICES,
        'type' => AgentsManager::TYPE_ENTERPRISE,
        'architecture' => AgentsManager::ARCHITECTURE_NOT_SPECIFIED,
    ],
    'android' => [
        'title' => 'Android',
        'system' => AgentsManager::SYSTEM_ANDROID,
        'category' => AgentsManager::CATEGORY_MOBILE_DEVICES,
        'type' => AgentsManager::TYPE_ENTERPRISE,
        'architecture' => AgentsManager::ARCHITECTURE_NOT_SPECIFIED,
    ],
    'vmware' => [
        'title' => 'VMware ESX',
        'system' => AgentsManager::SYSTEM_VMWARE,
        'category' => AgentsManager::CATEGORY_VIRTUALIZATION_HOSTS,
        'type' => AgentsManager::TYPE_ENTERPRISE,
        'architecture' => AgentsManager::ARCHITECTURE_NOT_SPECIFIED,
    ],
    'hyperv' => [
        'title' => 'Hyper-V',
        'system' => AgentsManager::SYSTEM_HYPERV,
        'category' => AgentsManager::CATEGORY_VIRTUALIZATION_HOSTS,
        'type' => AgentsManager::TYPE_ENTERPRISE,
        'architecture' => AgentsManager::ARCHITECTURE_NOT_SPECIFIED,
    ],
    'pcs' => [
        'title' => 'Virtuozzo',
        'system' => AgentsManager::SYSTEM_VIRTUOZZO,
        'category' => AgentsManager::CATEGORY_VIRTUALIZATION_HOSTS,
        'type' => AgentsManager::TYPE_ENTERPRISE,
        'architecture' => AgentsManager::ARCHITECTURE_64,
    ],
    'exchange' => [
        'title' => 'Microsoft Exchange Server',
        'system' => AgentsManager::SYSTEM_MICROSOFT_EXCHANGE_SERVER,
        'category' => AgentsManager::CATEGORY_APPLICATIONS,
        'type' => AgentsManager::TYPE_ENTERPRISE,
        'architecture' => AgentsManager::ARCHITECTURE_NOT_SPECIFIED,
    ],
    'sql' => [
        'title' => 'Microsoft SQL Server',
        'system' => AgentsManager::SYSTEM_MICROSOFT_SQL_SERVER,
        'category' => AgentsManager::CATEGORY_APPLICATIONS,
        'type' => AgentsManager::TYPE_ENTERPRISE,
        'architecture' => AgentsManager::ARCHITECTURE_NOT_SPECIFIED,
    ],
    'ad' => [
        'title' => 'Microsoft Active Directory',
        'system' => AgentsManager::SYSTEM_MICROSOFT_ACTIVE_DIRECTORY,
        'category' => AgentsManager::CATEGORY_APPLICATIONS,
        'type' => AgentsManager::TYPE_ENTERPRISE,
        'architecture' => AgentsManager::ARCHITECTURE_NOT_SPECIFIED,
    ],
];