<?php

/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class BuildInfo implements BuildInfoInterface
{
    // separator from key to value
    const PARAM_SEPARATOR = "=";
    // separator for vers.ion file lines
    const PARAM_LINE_SEPARATOR = "\n";
    // key of buildnumber in Vers.ion file
    const BUILD_NUMBER_KEY = "BUILD_NUMBER";
    const MAJOR_VERSION_KEY = "MAJOR_VERSION";
    const MINOR_VERSION_KEY = "MINOR_VERSION";

    /* @var array key-value store with params */
    private $_buildInfo;

    /**
     *
     * @param Service_File_Local $versionFile instance with build_number data
     */
    public function __construct(Service_FileInterface $versionFile = NULL)
    {
        if (is_null($versionFile)) {
            $versionFile = new Service_File_Local(DIR_APP . VERSION_FILE_NAME);
        }

        $versionData = explode(self::PARAM_LINE_SEPARATOR, trim($versionFile->getContents()));
        foreach ($versionData as $line) {
            $line = trim($line);

            if (empty($line)) {
                continue;
            }
            list($infoKey, $infoValue) = explode(self::PARAM_SEPARATOR, $line);
            $this->_buildInfo[trim($infoKey)] = trim($infoValue);
        }
    }

    /**
     * Method returns number of current (this) APS Package Build
     * @return int
     */
    public function getPackageBuildNumber()
    {
        return isset($this->_buildInfo[self::BUILD_NUMBER_KEY]) ? $this->_buildInfo[self::BUILD_NUMBER_KEY] : null;
    }

    /**
     * Method returns version of current (this) APS Package in format: Major.Minor.Build (example:1.0.1406)
     * @return string
     */
    public function getPackageVersion()
    {
        $major = isset($this->_buildInfo[self::MAJOR_VERSION_KEY]) ? $this->_buildInfo[self::MAJOR_VERSION_KEY] : null;
        $minor = isset($this->_buildInfo[self::MINOR_VERSION_KEY]) ? $this->_buildInfo[self::MINOR_VERSION_KEY] : null;
        $build = isset($this->_buildInfo[self::BUILD_NUMBER_KEY]) ? $this->_buildInfo[self::BUILD_NUMBER_KEY] : null;

        return $major . '.' . $minor . '.' . $build;
    }
}
