<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */


class Curl implements CurlInterface
{
    /** @var resource */
    private $_curlHandle;

    /** @var array */
    private $_headers = array();

    public function __construct()
    {
        $this->_curlHandle = curl_init();
        $this->setOption(CURLOPT_HEADER, false);
        $this->setOption(CURLOPT_HEADERFUNCTION, array($this, 'headerCallback'));
    }

    /**
     * @param string $option
     * @param string|bool|int|array $value
     * @return Curl
     */
    public function setOption($option, $value)
    {
        curl_setopt($this->_curlHandle, $option, $value);
        return $this;
    }

    /**
     * @return mixed
     */
    public function execute()
    {
        $this->_headers = array();
        return curl_exec($this->_curlHandle);
    }

    /**
     * @return string
     */
    public function getError()
    {
        return curl_error($this->_curlHandle);
    }

    /**
     * @param int $option
     * @return mixed
     */
    public function getInfo($option = NULL)
    {
        return curl_getinfo($this->_curlHandle, $option);
    }

    /**
     * @return int
     */
    public function getErrorNo()
    {
        return curl_errno($this->_curlHandle);
    }

    public function refresh()
    {
        curl_close($this->_curlHandle);
        $this->_curlHandle = curl_init();
    }

    /**
     * @param resource $curlHandle
     * @param string $header
     * @return int
     */
    public function headerCallback($curlHandle, $header)
    {
        if (trim($header) === '')
        {
            return strlen($header);
        }

        if (strpos($header, ':') === false)
        {
            $key = 0;
            $value = trim($header);
        }
        else
        {
            list($key, $value) = explode(':', $header, 2);
            $key = trim($key);
            $value = trim($value);
        }

        if (array_key_exists($key, $this->_headers))
        {
            if (!is_array($this->_headers[$key]))
            {
                $this->_headers[$key] = array($this->_headers[$key]);
            }
            $this->_headers[$key][] = $value;
        }
        else
        {
            $this->_headers[$key] = $value;
        }

        return strlen($header);
    }

    public function getHeaders()
    {
        return $this->_headers;
    }

    /**
     * Close resource descriptor
     */
    public function __destruct()
    {
        curl_close($this->_curlHandle);
    }
} 