<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

abstract class Enum
{
    /** @var int */
    private $_value;

    /** @var array */
    protected static $constants;

    /**
     * @param int $value
     * @throws InvalidArgumentException
     */
    public function __construct($value)
    {
        $constants = array_flip(static::getConstants());
        if (!array_key_exists($value, $constants)) {
            throw new InvalidArgumentException(Str::format(
                'Incorrect enum value: "%s".',
                $value
            ));
        }
        $this->_value = $value;
    }

    public static function getConstants()
    {
        $className = static::class;
        if (!isset(static::$constants[$className])) {
            $reflection = new ReflectionClass($className);
            static::$constants[$className] = $reflection->getConstants();
        }

        return static::$constants[$className];
    }

    /**
     * @return int
     */
    public function getValue()
    {
        return $this->_value;
    }
}
