<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Facade implements FacadeInterface
{
    /**
     * @param string $baasUrl
     * @param string $instanceId
     * @return Service_MspApiClient_Api
     */
    public function getMspApi($baasUrl, $instanceId = NULL)
    {
        $baasUrl = trim($baasUrl, " \t\n\r\0\x0B/");
        $restClient = new Service_BaseClient_RestClient();
        $restClient->setApiUri($baasUrl . MSP_API_URI);
        if (!is_null($instanceId))
        {
            $cookieFile = DIR_COOKIES . $instanceId;
            $restClient->setCookieFile($cookieFile);
        }
        return new Service_MspApiClient_Api($restClient);
    }

    /**
     * @param string $amsUrl
     * @return Service_AmsApiClient_ApiInterface
     */
    public function getAmsApi($amsUrl)
    {
        $amsUrl = trim($amsUrl, " \t\n\r\0\x0B/");
        $restClient = new Service_BaseClient_RestClient();
        $restClient->setApiUri($amsUrl . AMS_API_URI);
        return new Service_AmsApiClient_Api($restClient);
    }

    /**
     * @param string $openApiHost
     * @return Service_OpenApi_Api
     */
    public function getOpenApi($openApiHost)
    {
        $client = new Service_OpenApi_Client($openApiHost);
        return new Service_OpenApi_Api($client);
    }

    /**
     * @param APS\ControllerProxy $apsController
     * @return Service_APSApi_ApiInterface
     */
    public function getApsApi($apsController)
    {
        return new Service_APSApi_Api($apsController);
    }

    /**
     * @param object $config
     * @return Service_PbaApi_Api
     */
    public function getPbaApi($config)
    {
        $xmlRpcClient = new Service_PbaApi_Client($config->host, $config->port, $config->path, $config->secure);
        if ($config->auth) {
            $xmlRpcClient->setCredentials($config->username, $config->password);
        }
        return new Service_PbaApi_Api($xmlRpcClient);
    }
}
