<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

class Json
{
    const DEFAULT_JSON_DEPTH = 512;
    const DEFAULT_JSON_OPTIONS = 0;
    const DEFAULT_JSON_ASSOC = false;

    /**
     * @param mixed $value Value that you want to encode
     * @param int $options json_encode() $options value, optional, 0 by default
     * @param int $depth json_encode() $depth value, optional, 512 by default
     * @return false|string
     * @throws Exception
     */
    public static function encode(
        $value,
        $options = self::DEFAULT_JSON_OPTIONS,
        $depth = self::DEFAULT_JSON_DEPTH
    ) {
        $result = json_encode($value, $options, $depth);
        self::throwExceptionForLastError();

        return $result;
    }

    /**
     * @param string $json JSON-formatted string value that  you want to decode
     * @param bool $assoc json_decode() $assoc flag, optional, FALSE by default
     * @param int $depth json_encode() $options value, optional, 512 by default
     * @param int $options json_encode() $depth value, optional, 0 by default
     * @return mixed
     * @throws Exception
     */
    public static function decode(
        $json,
        $assoc = self::DEFAULT_JSON_ASSOC,
        $depth = self::DEFAULT_JSON_DEPTH,
        $options = self::DEFAULT_JSON_OPTIONS
    ) {
        $result = json_decode($json, $assoc, $depth, $options);
        self::throwExceptionForLastError();

        return $result;
    }

    /**
     * @throws Exception
     */
    public static function throwExceptionForLastError()
    {
        $error = json_last_error();
        if ($error === JSON_ERROR_NONE) {
            return;
        }
        $message = json_last_error_msg();
        throw new Exception($message, $error);
    }
}