<?php

/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */
class UomConvertHelper
{
    const MEASURABLE_MULTIPLIER = 1024;
    const MEASURABLE_B = 'b';
    const MEASURABLE_KB = 'kb';
    const MEASURABLE_MB = 'mb';
    const MEASURABLE_GB = 'gb';
    const MEASURABLE_TB = 'tb';

    /**
     * Measurement units converter (b, kb, mg, gb, tb)
     *
     * @param int|float $value
     * @param string $from
     * @param string $to
     * @return int|float|NULL
     */
    public static function convert($value, $from, $to)
    {
        $uomFromName = strtolower($from);
        $uomToName = strtolower($to);

        $init = 1;
        $names = array(
            self::MEASURABLE_B => $init,
            self::MEASURABLE_KB => ($init *= self::MEASURABLE_MULTIPLIER),
            self::MEASURABLE_MB => ($init *= self::MEASURABLE_MULTIPLIER),
            self::MEASURABLE_GB => ($init *= self::MEASURABLE_MULTIPLIER),
            self::MEASURABLE_TB => ($init * self::MEASURABLE_MULTIPLIER),
        );

        if (!isset($names[$uomFromName])) {
            return null;
        }

        if (!isset($names[$uomToName])) {
            return null;
        }

        $uomFromValue = $names[$uomFromName];
        $uomToValue = $names[$uomToName];

        return $value * $uomFromValue / $uomToValue;
    }
}
