<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require_once(__DIR__ . "/app/bootstrap.php");
require_once(__DIR__ . "/billing/BillingHelper.php");

chdir(__DIR__);
define('APS_RUNTIME_NO_PROCESS', true);

use Monolog\Formatter\LineFormatter;
use Monolog\Handler\StreamHandler;
use Monolog\Logger;

const OUTPUT_OPTION_UNDEFINED = "Option '%s' undefined \n";

const OPTION_PARSE_CHAR = ':';
const OPTION_APPLICATION = 'a';
const OPTION_VERBOSE = 'v';
const OPTION_HELP = 'h';

/**
 *  Prints usage to the screen
 */
function printScriptUsage()
{
    $fileName = basename($_SERVER['PHP_SELF']);
    echo "Usage: php {$fileName} [OPTIONS]
Example: php {$fileName} -a 10 -vv
Options:
  -a APPLICATION   Acronis Backup Service application ID
  -v               Verbose
";
}

function validateCommandLineParameters($opt)
{
    if (isset($opt[OPTION_HELP])) {
        return true;
    }

    if (!isset($opt[OPTION_APPLICATION]) || !ctype_digit($opt[OPTION_APPLICATION])) {
        printScriptUsage();
        return false;
    }
    return true;
}

/**
 * @param string $optionName
 * @param array $options
 * @return mixed | null
 */
function getOptions($optionName, $options)
{
    $option = null;
    switch ($optionName) {
        case OPTION_HELP:
            $option = isset($options[OPTION_HELP]);
            break;
        case OPTION_APPLICATION:
            $option = intval($options[OPTION_APPLICATION]);
            break;

        case OPTION_VERBOSE:
            if (isset($options[OPTION_VERBOSE])) {
                $option = is_array($options[OPTION_VERBOSE]) ? count($options[OPTION_VERBOSE]) : 1;
            } else {
                $option = 0;
            }
            break;
        default:
            echo sprintf(OUTPUT_OPTION_UNDEFINED, $optionName);
            printScriptUsage();
            exit;
    }
    return $option;
}

$opt = getopt(
    OPTION_APPLICATION . OPTION_PARSE_CHAR .
    OPTION_VERBOSE .
    OPTION_HELP
);

if (!validateCommandLineParameters($opt)) {
    exit(1);
};

if (getOptions(OPTION_HELP, $opt)) {
    printScriptUsage();
    exit;
}

$applicationId = getOptions(OPTION_APPLICATION, $opt);
$verbose = getOptions(OPTION_VERBOSE, $opt);

$output = Log::FIELD_TIME . ' ' . Log::FIELD_REQUEST . ' ' . Log::FIELD_TYPE . ' ' . Log::FIELD_MESSAGE . PHP_EOL;
$formatter = new LineFormatter($output);

$stream = new StreamHandler('php://stdout', Logger::NOTICE - $verbose * 50);
$stream->setFormatter($formatter);
Log::getInstance()->getLogger()->pushHandler($stream);

try {
    $apiFactory = new ApiFactory();
    $helper = new BillingHelper($apiFactory, $applicationId);
    $helper->process();
} catch (Exception $e) {
    Log::getInstance()->error($e->getMessage());
    exit(1);
}
