<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

use APS\ControllerProxy;

/**
 * @Copyright © 2002-2015 Acronis International GmbH. All rights reserved
 */
class MigrationHelper
{
    const PROPERTY_ID = "ID";
    const FIELD_APPLICATION_INSTANCE_ID = 'application_instance_id';
    const FIELD_GROUP_LOGIN = 'group_login';
    const FIELD_NAME = 'name';
    const FIELD_VALUE = 'value';
    const FIELD_SETTINGS = 'settings';
    const FIELD_USER_LOGIN = 'user_login';
    const FIELD_USER_QUOTA = 'user_quota';

    const RQL_IMPLEMENTING_SERVICE_USER = "implementing(http://aps-standard.org/types/core/service-user/1.0)";
    const RQL_IMPLEMENTING_SUBSCRIPTION_EQ_SUBSCRIPTION_ID = "implementing(http://parallels.com/aps/types/pa/subscription/1.0),eq(subscriptionId,%s)";
    const RQL_IMPLEMENTING_ACCOUNT_EQ_ID = "implementing(http://parallels.com/aps/types/pa/account/1.0),eq(id,%s)";
    const RQL_IMPLEMENTING_TENANT_EQ_GROUP_ID_LIMIT_1 = 'implementing(%s),eq(mspGroupId,%s),limit(0,1)';
    const RQL_IMPLEMENTING_TENANT_EQ_SUBSCRIPTION_ID_LIMIT_1 = 'implementing(%s),eq(subscriptionId,%s),limit(0,1)';

    const EXCEPTION_NOT_FOUND_MESSAGE = "GetUserInfoFromBackupAgent not found";
    const EXCEPTION_SUBSCRIPTION_NOT_FOUND_TEMPLATE = "Subscription with id %s not found";
    const EXCEPTION_GROUP_NOT_FOUND_BY_SUBSCRIPTION_TEMPLATE = "Group name is the subscription with id %s not found";
    const EXCEPTION_GLOBALS_NOT_FOUND_TEMPLATE = 'Globals is the subscription with id %s not found';
    const EXCEPTION_CREATE_TENANT_FAILED_MESSAGE = 'Failed to create tenant';
    const EXCEPTION_CREATE_USER_FAILED_MESSAGE = 'Failed to create user %s';
    const EXCEPTION_MIGRATION_GROUP_NOT_FOUND_MESSAGE = 'Migration group not found';
    const EXCEPTION_BAAS_USER_NOT_FOUND = 'User not found. Id: %s domainController: %s';
    const EXCEPTION_USER_QUOTA_NOT_FOUND = 'User quota not found';
    const EXCEPTION_PLACE_UPGRADE_ORDER_ERROR = 'Place upgrade resource order for subscription with id %s is failed';
    const EXCEPTION_ENDPOINT_HOST_NOT_FOUNT = 'Endpoint host not found';
    const EXCEPTION_SUBSCRIPTION_ALREADY_MIGRATED = 'Subscription %s already migrated';
    const EXCEPTION_GROUP_NOT_MIGRATED = 'Group %s not migrated via Migration portal';

    const EXCEPTION_CODE_INVALID_RESOURCE_VERSION = 426;

    const LOG_CREATING_TENANT_FOR_SUBSCRIPTION = 'Creating tenant for subscription with id %s...';
    const LOG_TENANT_FOR_SUBSCRIPTION_WAS_CREATED = 'Creating tenant for subscription with id %s...';
    const LOG_CREATING_USER = 'Creating user %s ...';
    const LOG_USER_WAS_CREATED = 'Creating user %s ...';

    const EUC_KIND_ID = 40;
    const BA_STORAGE_RESOURCE_NAME = 'BA Counter';
    const ACCOUNT_SERVICE_NAME = 'account';
    const UNLIMITED_QUOTA_VALUE = -1;
    const USE_PAY_TOOL = null;
    const USE_DEFAULT_PAY_TOOL = 1;

    const STORAGE_USAGE_COUNTER = 'StorageUsage';
    const SERVER_COUNTER = 'ServerCount';
    const VM_COUNTER = 'VmCount';
    const WORKSTATION_COUNTER = 'WorkstationCount';

    const MOBILE_DEVICE_COUNTER = 'MobileDeviceCount';
    const MAILBOX_COUNTER = 'MailboxCount';
    const WEBSITE_COUNTER = 'WebsiteCount';

    const COMPOSITE_RESOURCE_TYPE_VALUE = 1;
    const DEFAULT_SLIP_TIME = 3;
    const LONG_SLIP_TIME = 10;

    const APS_STATUS_READY = "aps:ready";
    const LINK_DATACENTER = "datacenter";

    /**
     * @var Service_OpenApi_Api
     */
    public $openApi;

    /**
     * @var \APS\ControllerProxy
     */
    public $apsc;

    /**
     * @var Service_MspApiClient_Api
     */
    public $mspApi;

    /**
     * @var Service_PbaApi_Api
     */
    private $pbaApi;

    /**
     * @var BackupAgentSoapClient
     */
    private $baSoapClient;

    private $tenantCounterNames = array(
        self::STORAGE_USAGE_COUNTER,
        self::SERVER_COUNTER,
        self::VM_COUNTER,
        self::WORKSTATION_COUNTER,
    );

    private $newCounterNames = array(
        self::MOBILE_DEVICE_COUNTER,
        self::MAILBOX_COUNTER,
        self::WEBSITE_COUNTER,
    );

    /**
     * @var string UUID
     */
    private $apsPackageId;

    public function __construct(ControllerProxy $apsc, Service_OpenApi_Api $api, Service_MspApiClient_Api $mspApi, Service_PbaApi_Api $pbaApi, BackupAgentSoapClient $baSoapClient, $apsPackageId)
    {
        $this->apsc = $apsc;
        $this->openApi = $api;
        $this->mspApi = $mspApi;
        $this->pbaApi = $pbaApi;
        $this->baSoapClient = $baSoapClient;
        $this->apsPackageId = $apsPackageId;
    }

    /**
     *  Get users id from Baas
     * @param Service_MspApiClient_Api $mspApi
     * @param string $userLogin
     * @return int
     */
    public function getUserId($mspApi, $userLogin)
    {
        $domainController = BACKUPAGENT_SERVER_DNS;
        $userInfo = $this->getUserInfoFromBackupAgent($userLogin);

        return $this->getUserIdFromBaas($mspApi, $domainController, $userInfo[self::PROPERTY_ID]);
    }

    /**
     * Get user info from BackupAgent
     * @param string $userName
     * @return int
     * @throws Exception
     */
    public function getUserInfoFromBackupAgent($userName)
    {
        $getSpecificUserInfo = new BackupAgent_GetSpecificUserInfo();
        $getSpecificUserInfo->userName = $userName;

        $userInfo = $this->baSoapClient->GetSpecificUserInfo($getSpecificUserInfo);

        if ($userInfo == null) {
            throw new Exception(self::EXCEPTION_NOT_FOUND_MESSAGE);
        }

        return $userInfo;
    }

    /**
     *  Get group id from Baas
     * @param Service_MspApiClient_Api $mspApi
     * @param string $domainController
     * @param int $migratedGroupId
     * @return int
     * @throws Exception
     */
    public function getMspGroupIdFromBaas($mspApi, $domainController, $migratedGroupId)
    {
        /** @var Service_MspApiClient_Response_Struct_MigrationGroup $migrationGroup */
        $migrationGroup = $mspApi->getMigrationGroup($domainController, $migratedGroupId);

        if ($migrationGroup == null) {
            throw new Exception(self::EXCEPTION_MIGRATION_GROUP_NOT_FOUND_MESSAGE);
        }

        if ($migrationGroup->getKind() == self::EUC_KIND_ID) {
            return $migrationGroup->getId();
        }
        $children = $mspApi->getGroups($migrationGroup->getId());

        if (count($children) == 0) {
            return $migrationGroup->getId();
        }

        return $children[0]->getId();
    }

    /**
     *  Get user id from Baas
     * @param Service_MspApiClient_Api $mspApi
     * @param string $domainController
     * @param int $migratedUserId
     * @return int
     * @throws Exception
     */
    public function getUserIdFromBaas($mspApi, $domainController, $migratedUserId)
    {
        /** @var Service_MspApiClient_Response_Struct_MigrationUser $user */
        $user = $mspApi->getMigrationUser($domainController, $migratedUserId);
        if ($user == null) {
            throw new Exception(sprintf(self::EXCEPTION_BAAS_USER_NOT_FOUND, $migratedUserId, $domainController));
        }

        return $user->getId();
    }

    /**
     * @param users $baasApsUser
     * @return int
     */
    public function getMspUserId($baasApsUser)
    {
        $mspApi = $baasApsUser->getMspApi();
        return $this->getUserId($mspApi, $baasApsUser->user->login);
    }

    /**
     * Creates tenant in Baas APS
     * @param $subscriptionId
     * @param $accountId
     * @param $mspGroupId
     * @param $datacenter
     * @param $fakeProvision
     * @return string
     * @throws Exception
     */
    public function createTenant($subscriptionId, $accountId, $mspGroupId, $datacenter, $fakeProvision = true)
    {
        Log::getInstance()->event(sprintf(self::LOG_CREATING_TENANT_FOR_SUBSCRIPTION, $subscriptionId));
        $subscription = $this->getSubscriptionById($subscriptionId);
        $global = $this->getGlobals($subscription);
        $account = $this->getAPSAccount($accountId);

        $newTenant = APS\TypeLibrary::newResourceByTypeId(\APS\TypeLibrary::getTypeByServiceId('tenant', $this->apsPackageId));
        $newTenant->datacenterId = $datacenter->id;
        $newTenant->ismigration = true;
        if ($fakeProvision) {
            $newTenant->mspGroupId = $mspGroupId;
        }
        $newTenant->subscription->set($subscription);
        $newTenant->account->set($account);
        $newTenant->global->set($global);

        $counter = \APS\TypeLibrary::newResourceByTypeId('http://aps-standard.org/types/core/resource/1.0#Counter');
        $counter->limit = 0;
        $counter->usage = 0;

        $subscriptionResources = $subscription->resources();

        $newTenant->StorageUsage = clone $counter;
        $newTenant->StorageUsage->limit = $this->getSubscriptionResourceLimit($subscriptionResources, self::STORAGE_USAGE_COUNTER);

        $newTenant->ServerCount = clone $counter;
        $newTenant->ServerCount->limit = $this->getSubscriptionResourceLimit($subscriptionResources, self::SERVER_COUNTER);

        $newTenant->VmCount = clone $counter;
        $newTenant->VmCount->limit = $this->getSubscriptionResourceLimit($subscriptionResources, self::VM_COUNTER);

        $newTenant->WorkstationCount = clone $counter;
        $newTenant->WorkstationCount->limit = $this->getSubscriptionResourceLimit($subscriptionResources, self::WORKSTATION_COUNTER);

        $newTenant->MobileDeviceCount = clone $counter;
        $newTenant->MobileDeviceCount->limit = $this->getSubscriptionResourceLimit($subscriptionResources, self::MOBILE_DEVICE_COUNTER);

        $newTenant->MailboxCount = clone $counter;
        $newTenant->MailboxCount->limit = $this->getSubscriptionResourceLimit($subscriptionResources, self::MAILBOX_COUNTER);

        $newTenant->WebsiteCount = clone $counter;
        $newTenant->WebsiteCount->limit = $this->getSubscriptionResourceLimit($subscriptionResources, self::WEBSITE_COUNTER);

        return $newTenant;
    }

    public function provisionTenant($newTenant, $subscriptionId, $mspGroupId, $fakeProvision, $datacenter, $providerApsc)
    {
        try {
            $result = $this->apsc->provisionResource($newTenant);
            while ($result->aps->status !== self::APS_STATUS_READY) {
                sleep(3);
                $result = $providerApsc->getResource($result->aps->id);
            }
            $link = $result->datacenter->getLink()->href;
            while (!$providerApsc->getResource($link)) {
                sleep(self::DEFAULT_SLIP_TIME);
            }
        } catch (Exception $e) {
            Log::getInstance()->error($e->getMessage());
            $result = $this->handleProvisionException($subscriptionId, $mspGroupId, $fakeProvision, $e);
        }

        if (!isset($result->aps->id)) {
            throw new Exception(self::EXCEPTION_CREATE_TENANT_FAILED_MESSAGE);
        }

        Log::getInstance()->event(sprintf(self::LOG_TENANT_FOR_SUBSCRIPTION_WAS_CREATED, $subscriptionId));
        return $result;
    }

    /**
     *  Gets users
     * @var $account
     * @return array of users
     */
    public function getUsers($account)
    {
        $link = $account->users->getLink();
        $result = $this->apsc->getResources(self::RQL_IMPLEMENTING_SERVICE_USER, $link->href);
        return $result;
    }

    /**
     * Create user
     * @param $serviceUser
     * @param $tenant
     * @param boolean $activateUser
     * @return string
     * @throws Exception
     */
    public function createUser($serviceUser, $tenant, $activateUser)
    {
        Log::getInstance()->event(sprintf(self::LOG_CREATING_USER, $serviceUser->displayName));
        $apsc = $this->apsc;

        $defaultCounter = new Counter();
        $defaultCounter->limit = self::UNLIMITED_QUOTA_VALUE;


        $user = APS\TypeLibrary::newResourceByTypeId(\APS\TypeLibrary::getTypeByServiceId('users', $this->apsPackageId));
        $user->tenant->set($tenant);
        $user->user->set($serviceUser);

        $user->mspUserId = $this->getUserId($this->mspApi, $serviceUser->login);

        $user->userStorageUsage = $serviceUser->userStorageUsage;
        $user->userStorageUsage->limit = UomConvertHelper::convert(
            $user->userStorageUsage->limit,
            UomConvertHelper::MEASURABLE_GB,
            UomConvertHelper::MEASURABLE_B
        );

        $user->notifyBackup = Enum_NotificationsBackup::DISABLE_NOTIFICATIONS;
        $user->notificationsManagement = false;
        $user->UserVmCount = clone $defaultCounter;
        $user->UserWorkstationCount = clone $defaultCounter;
        $user->UserMobileDeviceCount = clone $defaultCounter;
        $user->userServerCount = clone $defaultCounter;
        $user->UserMailboxCount = clone $defaultCounter;
        $user->UserWebsiteCount = clone $defaultCounter;
        $user->activateDuringCreation = $activateUser;

        try {
            $result = $apsc->provisionResource($user);
        } catch (Exception $e) {
            Log::getInstance()->error($e->getMessage());
            throw new Exception(sprintf(self::EXCEPTION_CREATE_USER_FAILED_MESSAGE, $serviceUser->displayName));
        }

        if (!isset($result->aps->id)) {
            throw new Exception(sprintf(self::EXCEPTION_CREATE_USER_FAILED_MESSAGE, $serviceUser->displayName));
        }

        Log::getInstance()->event(sprintf(self::LOG_USER_WAS_CREATED, $serviceUser->displayName));

        return $result;
    }

    /**
     * @param $subscriptionId
     * @return mixed
     * @throws Exception
     */
    public function getSubscriptionById($subscriptionId)
    {
        $resultAnswer = $this->apsc->getResources(sprintf(self::RQL_IMPLEMENTING_SUBSCRIPTION_EQ_SUBSCRIPTION_ID, $subscriptionId));
        $subscription = $resultAnswer[0];
        if (empty($subscription)) {
            throw new Exception(sprintf(self::EXCEPTION_SUBSCRIPTION_NOT_FOUND_TEMPLATE, $subscriptionId));
        }

        return $subscription;
    }

    public function getSubscriptionResources($subscriptionId)
    {
        $apsSubscription = $this->getSubscriptionById($subscriptionId);
        return $apsSubscription->resources();
    }

    /**
     * @param $accountId
     * @return mixed
     */
    public function getAPSAccount($accountId)
    {
        $account = $this->apsc->getResources(sprintf(self::RQL_IMPLEMENTING_ACCOUNT_EQ_ID, $accountId));
        return reset($account);
    }


    private function getGlobals($subscription)
    {
        $resource = null;
        $resources = $subscription->resources();
        foreach ($resources as $resource) {
            if (!isset($resource->apsType)) {
                continue;
            }
            if (!isset($resource->property) && $resource->apsType == \APS\TypeLibrary::getTypeByServiceId('globals', $this->apsPackageId)) {
                break;
            }
        }
        if (empty($resource)) {
            throw new Exception(sprintf(self::EXCEPTION_GLOBALS_NOT_FOUND_TEMPLATE, $subscription->subscriptionId));
        }
        return $this->apsc->getResource($resource->apsId);
    }


    /**
     * Finalization of migration. After this method provisioning of
     * users and tenant goes in usual way (ismigration flag is set to false)
     * @param int $tenant
     * @param array $newUsers
     */
    public function setMigrationIsCompleted($tenant, $newUsers)
    {
        foreach ($newUsers as $id => $newUser) {
            $user = $this->apsc->getResource($newUser->aps->id);
            while ($user->aps->status !== self::APS_STATUS_READY) {
                sleep(self::DEFAULT_SLIP_TIME);
                $user = $this->apsc->getResource($newUser->aps->id);
            }
        }

        $tenant->ismigration = false;
        $method = \APS\Proto::PUT;
        $path = $this->apsc->getIo()->applicationPath("tenant/{$tenant->aps->id}");
        $data = json_encode($tenant);
        $this->apsc->getIo()->sendRequest($method, $path, $data);
    }

    /**
     * @param $groupId
     * @return Service_MspApiClient_Response_Struct_UserUsage
     */
    public function getUsageFromABC($groupId)
    {
        $group = $this->mspApi->getGroupById($groupId);
        return $group->getUsage();
    }

    public function updateSubscriptionLimits($tenant, $continueStatus, $terminateStatus, $abcStorageResourceId, $compositeResourceIDs)
    {
        $resourceUpgradeOrders = array();

        if (!is_null($tenant->mspGroupId)) {
            $usageABC = $this->getUsageFromABC($tenant->mspGroupId);
            $this->updateMachineCounterLimit($tenant, $usageABC);
            $newResourceUpgradeOrder = $this->processingNewResources($tenant, $usageABC);
            $resourceUpgradeOrders = array_merge($resourceUpgradeOrders, $newResourceUpgradeOrder);
        }

        $counters = $this->getCounters($tenant, $this->tenantCounterNames);
        if (isset($counters[self::STORAGE_USAGE_COUNTER])) {
            $counter = $counters[self::STORAGE_USAGE_COUNTER];
            $upgradeStorageOrder = $this->updateStorageCounter($tenant, $counter, $abcStorageResourceId, $compositeResourceIDs);
            $resourceUpgradeOrders = array_merge($resourceUpgradeOrders, $upgradeStorageOrder);

            unset($counters[self::STORAGE_USAGE_COUNTER]);
        }

        foreach ($counters as $counterName => $counter) {
            $upgradeCounterOrders = $this->updateCounter($tenant, $counter, $counterName);
            $resourceUpgradeOrders = array_merge($resourceUpgradeOrders, $upgradeCounterOrders);
        }

        if (!empty($resourceUpgradeOrders)) {
            $this->placeUpgradeOrder($tenant, $continueStatus, $terminateStatus, $resourceUpgradeOrders);
        }
    }

    /**
     * @param $resources
     * @param $resourceName
     * @return null|Service_PbaApi_Api_Response_Struct_SubscriptionResourcesList
     */
    private function getSubscriptionResourceByName($resources, $resourceName)
    {
        foreach ($resources as $resource) {
            /** @var Service_PbaApi_Api_Response_Struct_SubscriptionResourcesList $resource */
            if ($resource->getResourceName() == $resourceName) {
                return $resource;
            }
        }
        return null;
    }

    /**
     * @param $resources
     * @param $resourceId
     * @return null|Service_PbaApi_Api_Response_Struct_SubscriptionResourcesList
     */
    private function getSubscriptionResourceById($resources, $resourceId)
    {
        foreach ($resources as $resource) {
            /** @var Service_PbaApi_Api_Response_Struct_SubscriptionResourcesList $resource */
            if ($resource->getResourceId() == $resourceId) {
                return $resource;
            }
        }
        return null;
    }


    /**
     * @param $instanceId
     * @param $ownerId
     * @return array
     * @throws Exception
     */
    public function getUserForMigrate($instanceId, $ownerId)
    {
        $account = $this->getAPSAccount($ownerId);
        $users = $this->getUsers($account);
        $serviceSettings = $this->openApi->getServiceInstances($instanceId, self::ACCOUNT_SERVICE_NAME);
        $usersForMigrate = array();
        foreach ($serviceSettings as $serviceSetting) {
            $serviceUser = $this->getUserQuotaFromServiceSettings($serviceSetting[self::FIELD_SETTINGS]);
            foreach ($users as $user) {
                if ($user->login == $serviceUser->name) {
                    $counter = new Counter();
                    $counter->limit = $serviceUser->quota;
                    $counter->usage = 0;
                    $user->userStorageUsage = $counter;
                    $usersForMigrate[] = $user;
                }
            }
        }

        return $usersForMigrate;
    }

    /**
     * @param $serviceSettings
     * @return stdClass
     * @throws Exception
     */
    private function getUserQuotaFromServiceSettings($serviceSettings)
    {
        $user = new stdClass();
        foreach ($serviceSettings as $setting) {
            if ($setting[self::FIELD_NAME] == self::FIELD_USER_LOGIN) {
                $user->name = $setting[self::FIELD_VALUE];
            }
            if ($setting[self::FIELD_NAME] == self::FIELD_USER_QUOTA) {
                $user->quota = $setting[self::FIELD_VALUE];
            }
        }
        if (!isset($user->name) || !isset($user->quota)) {
            throw new Exception(self::EXCEPTION_USER_QUOTA_NOT_FOUND);
        }

        return $user;
    }

    /**
     * @param $tenant
     * @param array $counterFilter
     * @return array
     */
    public function getCounters($tenant, $counterFilter)
    {
        $counters = array();
        $resources = $tenant->subscription->resources();
        foreach ($resources as $resource) {
            if (!isset($resource->apsType)) {
                continue;
            }
            if ($resource->apsType == \APS\TypeLibrary::getTypeByServiceId('tenant', $this->apsPackageId)
                && isset($resource->property)
                && in_array($resource->property, $counterFilter)
            ) {
                $counters[$resource->property] = $resource;
            }
        }
        return $counters;
    }

    public function fetchEndPointHost()
    {
        $ehif = new EndpointHostIpFetcher($this->apsc);
        return $ehif->fetchApsEndpointIpAddressOrFallback();
    }

    public function getMspGroupId($groupLogin)
    {
        try {
            $groupInfo = $this->getUserInfoFromBackupAgent($groupLogin);
            $mspGroupId = $this->getMspGroupIdFromBaas($this->mspApi, BACKUPAGENT_SERVER_DNS, $groupInfo[self::PROPERTY_ID]);
        } catch (Exception $e) {
            Log::getInstance()->error($e->getMessage());
            throw new Exception(sprintf(self::EXCEPTION_GROUP_NOT_MIGRATED, $groupLogin));
        }

        return $mspGroupId;
    }

    public function checkTenantMigrated($mspGroupId, $subscriptionId, $fakeProvision)
    {
        $tenant = $fakeProvision ?
            $this->getTenantByGroupId($mspGroupId) :
            $this->getTenantBySubscriptionId($subscriptionId);
        if (!empty($tenant) && $tenant->ismigration == false) {
            throw new Exception(sprintf(self::EXCEPTION_SUBSCRIPTION_ALREADY_MIGRATED, $tenant->subscription->subscriptionId));
        }
    }

    public function getTenantByGroupId($mspGroupId)
    {
        $tenant = null;
        $tenants = $this->apsc->getResources(sprintf(self::RQL_IMPLEMENTING_TENANT_EQ_GROUP_ID_LIMIT_1, \APS\TypeLibrary::getTypeByServiceId('tenant', $this->apsPackageId), $mspGroupId));
        if (!empty($tenants)) {
            $tenant = $tenants[0];
        }
        return $tenant;
    }

    public function getTenantBySubscriptionId($subscriptionId)
    {
        $tenant = null;
        $tenants = $this->apsc->getResources(sprintf(self::RQL_IMPLEMENTING_TENANT_EQ_SUBSCRIPTION_ID_LIMIT_1, \APS\TypeLibrary::getTypeByServiceId('tenant', $this->apsPackageId), $subscriptionId));
        if (!empty($tenants)) {
            $tenant = $tenants[0];
        }
        return $tenant;
    }

    /**
     * @param $subscriptionId
     * @param $mspGroupId
     * @param $fakeProvision
     * @param $e
     * @return mixed
     * @throws Exception
     */
    private function handleProvisionException($subscriptionId, $mspGroupId, $fakeProvision, $e)
    {
        $tenant = $fakeProvision ?
            $this->getTenantByGroupId($mspGroupId) :
            $this->getTenantBySubscriptionId($subscriptionId);
        if (empty($tenant)) {
            throw new Exception(self::EXCEPTION_CREATE_TENANT_FAILED_MESSAGE, null, $e);
        }

        if ($tenant->ismigration == false) {
            throw new Exception(sprintf(self::EXCEPTION_SUBSCRIPTION_ALREADY_MIGRATED, $subscriptionId), null, $e);
        }

        return $tenant;
    }

    /**
     * @param $tenant
     * @param $continueStatus
     * @param $terminateStatus
     * @param $resourceUpgradeOrders
     * @throws Exception
     * @throws Service_PbaApi_Api_Exception
     */
    private function placeUpgradeOrder($tenant, $continueStatus, $terminateStatus, $resourceUpgradeOrders)
    {
        $result = $this->pbaApi->placeResourceUpgradeOrder(
            $tenant->subscription->subscriptionId,
            self::USE_PAY_TOOL,
            self::USE_DEFAULT_PAY_TOOL,
            $tenant->accountId,
            $this->fetchEndPointHost(),
            $resourceUpgradeOrders
        );
        do {
            sleep(self::DEFAULT_SLIP_TIME);
            $order = $this->pbaApi->getOrder($result->getNewOrderId());
        } while (!in_array($order->getOrderStatusID(), array_merge($continueStatus, $terminateStatus)));

        if (in_array($order->getOrderStatusID(), $terminateStatus)) {
            throw new Exception(sprintf(self::EXCEPTION_PLACE_UPGRADE_ORDER_ERROR, $tenant->subscription->subscriptionId));
        }
    }

    /**
     * @param $tenant
     * @param $counter
     * @param $abcStorageResourceId
     * @param array $compositeResourceIDs
     * @return array
     * @throws Service_PbaApi_Api_Exception
     */
    private function updateStorageCounter($tenant, $counter, $abcStorageResourceId, $compositeResourceIDs)
    {
        $resourceUpgradeOrders = array();
        $subscriptionResourceLimit = $this->pbaApi->getSubscriptionResourcesList($tenant->subscription->subscriptionId);

        list($compositeStorageIncludedAmount, $compositeStorageAdditionalAmount) = $this->processingCompositeStorageResources($subscriptionResourceLimit, $compositeResourceIDs);

        $baStorageUsage = $this->getSubscriptionResourceByName($subscriptionResourceLimit, self::BA_STORAGE_RESOURCE_NAME);
        $abcStorageUsage = $this->getSubscriptionResourceById($subscriptionResourceLimit, $abcStorageResourceId);
        list($storageIncludedAmount, $storageAdditionalAmount) = $this->getBAStorageLimits($baStorageUsage);

        $storageIncludedAmount += $compositeStorageIncludedAmount;
        $storageAdditionalAmount += $compositeStorageAdditionalAmount;

        $resourceUpgradeOrder = null;
        if ($storageAdditionalAmount > 0) {
            $additionalAmount = (double)ceil(UomConvertHelper::convert($storageAdditionalAmount, UomConvertHelper::MEASURABLE_B, $abcStorageUsage->getUnit()));
            $resourceUpgradeOrder = new Service_PbaApi_Api_Request_Struct_ResourceUpgradeOrder();
            $resourceUpgradeOrder->setResourceID($counter->id);
            $resourceUpgradeOrder->setNewResourceID($counter->id);
            $resourceUpgradeOrder->setAmount($additionalAmount);
            $resourceUpgradeOrders[] = $resourceUpgradeOrder;
        }
        $tenant->StorageUsage->limit = $storageIncludedAmount;
        return $resourceUpgradeOrders;
    }

    /**
     * @param $baStorageUsage
     * @return array
     */
    private function getBAStorageLimits($baStorageUsage)
    {
        if (!is_null($baStorageUsage)) {
            $storageIncludedAmount = ceil(UomConvertHelper::convert($baStorageUsage->getIncludedAmount(), $baStorageUsage->getUnit(), UomConvertHelper::MEASURABLE_B));
            $storageAdditionalAmount = ceil(UomConvertHelper::convert($baStorageUsage->getAdditionalAmount(), $baStorageUsage->getUnit(), UomConvertHelper::MEASURABLE_B));
        } else {
            $storageIncludedAmount = 0;
            $storageAdditionalAmount = 0;
        }
        return array($storageIncludedAmount, $storageAdditionalAmount);
    }

    /**
     * @param $tenant
     * @param Service_MspApiClient_Response_Struct_UserUsage $usageABC
     */
    private function updateMachineCounterLimit($tenant, Service_MspApiClient_Response_Struct_UserUsage $usageABC)
    {
        $tenant->VmCount->limit = $usageABC->getVmCount();
        $tenant->WorkstationCount->limit = $usageABC->getWorkstationCount();
        $tenant->ServerCount->limit = $usageABC->getServerCount();
    }

    /**
     * @param $tenant
     * @param Service_MspApiClient_Response_Struct_UserUsage $usageABC
     * @return array
     */
    private function processingNewResources($tenant, Service_MspApiClient_Response_Struct_UserUsage $usageABC)
    {
        $newResourceUpgradeOrders = array();
        $groupInfo = $this->mspApi->getGroupById($tenant->mspGroupId);
        $newCounters = $this->getCounters($tenant, $this->newCounterNames);

        $privileges = $groupInfo->getPrivileges();
        if (!is_null($privileges->getMobileCount())) {
            if (isset($newCounters[self::MOBILE_DEVICE_COUNTER])) {
                $tenant->MobileDeviceCount->limit = $usageABC->getMobileCount();
                $newOrders = $this->updateCounter(
                    $tenant,
                    $newCounters[self::MOBILE_DEVICE_COUNTER],
                    self::MOBILE_DEVICE_COUNTER
                );
                $newResourceUpgradeOrders = array_merge($newResourceUpgradeOrders, $newOrders);
            } else {
                $tenant->MobileDeviceCount->limit = 0;
            }
        }

        if (!is_null($privileges->getMailboxCount())) {
            if (isset($newCounters[self::MAILBOX_COUNTER])) {
                $tenant->MailboxCount->limit = $usageABC->getMailboxCount();
                $newOrders = $this->updateCounter(
                    $tenant,
                    $newCounters[self::MAILBOX_COUNTER],
                    self::MAILBOX_COUNTER
                );
                $newResourceUpgradeOrders = array_merge($newResourceUpgradeOrders, $newOrders);
            } else {
                $tenant->MailboxCount->limit = 0;
            }
        }

        if (!is_null($privileges->getWebsiteCount())) {
            if (isset($newCounters[self::WEBSITE_COUNTER])) {
                $tenant->WebsiteCount->limit = $usageABC->getWebsiteCount();
                $newOrders = $this->updateCounter(
                    $tenant,
                    $newCounters[self::WEBSITE_COUNTER],
                    self::WEBSITE_COUNTER
                );
                $newResourceUpgradeOrders = array_merge($newResourceUpgradeOrders, $newOrders);
            } else {
                $tenant->WebsiteCount->limit = 0;
            }
        }

        return $newResourceUpgradeOrders;
    }

    /**
     * @param $tenant
     * @param $counter
     * @param $counterName
     * @return array
     */
    private function updateCounter($tenant, $counter, $counterName)
    {
        $resourceUpgradeOrders = array();

        switch (true) {
            case !isset($counter->limit):
                $tenant->$counterName->limit = self::UNLIMITED_QUOTA_VALUE;
                break;
            case ($tenant->$counterName->limit > $counter->limit):
                $additionalAmount = (double)($tenant->$counterName->limit - $counter->limit);
                $resourceUpgradeOrder = new Service_PbaApi_Api_Request_Struct_ResourceUpgradeOrder();
                $resourceUpgradeOrder->setResourceID($counter->id);
                $resourceUpgradeOrder->setNewResourceID($counter->id);
                $resourceUpgradeOrder->setAmount($additionalAmount);
                $resourceUpgradeOrders[] = $resourceUpgradeOrder;
                break;
            case ($tenant->$counterName->limit <= $counter->limit):
                $tenant->$counterName->limit = $counter->limit;
                break;
        }

        return $resourceUpgradeOrders;
    }

    private function processingCompositeStorageResources($subscriptionResourceLimit, $compositeResourceIDs)
    {
        $includedAmount = 0;
        $additionalAmount = 0;

        $compositeResources = $this->getCompositeResources($subscriptionResourceLimit, $compositeResourceIDs);

        foreach ($compositeResources as $compositeResource) {
            $includedResourceInfo = $this->pbaApi->getIncludedResources($compositeResource->getResourceId());
            list($compositeStorageIncludedAmount, $compositeStorageAdditionalAmount) = $this->getStorageFromIncludedResources($includedResourceInfo, $compositeResource);
            $includedAmount += $compositeStorageIncludedAmount;
            $additionalAmount += $compositeStorageAdditionalAmount;
        }

        return array($includedAmount, $additionalAmount);
    }

    /**
     * @param Service_PbaApi_Api_Response_Struct_GetIncludedResources[] $includedResourceInfo
     * @param Service_PbaApi_Api_Response_Struct_SubscriptionResourcesList $compositeResource
     * @return array
     */
    private function getStorageFromIncludedResources($includedResourceInfo, $compositeResource)
    {
        $includedAmount = 0;
        $additionalAmount = 0;
        foreach ($includedResourceInfo as $resource) {
            if ($resource->getIncludedResourceName() == self::BA_STORAGE_RESOURCE_NAME) {
                $includedAmount += ceil(UomConvertHelper::convert(
                    $compositeResource->getIncludedAmount() * $resource->getMultiplier(),
                    $resource->getIncludedResourceMeasurementUnitName(),
                    UomConvertHelper::MEASURABLE_B
                ));

                $additionalAmount += ceil(UomConvertHelper::convert(
                    $compositeResource->getAdditionalAmount() * $resource->getMultiplier(),
                    $resource->getIncludedResourceMeasurementUnitName(),
                    UomConvertHelper::MEASURABLE_B
                ));
            }
        }

        return array($includedAmount, $additionalAmount);
    }

    /**
     * @param $subscriptionResourceLimit
     * @param $compositeResourceIDs
     * @return array
     * @throws Service_PbaApi_Api_Exception
     */
    private function getCompositeResources($subscriptionResourceLimit, $compositeResourceIDs)
    {
        $compositeResources = array();
        /** @var Service_PbaApi_Api_Response_Struct_SubscriptionResourcesList $resource */
        foreach ($subscriptionResourceLimit as $resource) {
            if (empty($compositeResourceIDs)) {
                $resourceInfo = $this->pbaApi->getResource($resource->getResourceId());
                if ($resourceInfo->getResourceType() === self::COMPOSITE_RESOURCE_TYPE_VALUE) {
                    $compositeResources[] = $resource;
                }
            } else {
                if (in_array($resource->getResourceId(), $compositeResourceIDs)) {
                    $resourceInfo = $this->pbaApi->getResource($resource->getResourceId());
                    if ($resourceInfo->getResourceType() === self::COMPOSITE_RESOURCE_TYPE_VALUE) {
                        $compositeResources[] = $resource;
                    }
                }
            }
        }
        return $compositeResources;
    }

    private function getSubscriptionResourceLimit($subscriptionResources, $property)
    {
        foreach ($subscriptionResources as $resource) {
            if (!isset($resource->apsType)) {
                continue;
            }
            if (isset($resource->property)
                && $resource->property == $property
                && $resource->apsType == \APS\TypeLibrary::getTypeByServiceId('tenant', $this->apsPackageId)
            ) {
                return isset($resource->limit) ? $resource->limit : self::UNLIMITED_QUOTA_VALUE;
            }
        }
        return 0;
    }
}
