SELECT
  CASE WHEN gr."Value" IS NULL
    THEN cr."CompanyName"
  ELSE gr."Value" END      AS "ReportGroup",
  br."VendorAccountID"     AS "CRID",
  cr."CompanyName"         AS "CRName",
  CASE WHEN be."Value" IS NOT NULL
    THEN be."Value"
  ELSE v."CompanyName" END AS "CRVendor",
  cr."AAN"                 AS "CRAAN",
  cr."ContactEmail"        AS "CREmail",
  cr."BaseDiscount"        AS "CRBDisc",
  cr."T1Support"           AS "CRT1Disc",
  br."CurrencyID"          AS "CRCurrency",
  SUM(br."ExtendedPrice")  AS "Overall",
  round(
      SUM(br."ExtendedPrice") * (1 - (cr."BaseDiscount" + cr."T1Support")),
      br."CurrencyPrecision"
  )                        AS "OverallDiscount"
FROM
  "AcronisServiceProviderBreakdown" AS br
  JOIN
  (
    SELECT
      a."AccountID",
      a."CompanyName",
      a."VendorAccountID",

      CASE WHEN a."AdminEmail" = a."BillEmail"
        THEN a."AdminEmail"
      ELSE a."AdminEmail" || ', ' || a."BillEmail" END               AS "ContactEmail",

      CASE WHEN t1."Value" = '1'
        THEN 0.10
      ELSE 0.0 END                                                   AS "T1Support",

      aan."Value"                                                    AS "AAN",

      CASE
      WHEN bd."Value" IS NULL
        THEN 0.0
      ELSE CAST(TRIM(BOTH '%' FROM bd."Value") AS DECIMAL) / 100 END AS "BaseDiscount"
    FROM
      "Account" a
      JOIN "AccAttr" AS aan ON aan."ObjID" = a."AccountID" AND aan."ATID" = 'AAN'
      LEFT JOIN "AccAttr" AS t1 ON t1."ObjID" = a."AccountID" AND t1."ATID" = 'T1Support'
      LEFT JOIN "AccAttr" AS bd ON bd."ObjID" = a."AccountID" AND bd."ATID" = 'BaseDiscount'
  ) AS cr ON cr."AccountID" = br."VendorAccountID"
  JOIN "Account" AS v ON v."AccountID" = cr."VendorAccountID"
  LEFT JOIN "AccAttr" AS be ON be."ObjID" = cr."AccountID" AND be."ATID" = 'BillingEntity'
  LEFT JOIN "AccAttr" AS gr ON gr."ObjID" = br."VendorAccountID" AND gr."ATID" = 'ReportGroup'
WHERE
  br."ResourceSKU" IS NOT NULL
GROUP BY
  "ReportGroup",
  "CRID",
  "CRName",
  "CRVendor",
  "CRAAN",
  "CREmail",
  "CRBDisc",
  "CRT1Disc",
  "CRCurrency",
  br."CurrencyPrecision"
ORDER BY
  "CRName",
  "CRVendor",
  "CRCurrency",
  "CRAAN",
  "CREmail",
  "CRBDisc",
  "CRT1Disc",
  "Overall",
  "OverallDiscount";