SELECT
  CASE WHEN gr."Value" IS NULL
    THEN cr."CompanyName"
  ELSE gr."Value" END                                    AS "ReportGroup",
  br."CompanyName"                                       AS "SPName",
  br."AAN"                                               AS "SPAAN",
  cr."CompanyName"                                       AS "CRName",
  aan."Value"                                            AS "CRAAN",
  br."AccountID"                                         AS "SPID",
  br."subscriptionID"                                    AS "SubscriptionID",
  br."PlanName"                                          AS "PlanName",
  br."ContactEmail"                                      AS "SPEmail",
  br."LType"                                             AS "LType",
  trim(LEADING 'en ' FROM split_part(c."Name", chr(9), 1)) AS "SPCountry",
  ''                                                     AS "SPRegion",
  br."DatacenterID"                                      AS "SPDatacenter",
  br."resourceID"                                        AS "ResourceID",
  br."resourceName"                                      AS "ResourceName",
  br."ResourceSKU"                                       AS "ResourceSKU",
  br."CurrencyID"                                        AS "SPCurrency",
  br."StatusName"                                        AS "StatusName",
  mc."Value"                                             AS "MinCom",
  mc_tier."Value"                                        AS "MinComTier",
  mc_month."Value"                                       AS "MinComMonth",
  to_char(to_timestamp(br."StartDate"), 'MM/DD/YYYY')    AS "Start",
  to_char(to_timestamp(br."EndDate"), 'MM/DD/YYYY')      AS "End",
  CASE WHEN cls."Value" IS NOT NULL
    THEN to_char(to_timestamp(cls."Value" :: INT), 'MM/DD/YYYY')
  ELSE '' END                                            AS "Close",
  br."ServQty"                                           AS "ResourceQuantity",
  CASE WHEN wsd."Value" = '1'
    THEN 'yes'
  ELSE '' END                                            AS "WSDiscount",
  br."Discount"                                          AS "Discount",
  br."UnitPrice"                                         AS "ResourcePrice",
  br."UnitPriceWithDiscount"                             AS "ResourcePriceWithDiscount",
  br."ExtendedPrice"                                     AS "Amount"
FROM
  "AcronisServiceProviderBreakdown" AS br
  JOIN "Account" AS cr ON cr."AccountID" = br."VendorAccountID"
  JOIN "AccAttr" AS aan ON aan."ObjID" = br."VendorAccountID" AND aan."ATID" = 'AAN'
  LEFT JOIN "AccAttr" AS mc ON mc."ObjID" = br."AccountID" AND mc."ATID" = 'MinCom'
  LEFT JOIN "AccAttr" AS mc_tier ON mc_tier."ObjID" = br."AccountID" AND mc_tier."ATID" = 'MinComTier'
  LEFT JOIN "AccAttr" AS mc_month ON mc_month."ObjID" = br."AccountID" AND mc_month."ATID" = 'MinComMonth'
  LEFT JOIN "SubscrParamValue" AS wsd
    ON wsd."subscriptionID" = br."subscriptionID" AND wsd."IdParameter" = 'WS_Discount'
  LEFT JOIN "SubscrParamValue" AS cls
      ON cls."subscriptionID" = br."subscriptionID" AND cls."IdParameter" = 'CloseDate'
  JOIN "Country" AS c ON c."CountryID" = br."CountryID"
  LEFT JOIN "AccAttr" AS gr ON gr."ObjID" = br."VendorAccountID" AND gr."ATID" = 'ReportGroup'
WHERE
  br."ResourceSKU" IS NOT NULL
ORDER BY
  "SPName",
  "LType",
  "SPDatacenter",
  "StatusName",
  "Start",
  "End",
  "ResourceSKU";
