CREATE TEMPORARY VIEW "AcronisServiceProviderBreakdown" AS
  SELECT
    sp."AccountID",
    sp."CompanyName",
    sp."VendorAccountID",
    sp."ContactEmail",
    sp."AAN",
    sp."CurrencyID",
    sp."CurrencyPrecision",
    sp."CountryID",
    s."subscriptionID",
    CASE WHEN s."Status" = 15
      THEN 'Trial'
    WHEN s."Status" = 30
      THEN 'Production'
    WHEN s."Status" = 40
      THEN 'Trial (Graced)'
    WHEN s."Status" = 50
      THEN 'Trial (Expired)'
    WHEN s."Status" = 60 AND s."PeriodType" = 1
      THEN 'Trial (Terminated)'
    WHEN s."Status" = 60 AND s."PeriodType" = 3
      THEN 'Production (Churn)'
    WHEN s."Status" IN (80, 85, 89) AND s."PeriodType" = 1
      THEN 'Trial (Hold)'
    ELSE 'Production (Hold)' END                         AS "StatusName",
    s."StartDate",
    s."EndDate",
    s."DatacenterID",
    CASE WHEN o."Descr" = 'Minimum commitment'
      THEN 0.0
    ELSE s."Discount"
    END                                                  AS "Discount",
    trim(LEADING 'en ' FROM split_part(p.name, chr(9), 1)) AS "PlanName",
    CASE WHEN p."name" LIKE '%Per Device%'
      THEN 'Per Device'
    WHEN p."name" LIKE '%Per GB%'
      THEN 'Per GB'
    ELSE NULL
    END                                                  AS "Product",
    CASE WHEN p."name" LIKE '%Per Device%'
      THEN 'SPQAMSENS'
    WHEN p."name" LIKE '%Per GB%'
      THEN 'SPQBMSENS'
    ELSE NULL
    END                                                  AS "SKU",
    pc."name"                                            AS "LType",
    od."resourceID",
    CASE WHEN o."Descr" = 'Minimum commitment'
      THEN
        CASE WHEN p."name" LIKE '%Per Device%'
          THEN 'Per Device - Aggregate'
        ELSE 'Per GB - Aggregate' END
    ELSE
      abr."resourceName"
    END                                                  AS "resourceName",

    CASE WHEN o."Descr" = 'Minimum commitment'
      THEN
        CASE WHEN p."name" LIKE '%Per Device%'
          THEN 'SPQAMSENS'
        WHEN p."name" LIKE '%Per GB%'
          THEN 'SPQBMSENS'
        ELSE NULL
        END
    ELSE od."SKU" END                                    AS "ResourceSKU",
    od."ServQty",
    CASE WHEN s."PeriodType" = 1
      THEN 0.0
    ELSE od."UnitPrice_Value"
    END                                                  AS "UnitPrice",
    CASE WHEN s."PeriodType" = 1
      THEN 0.0
    WHEN o."Descr" = 'Minimum commitment'
      THEN round(od."UnitPrice_Value", sp."CurrencyPrecision" + 2)
    ELSE round(od."UnitPrice_Value" * (1 - s."Discount"), sp."CurrencyPrecision" + 2)
    END                                                  AS "UnitPriceWithDiscount",
    CASE WHEN s."PeriodType" = 1
      THEN 0.0
    WHEN o."Descr" = 'Minimum commitment'
      THEN round(od."Duration" * od."ServQty" * od."UnitPrice_Value", sp."CurrencyPrecision")
    ELSE round(od."Duration" * od."ServQty" * od."UnitPrice_Value" * (1 - s."Discount"), sp."CurrencyPrecision")
    END                                                  AS "ExtendedPrice",

    od."Duration",
    od."DetSDate",
    od."DetEDate"

  FROM (
         SELECT
           a."AccountID",
           a."CompanyName",
           a."VendorAccountID",
           a."BillEmail"             AS "ContactEmail",
           a."CountryID",
           aan."Value"               AS "AAN",
           a."AccCurrencyCurrencyID" AS "CurrencyID",
           CASE WHEN a."AccCurrencyCurrencyID" IN ('AOA', 'CNY', 'HKD', 'MOP', 'TWD')
             THEN 1
           WHEN a."AccCurrencyCurrencyID" IN ('BHD', 'JOD', 'KWD', 'LYD', 'OMR', 'TND')
             THEN 3
           WHEN a."AccCurrencyCurrencyID" IN (
             'AMD', 'BIF', 'BYR', 'CLP', 'COP', 'DJF', 'GNF', 'HUF', 'IDR', 'IQD', 'IRR', 'ISK', 'JPY', 'KHR', 'KMF', 'KPW',
             'KRW', 'LAK', 'LBP', 'MMK', 'PYG', 'RWF', 'SLL', 'STD', 'UGX', 'VND', 'VUV', 'XAF', 'XOF', 'XPF', 'YER', 'ZMK'
           )
             THEN 0
           ELSE 2 END                AS "CurrencyPrecision"

         FROM
           "Account" a
           JOIN "AccAttr" AS aan ON aan."ObjID" = a."AccountID" AND aan."ATID" = 'AAN'
         WHERE
           a."Type" = 3 AND aan."Value" NOT IN ('Acronis', '00000000')
       ) AS sp
    JOIN (
           SELECT
             s."AccountID",
             s."subscriptionID",
             s."Status",

             CASE WHEN sd."Value" IS NULL
               THEN
                 s."startDate"
             ELSE
               sd."Value" :: INT END                                      AS "StartDate",

             CASE WHEN ed."Value" IS NULL
               THEN
                 s."ExpirationDate"
             ELSE
               ed."Value" :: INT END                                      AS "EndDate",

             s."LastBillDate",
             s."PlanID",
             s."PeriodType",
             dc."Value"                                                   AS "DatacenterID",

             CASE WHEN dsc."Value" IS NULL
               THEN
                 0.0
             ELSE
               CAST(TRIM(BOTH '%' FROM dsc."Value") AS DECIMAL) / 100 END AS "Discount"

           FROM "Subscription" s
             JOIN "SubscrParamValue" AS dc
               ON dc."subscriptionID" = s."subscriptionID" AND dc."IdParameter" = 'Datacenter'
             LEFT JOIN "SubscrParamValue" AS dsc
               ON dsc."subscriptionID" = s."subscriptionID" AND dsc."IdParameter" = 'Discount'
             LEFT JOIN "SubscrParamValue" AS exc
               ON exc."subscriptionID" = s."subscriptionID" AND exc."IdParameter" = 'ExcludeFromReport'
             LEFT JOIN "SubscrParamValue" AS sd
               ON sd."subscriptionID" = s."subscriptionID" AND sd."IdParameter" = 'StartDate'
             LEFT JOIN "SubscrParamValue" AS ed
               ON ed."subscriptionID" = s."subscriptionID" AND ed."IdParameter" = 'EndDate'
           WHERE
               exc."Value" IS NULL OR exc."Value" = '' OR exc."Value" = '0'
         ) AS s ON s."AccountID" = sp."AccountID"
    JOIN "Plan" AS p ON p."PlanID" = s."PlanID"
    JOIN "PlanCategory" AS pc ON pc."planCategoryID" = p."planCategoryID"
    JOIN "SalesOrder" AS o ON o."Customer_AccountID" = sp."AccountID"
    JOIN "OrdDet" AS od ON od."OrderID" = o."OrderID" AND od."subscriptionID" = s."subscriptionID"
    JOIN "AcronisBillableResources" AS abr ON abr."resourceId" = od."resourceID"
  WHERE
    ((
      o."OrderTypeID" IN ('CF', 'CH', 'BO') AND
      od."DetTypeID" = 140
    ) OR
    (
      o."OrderTypeID" = 'BO' AND
      o."Descr" = 'Minimum commitment'
    )) AND
    od."DetSDate" >= extract(EPOCH FROM date_trunc('month', now()) - INTERVAL '1 month') AND
    od."DetEDate" <= extract(EPOCH FROM date_trunc('month', now()))
;