<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require_once(__DIR__ . "/../app/bootstrap.php");

define('IMPORTER_DIR_ROOT', __DIR__ . DS . 'import_external_accounts' . DS);
define('IMPORTER_DIR_SRC', IMPORTER_DIR_ROOT . 'src' . DS);
define('IMPORTER_DIR_CONFIG', IMPORTER_DIR_ROOT . 'config' . DS);
define('IMPORTER_CONFIG_NAME', IMPORTER_DIR_CONFIG . 'import.ini');

require_once(IMPORTER_DIR_SRC . 'ExternalAccountsImporter.php');

chdir(DIR_ROOT);
define('APS_RUNTIME_HOME', DIR_ROOT);
define('APS_RUNTIME_NO_PROCESS', true);

use Monolog\Formatter\LineFormatter;
use Monolog\Handler\StreamHandler;
use Monolog\Logger;

$output = Log::FIELD_TIME . ' ' . Log::FIELD_REQUEST . ' ' . Log::FIELD_TYPE . ' ' . Log::FIELD_MESSAGE . PHP_EOL;
$formatter = new LineFormatter($output);

$stream = new StreamHandler('php://stdout', Logger::NOTICE);
$stream->setFormatter($formatter);
Log::getInstance()->getLogger()->pushHandler($stream);

try {
    if (!file_exists(IMPORTER_CONFIG_NAME) || !is_readable(IMPORTER_CONFIG_NAME)) {
        throw new \Exception(Str::format('Unable to read the configuration file "%s".', IMPORTER_CONFIG_NAME));
    }

    $config = parse_ini_file(IMPORTER_CONFIG_NAME);

    $settings = array('application_instance_id', 'http_aps_instance_id', 'data_path');
    foreach($settings as $setting) {
        if (empty($config[$setting])) {
            throw new \Exception(Str::format(
                'Please specify the parameter "%s" in the configuration file "%s".',
                $setting,
                IMPORTER_CONFIG_NAME
            ));
        }
    }

    $importDataPath = $config['data_path'];
    if (!file_exists($importDataPath) || !is_readable($importDataPath)) {
        throw new \Exception("Unable to read data for import from \"{$importDataPath}\".");
    }

    $accountsImportData = json_decode(file_get_contents($importDataPath), true);
    if (is_null($accountsImportData)) {
        throw new \Exception("Unable to parse json from \"{$importDataPath}\".");
    }

    $importer = new ExternalAccountsImporter(
        new ApiFactory(),
        (string) $config['http_aps_instance_id'],
        (int) $config['application_instance_id']
    );
    $importer->import($accountsImportData);
} catch (Exception $e) {
    Log::getInstance()->error($e->getMessage());
    exit(1);
}
