<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require_once(__DIR__ . "/../app/bootstrap.php");
require_once(__DIR__ . "/request_usage_report/src/ReportHelper.php");

chdir(DIR_ROOT);
define('APS_RUNTIME_HOME', DIR_ROOT);
define('APS_RUNTIME_NO_PROCESS', true);

use Monolog\Formatter\LineFormatter;
use Monolog\Handler\StreamHandler;
use Monolog\Logger;

const OUTPUT_OPTION_UNDEFINED = "Option '%s' undefined \n";

const OPTION_PARSE_CHAR = ':';
const OPTION_APPLICATION = 'a';
const OPTION_DRY_RUN_MODE = 'd';
const OPTION_HELP = 'h';

/**
 *  Prints usage to the screen
 */
function printScriptUsage()
{
    $fileName = basename($_SERVER['PHP_SELF']);
    echo <<<USAGE
Usage: php {$fileName} [OPTIONS]
Example: php {$fileName} -a 10
Options:
  -a APPLICATION    Acronis Backup Service application ID.
  -d                Dry run mode. Run script without request to Acronis Backup Cloud.
USAGE;
}

function validateCommandLineParameters($opt)
{
    if (isset($opt[OPTION_HELP])) {
        return true;
    }

    if (!isset($opt[OPTION_APPLICATION]) || !ctype_digit($opt[OPTION_APPLICATION])) {
        return false;
    }

    if (isset($opt[OPTION_DRY_RUN_MODE])) {
        return true;
    }

    return true;
}

/**
 * @param string $optionName
 * @param array $options
 * @return mixed | null
 */
function getOptions($optionName, $options)
{
    $option = null;
    switch ($optionName) {
        case OPTION_HELP:
            $option = isset($options[OPTION_HELP]);
            break;
        case OPTION_APPLICATION:
            $option = intval($options[OPTION_APPLICATION]);
            break;
        case OPTION_DRY_RUN_MODE:
            $option = isset($options[OPTION_DRY_RUN_MODE]) ? true : false;
            break;
        default:
            echo sprintf(OUTPUT_OPTION_UNDEFINED, $optionName);
            printScriptUsage();
            exit;
    }
    return $option;
}

$opt = getopt(
    OPTION_APPLICATION . OPTION_PARSE_CHAR .
    OPTION_DRY_RUN_MODE .
    OPTION_HELP
);

if (!validateCommandLineParameters($opt)) {
    printScriptUsage();
    exit(1);
};

if (getOptions(OPTION_HELP, $opt)) {
    printScriptUsage();
    exit;
}

$applicationId = getOptions(OPTION_APPLICATION, $opt);
$dryRunMode = getOptions(OPTION_DRY_RUN_MODE, $opt);

$output = Log::FIELD_TIME . ' ' . Log::FIELD_REQUEST . ' ' . Log::FIELD_TYPE . ' ' . Log::FIELD_MESSAGE . PHP_EOL;
$formatter = new LineFormatter($output);

$stream = new StreamHandler('php://stdout', Logger::NOTICE);
$stream->setFormatter($formatter);
Log::getInstance()->getLogger()->pushHandler($stream);

try {
    $apiFactory = new ApiFactory();
    $helper = new ReportHelper($apiFactory, $applicationId, $dryRunMode);
    $helper->process();
} catch (Exception $e) {
    Log::getInstance()->error($e->getMessage());
    exit(1);
}
