<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require_once(__DIR__ . '/app/bootstrap.php');

/**
 * Class tenant
 * @type("http://acronis.com/aps/cloud-backup-v1/tenant/9.2")
 * @implements("http://aps-standard.org/types/core/resource/1.0")
 * @implements("http://aps-standard.org/types/core/suspendable/1.0")
 */
class tenant extends APSResource
{
    const STORAGE_USAGE = 'StorageUsage';
    const STORAGE_USAGE_MBH = 'StorageUsageMbH';
    const ACRONIS_STORAGE_USAGE = 'AcronisHostedStorage';
    const ACRONIS_STORAGE_USAGE_MBH = 'AcronisHostedStorageMbH';
    const PROVIDER_STORAGE_USAGE = 'ServiceProviderHostedStorage';
    const PROVIDER_STORAGE_USAGE_MBH = 'ServiceProviderHostedStorageMbH';
    const WORKSTATION_COUNT = 'WorkstationCount';
    const WORKSTATION_COUNT_ITEM = 'WorkstationCountItemH';
    const SERVER_COUNT = 'ServerCount';
    const SERVER_COUNT_ITEMH = 'ServerCountItemH';
    const VM_COUNT = 'VmCount';
    const VM_COUNT_ITEMH = 'VmCountItemH';
    const MOBILE_DEVICE_COUNT = 'MobileDeviceCount';
    const MOBILE_DEVICE_COUNT_ITEM = 'MobileDeviceCountItemH';
    const MAILBOX_COUNT = 'MailboxCount';
    const MAILBOX_COUNT_ITEMH = 'MailboxCountItemH';
    const WEBSITE_COUNT = 'WebsiteCount';

    const UNLIMITED_VALUE = -1;

    const ON_USER_CHANGED = 'onUserChanged';
    const ON_ACCOUNT_CHANGE = 'onAccountChange';
    const ON_SUBSCRIPTION_CHANGE = 'onSubscriptionChange';
    const ON_TENANT_AVAILABLE = "onTenantAvailable";

    const LOG_CALLED_UPDATE_COUNTER_LIMITS = 'Update counter limits for tenant was called.';
    const LOG_CALLED_GET_PARTNER_CREDENTIALS = 'Get partner credentials for tenant was called.';
    const LOG_CALLED_UPGRADE = 'Tenant (%s) upgrade was called';
    const LOG_CALLED_LINK_DATACENTER = 'Tenant (%s) link datacenter was called';

    const ERROR_FAILED_LOCK_GROUP = 'Failed to lock group id: %s';

    const EXCEPTION_CREATING_TENANT_MESSAGE = 'Creating tenant (%s) for subscription (%s)';
    const EXCEPTION_MORE_THAN_1_COMMITMENT_MESSAGE = 'Only one minimum commitment resource can be included into a service plan.';
    const EXCEPTION_RESOURCE_TYPE_MUST_BE_REFERENCE_MESSAGE = 'Resource type "%s" must be a reference.';

    const LINK_NAME_PROFILE = 'profile';
    const LINK_NAME_DATACENTER = 'datacenter';

    const TYPE_RESELLER = 'RESELLER';
    const TYPE_CUSTOMER = 'CUSTOMER';
    const TYPE_PARTNER = 'PARTNER';

    const LOG_METHOD_CALL = 'Resource: %s Account: %s Subscription: %s Call: %s';

    /* ======= RESOURCE PROPERTIES ======= */

    /**
     * @link("http://acronis.com/aps/cloud-backup-v1/globals/9.0")
     * @required
     */
    public $global;

    /**
     * @link("http://aps-standard.org/types/core/subscription/1.0")
     * @required
     */
    public $subscription;

    /**
     * @link("http://aps-standard.org/types/core/account/1.0")
     * @required
     */
    public $account;

    /**
     * @link("http://acronis.com/aps/cloud-backup-v1/users/7.0[]")
     */
    public $users;

    /**
     * @link("http://acronis.com/aps/cloud-backup-v1/admin/4.0[]")
     * @deprecated
     */
    public $admins;

    /**
     * @link("http://acronis.com/aps/cloud-backup-v1/backup-profile/5.0")
     */
    public $profile;

    /**
     * @link("http://acronis.com/aps/cloud-backup-v1/datacenter/4.0")
     */
    public $datacenter;

    /**
     * @link("http://acronis.com/aps/cloud-backup-v1/minimal-commitment-tier/4.0")
     */
    public $minimalCommitmentTier;

    /**
     * @type(integer)
     * @readonly
     */
    public $minimalCommitmentDiscountId;

    /**
     * @type(integer)
     * @readonly
     */
    public $startProductionTime;

    /**
     * @link("http://aps-standard.org/types/core/admin-user/1.0[]");
     * @deprecated
     */
    public $adminusers;

    /**
     * @type(integer)
     * @title("Account ID")
     * @readonly
     */
    public $accountId;

    /**
     * @type(string)
     * @title("Subscription ID")
     * @readonly
     */
    public $subscriptionId;

    /**
     * @type(StorageRelation[])
     */
    public $storageRelations = array();

    /**
     * @type(string)
     */
    public $datacenterId;

    /**
     * @type(integer)
     */
    public $storageId;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Storage size for customer")
     * @unit("b")
     */
    public $StorageUsage;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Own Cloud Storage usage")
     * @unit("b")
     */
    public $AcronisHostedStorage;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Service Provider Cloud Storage usage")
     * @unit("b")
     */
    public $ServiceProviderHostedStorage;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Server count for customer")
     * @unit("unit")
     */
    public $ServerCount;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("VM count for customer")
     * @unit("unit")
     */
    public $VmCount;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Workstation count for customer")
     * @unit("unit")
     */
    public $WorkstationCount;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("The number of mobile devices")
     * @unit("unit")
     */
    public $MobileDeviceCount;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("The number of Windows servers essentials")
     * @unit("unit")
     * @deprecated
     */
    public $WindowsServerEssentialsCount;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("The number of virtual hosts")
     * @unit("unit")
     * @deprecated
     */
    public $VirtualHostsCount;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("The number of Mailbox")
     * @unit("unit")
     */
    public $MailboxCount;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("The number of websites")
     * @unit("unit")
     */
    public $WebsiteCount;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Storage size for customer in hour")
     * @unit("mb-h")
     */
    public $StorageUsageMbH;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Own Cloud Storage usage in hour")
     * @unit("mb-h")
     */
    public $AcronisHostedStorageMbH;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Service Provider Cloud Storage usage in hour")
     * @unit("mb-h")
     */
    public $ServiceProviderHostedStorageMbH;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Server count for customer in hour")
     * @unit("item-h")
     */
    public $ServerCountItemH;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("VM count for customer in hour")
     * @unit("item-h")
     */
    public $VmCountItemH;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Workstation count for customer in hour")
     * @unit("item-h")
     */
    public $WorkstationCountItemH;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("The number of mobile devices in hour")
     * @unit("item-h")
     */
    public $MobileDeviceCountItemH;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("The number of Windows servers essentials in hour")
     * @unit("item-h")
     * @deprecated
     */
    public $WindowsServerEssentialsCountItemH;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("The number of virtual hosts in hour")
     * @unit("item-h")
     * @deprecated
     */
    public $VirtualHostsCountItemH;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("The number of Mailbox in hour")
     * @unit("item-h")
     */
    public $MailboxCountItemH;

    /**
     * @type(integer)
     */
    public $mspGroupId;

    /**
     * @type(boolean)
     * @title("Is Migration")
     */
    public $ismigration = false;

    /**
     * @type(string)
     */
    public $type;

    /* ======= OTHER PROPERTIES ======= */

    /**
     * @var array
     */
    private $_subscriptionResources;

    /* ======= RESOURCE METHODS ======= */

    public function provision()
    {
        $this->accountId = $this->account->id;
        $this->subscriptionId = $this->subscription->subscriptionId;

        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));

        $this->initializeTenantType();

        $this->getTenantStrategy()->provision();

        $this->throwRestAccepted();
    }

    public function provisionAsync()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->provisionAsync();

        $this->_subscribeToOnTenantAvailableEvent();
    }

    /**
     * @param tenant $new
     */
    public function configure($new)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        // We don't use our custom updateCounterLimits() method because in OSA 7.0 it does not work.
        // In OSA 7.0 when configure is called subscription contains resources with old limits.
        parent::configure($new);
        $this->getTenantStrategy()->configure($new);
    }

    public function unprovision()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->unprovision();
    }

    /**
     * We define operation for enable
     * @verb(PUT)
     * @path("/enable")
     *
     * @param()
     */
    public function enable()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->enable();
    }

    /**
     * We define operation for disable
     * @verb(PUT)
     * @path("/disable")
     *
     * @param()
     */
    public function disable()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->disable();
    }

    public function retrieve()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->retrieve();
    }

    public function upgrade()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));

        $this->_removeAdmins();

        $this->aps->type = $this->getSelfApsType();
        if (is_null($this->accountId)) {
            $this->accountId = $this->account->id;
        }
        if (is_null($this->subscriptionId)) {
            $this->subscriptionId = $this->subscription->subscriptionId;
        }
        if (is_null($this->type)) {
            $this->initializeTenantType();
        }

        // remove deprecated properties
        if (isset($this->isReseller)) {
            unset($this->isReseller);
        }
        if (isset($this->TENANTID)) {
            unset($this->TENANTID);
        }
        if (isset($this->GROUPPASS)) {
            unset($this->GROUPPASS);
        }
        if (isset($this->branding)) {
            unset($this->branding);
        }
        if (isset($this->unlimitedQuotas)) {
            unset($this->unlimitedQuotas);
        }
        if (isset($this->isExternalGroup)) {
            unset($this->isExternalGroup);
        }

        $this->getTenantStrategy()->upgrade();
    }

    /* ======= RESOURCE EVENTS ======= */

    /**
     * @verb(POST)
     * @path("/onTenantAvailable")
     *
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onTenantAvailable($event)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->onTenantAvailable($event);
        $this->_unsubscribeFromOnTenantAvailableEvent();
    }

    /**
     * @verb(POST)
     * @path("/onSubscriptionChange")
     *
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onSubscriptionChange($event)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->onSubscriptionChange($event);
    }

    /**
     * @verb(POST)
     * @path("/onAccountChange")
     *
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onAccountChange($event)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->onAccountChange($event);
    }

    /**
     * @verb(POST)
     * @path("/onUserChanged")
     *
     * @param("http://aps-standard.org/types/core/resource/1.0#Notification",body)
     */
    public function onUserChanged($event)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->onUserChanged($event);
    }

    /* ======= RESOURCE OPERATIONS ======= */

    /**
     * @verb(POST)
     * @path("/administrators")
     * @param(object,body)
     * @param(object,query)
     */
    public function createAdministrator($parameters, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->createAdministrator($parameters, $datacenterId);
    }

    /**
     * @verb(GET)
     * @path("/administrators")
     */
    public function getAdministrators()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->getAdministrators();
    }

    /**
     * @verb(GET)
     * @path("/administrators/{adminId}")
     * @param(string,path)
     * @param(string,query)
     *
     * @throws \Rest\RestException
     * @return stdClass
     */
    public function getAdministrator($adminId, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->getAdministrator($adminId, $datacenterId);
    }

    /**
     * @verb(PUT)
     * @path("/administrators/{adminId}")
     * @param(string,path)
     * @param(object,body)
     * @param(string,query)
     *
     * @throws \Rest\RestException
     */
    public function updateAdministrator($adminId, $parameters, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->updateAdministrator($adminId, $parameters, $datacenterId);
    }

    /**
     * @verb(DELETE)
     * @path("/administrators/{adminId}")
     * @param(string,path)
     * @param(string,query)
     *
     * @throws \Rest\RestException
     */
    public function deleteAdministrator($adminId, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->deleteAdministrator($adminId, $datacenterId);
    }

    /**
     * @verb(GET)
     * @path("/administrators/{adminId}/console")
     * @param(string,path)
     * @param(string,query)
     *
     * @throws \Rest\RestException
     * @return stdClass
     */
    public function getAdministratorConsole($adminId, $datacenterId)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->getAdministratorConsole($adminId, $datacenterId);
    }

    /**
     * @verb(PUT)
     * @path("/administrators/{adminId}/resend")
     * @param(string,path)
     * @param(string,query)
     *
     * @throws \Rest\RestException
     */
    public function resendAdministratorLink($adminId, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->resendAdministratorLink($adminId, $datacenterId);
    }

    /**
     * @verb(PUT)
     * @path("/administrators/{adminId}/enable")
     * @param(string,path)
     * @param(string,query)
     *
     * @throws \Rest\RestException
     */
    public function enableAdministrator($adminId, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->enableAdministrator($adminId, $datacenterId);
    }

    /**
     * @verb(PUT)
     * @path("/administrators/{adminId}/disable")
     * @param(string,path)
     * @param(string,query)
     *
     * @throws \Rest\RestException
     */
    public function disableAdministrator($adminId, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->disableAdministrator($adminId, $datacenterId);
    }

    /**
     * @verb(GET)
     * @path("/administratorGuideLink")
     * @throws Exception
     * @return array
     */
    public function administratorGuideLink()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));

        return $this->getTenantStrategy()->getUserGuideUrl();
    }

    /**
     * @verb(GET)
     * @path("/redirectUrl")
     * @throws Exception
     * @return array
     */
    public function adminUrlRedirect()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->adminUrlRedirect();
    }

    /**
     * @verb(GET)
     * @path("/optionalCounters")
     * @param()
     */
    public function getOptionalCounters()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->getOptionalCounters();
    }

    /**
     * @verb(GET)
     * @path("/updateStatistics")
     * @param()
     */
    public function updateUsageAndLimits()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->updateUsageAndLimits();
    }

    /**
     * @verb(GET)
     * @path("/getDatacenter")
     * @return object
     */
    public function getDatacenter()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->getDatacenter();
    }

    /**
     * @verb(GET)
     * @path("/getDatacenters")
     * @return array
     */
    public function getDatacenters()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->getDatacenters();
    }

    /**
     * @verb(GET)
     * @path("/storages")
     * @return array
     */
    public function getStorages()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->getStorages();
    }

    /**
     * @verb(GET)
     * @path("/plans")
     * @return array
     */
    public function getServicePlans()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->getServicePlans();
    }

    /**
     * @verb(GET)
     * @path("/plans/{planId}")
     * @param(string,path)
     * @return object
     */
    public function getServicePlan($planId)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->getServicePlan($planId);
    }

    /**
     * @verb(PUT)
     * @path("/plans/{planId}")
     * @param(string,path)
     * @param(object,body)
     */
    public function setStorageForPlan($planId, $parameters)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->setStorageForPlan($planId, $parameters);
    }

    /**
     * @verb(DELETE)
     * @path("/plans/{planId}")
     * @param(string,path)
     */
    public function unsetStorageForPlan($planId)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        $this->getTenantStrategy()->unsetStorageForPlan($planId);
    }

    /**
     * @verb(GET)
     * @path("/loginExists")
     *
     * @param(string,query)
     * @param(string,query)
     *
     * @return(boolean)
     */
    public function loginExists($login, $datacenterId = '')
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        Log::getInstance()->event(self::LOG_CALLED_METHOD_WAS_CALLED, __METHOD__);
        return $this->getTenantStrategy()->loginExists($login, $datacenterId);
    }

    /**
     * @verb(GET)
     * @path("/agents")
     * @param(string)
     * @return(object)
     */
    public function getAgents($language)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->getAgents($language);
    }

    /**
     * @verb(GET)
     * @path("/quotasStatistics")
     * @param()
     * @return array
     */
    public function getQuotasStatistics()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));
        return $this->getTenantStrategy()->getQuotasStatistics();
    }

    /* ======= OTHER METHODS ======= */

    /**
     * @return Model_Resource_TenantInterface
     * @throws Exception
     */
    protected function getTenantStrategy()
    {
        switch ($this->type) {
            case self::TYPE_RESELLER:
                return new Model_Resource_Tenant_Reseller($this, $this->_apsc, $this->_facade, $this->_managerFactory);
            case self::TYPE_PARTNER:
                return new Model_Resource_Tenant_Partner($this, $this->_apsc, $this->_facade, $this->_managerFactory);
            case self::TYPE_CUSTOMER:
                return new Model_Resource_Tenant_Customer($this, $this->_apsc, $this->_facade, $this->_managerFactory);
            default:
                throw new Exception(Str::format('Unsupported type "%s".', $this->type));
        }
    }

    /* ----- Common methods -------- */

    /**
     * @return PartnerCredentials
     */
    public function getPartnerCredentials()
    {
        Log::getInstance()->event(self::LOG_CALLED_GET_PARTNER_CREDENTIALS);
        $datacenter = $this->datacenter;
        $credentials = new PartnerCredentials();
        $credentials->baasUrl = $datacenter->url;
        $credentials->partnerLogin = $datacenter->login;
        $credentials->partnerPassword = $datacenter->password;
        return $credentials;
    }

    /**
     * @return string
     */
    protected function getSelfApsType()
    {
        return self::CLOUD_BACKUP_TENANT_TYPE;
    }

    /**
     * @return users
     */
    public function createUser()
    {
        return new users(
            $this->_facade,
            $this->_managerFactory,
            $this->_sessionStore,
            $this->_apsc
        );
    }

    protected function throwRestAccepted()
    {
        throw new \Rest\Accepted(
            $this,
            Str::format(
                self::EXCEPTION_CREATING_TENANT_MESSAGE,
                $this->account->companyName,
                $this->subscription->subscriptionId
            ),
            self::ASYNC_OPERATION_TIMEOUT
        );
    }

    protected function initializeTenantType()
    {
        if ($this->_isResellerSubscription()) {
            $this->type = self::TYPE_RESELLER;
        } else {
            $apsApi = $this->_getApsApi();
            $profile = $apsApi->getProfile($this->_getSubscriptionResources());
            $this->type = is_null($profile) || $profile->mspGroupType != profile::MSP_GROUP_TYPE_RESELLER ? self::TYPE_CUSTOMER : self::TYPE_PARTNER;
        }
    }

    private function _getSubscriptionResources()
    {
        if (is_null($this->_subscriptionResources)) {
            $this->_subscriptionResources = $this->subscription->resources();
        }
        return $this->_subscriptionResources;
    }

    /**
     * @return bool
     */
    private function _isResellerSubscription()
    {
        $resources = $this->_getSubscriptionResources();
        if (!is_array($resources)) {
            return false;
        }
        foreach ($resources as $res) {
            if (!isset($res->apsType)) {
                continue;
            }
            if ($res->apsType == self::CLOUD_BACKUP_RESELLER_TYPE) {
                return true;
            }
        }
        return false;
    }

    private function _removeAdmins()
    {
        if (!isset($this->admins)) {
            return;
        }
        $admins = $this->admins;
        if (!count($admins)) {
            $admins = $this->_getApsApi()->getAdminsForTenant($this);
        }
        foreach ($admins as $admin) {
            $this->_apsc->unregisterResource($admin);
        }
    }

    private function _subscribeToOnTenantAvailableEvent()
    {
        $eventSubscription = new \APS\EventSubscription(\APS\EventSubscription::Available, self::ON_TENANT_AVAILABLE);
        $this->subscribeOnEventById($eventSubscription, $this->aps->id);
    }

    private function _unsubscribeFromOnTenantAvailableEvent()
    {
        $this->unsubscribeFromEventById(\APS\EventSubscription::Available, $this->aps->id);
    }
}
