<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require_once(__DIR__ . '/app/bootstrap.php');

/**
 * Class tenantCCPv2
 * @type("http://acronis.com/aps/cloud-backup-v1/tenant/9.5")
 * @implements("http://aps-standard.org/types/core/resource/1.0")
 * @implements("http://aps-standard.org/types/core/suspendable/1.0")
 * @implements("http://aps-standard.org/types/core/subscription/service/1.0")
 */
class tenantCCPv2 extends tenant
{
    const LOG_EVENT_START_OPERATION = "Start %s: %s %s body: %s";
    const LOG_EVENT_FINISH_OPERATION = "Finish %s: %s %s body: %s";

    const OPERATION_PROVISION_RESOURCE = "provisionResource";
    const OPERATION_CONFIGURE_RESOURCE = "configureResource";

    const EXCEPTION_UNSUPPORTED_OPERATION_MESSAGE = "Unsupported action: %s";

    const APS_2_2_RESOURCES = '/aps/2.2/resources/';

    /* ======= RESOURCE PROPERTIES ======= */

    /* ======= OTHER PROPERTIES ======= */

    /* ======= RESOURCE METHODS ======= */

    public function upgrade()
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));

        parent::upgrade();
        $this->unsubscribeFromOnUserAvailableEvent();
    }
    /* ======= RESOURCE EVENTS ======= */

    /* ======= RESOURCE OPERATIONS ======= */

    /**
     * @verb(POST)
     * @path("/setUsersLicensesAfterUpsell")
     *
     * @param(object,body)
     */
    public function setUsersLicensesAfterUpsell($data)
    {
        Log::getInstance()->event(Str::format(
            self::LOG_METHOD_CALL, $this->aps->id, $this->accountId, $this->subscriptionId, __METHOD__
        ));

        $apsc = $this->_apsc->impersonate($this);
        foreach ($data as $operation => $items) {
            if (!is_array($items)) {
                continue;
            }

            foreach ($items as $item) {
                switch ($operation) {
                    case self::OPERATION_PROVISION_RESOURCE:
                        $method = \APS\Proto::POST;
                        $path = self::APS_2_2_RESOURCES;
                        $data = json_encode($item);
                        break;
                    case self::OPERATION_CONFIGURE_RESOURCE:
                        $method = \APS\Proto::PUT;
                        $path = self::APS_2_2_RESOURCES . $item->aps->id;
                        $data = json_encode($item);
                        break;
                    default:
                        throw new Exception(Str::format(self::EXCEPTION_UNSUPPORTED_OPERATION_MESSAGE, $operation));
                }
                Log::getInstance()->event(self::LOG_EVENT_START_OPERATION, $operation, $method, $path, $data);
                $out = $apsc->getIo()->sendRequest($method, $path, $data);
                Log::getInstance()->event(self::LOG_EVENT_FINISH_OPERATION, $operation, $method, $path, $out);
            }
        }
    }

    /* ======= OTHER METHODS ======= */

    /**
     * @return usersCCPv2
     */
    public function createUser()
    {
        return new usersCCPv2(
            $this->_facade,
            $this->_managerFactory,
            $this->_sessionStore,
            $this->_apsc
        );
    }

    /**
     * @return Model_Resource_TenantInterface
     * @throws Exception
     */
    protected function getTenantStrategy()
    {
        switch ($this->type) {
            case self::TYPE_RESELLER:
                return new Model_Resource_Tenant_Reseller($this, $this->_apsc, $this->_facade, $this->_managerFactory);
            case self::TYPE_PARTNER:
                return new Model_Resource_Tenant_Partner($this, $this->_apsc, $this->_facade, $this->_managerFactory);
            case self::TYPE_CUSTOMER:
                return new Model_Resource_Tenant_CustomerCCPv2($this, $this->_apsc, $this->_facade, $this->_managerFactory);
            default:
                throw new Exception(Str::format('Unsupported type "%s".', $this->type));
        }
    }

    /**
     * @return string
     */
    protected function getSelfApsType()
    {
        return self::CLOUD_BACKUP_TENANT_TYPE_CCPv2;
    }

    public function unsubscribeFromOnUserAvailableEvent()
    {
        $this->unsubscribeFromEventByType(\APS\EventSubscription::Available, \APS\TypeLibrary::getTypeByServiceId('users'));
    }
}
