<?php
/**
 * @Copyright © 2002-2015 Acronis International GmbH. All rights reserved
 */

class BackupAgent_to_array {

    /**
     * Create an array of all elements in this object
     *
     * @access public
     * @return array
     */
    public function to_array() {
        $arr = array();
        $this->convert($this, $arr);
        return $this->convert_anytype($this->strip_array($arr));
    }

    /**
     * Strip arrays
     *
     * @access private
     * @param array
     * @return array $stripped_array
     */
    private function strip_array($array) {
        if (count($array) == 1) {
            return array_shift($array);
        } else {
            return $array;
        }
    }

    /**
     * Convert an object to an array
     *
     * @access private
     * @param mixed $object
     * @param array $restructarray byRef
     */
    private function convert($object, &$restructarray) {
        if (is_object($object) || is_array($object)) {
            foreach($object as $key => $value) {
                $this->convert($value, $restructarray[$key]);
            }
        } else {
            $restructarray = $object;
        }
    }

    /**
     * */
    private function convert_anytype($array) {
        if (!is_array($array)) {
            return $array;
        }
        foreach ($array as $key => $value) {
            if ( $key === 'anyType') {
                foreach ($array[$key] as $value2) {
                    $array[] = $value2;
                }
                unset($array[$key]);
            } elseif (is_array($value)) {
                $array[$key] = $this->convert_anytype($value);
            }
        }
        return $array;
    }
}


/**
 * BackupAgent classes
 *
 * @package Controller-product
 * @author -
 * @version $Id: classes.php 554321 2010-02-12 12:11:58Z alivanov $
 */

class BackupAgent_CheckDatabaseConnection extends BackupAgent_to_array {

}

class BackupAgent_CheckDatabaseConnectionResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $CheckDatabaseConnectionResult;

}

class BackupAgent_CheckASPNETVersion extends BackupAgent_to_array {

}

class BackupAgent_CheckASPNETVersionResponse extends BackupAgent_to_array {

}

class BackupAgent_UpdateClientMessage extends BackupAgent_to_array {

    /**
     * @var (object)ClientMessage
     */
    public $message;

}

class BackupAgent_ClientMessage extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $Available;

    /**
     * @var string
     *     NOTE: Language should follow the following restrictions
     *     You can have one of the following value
     *     NL
     *     EN
     *     FR
     *     ES
     *     DE
     *     IT
     *     UK
     */
    public $Language;

    /**
     * @var string
     */
    public $Message;

    /**
     * @var string
     */
    public $Link;

}

class BackupAgent_UpdateClientMessageResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $UpdateClientMessageResult;

}

class BackupAgent_UpdateClientMessagesLink extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $link;

}

class BackupAgent_UpdateClientMessagesLinkResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $UpdateClientMessagesLinkResult;

}

class BackupAgent_GetClientMessage extends BackupAgent_to_array {

    /**
     * @var string
     *     NOTE: language should follow the following restrictions
     *     You can have one of the following value
     *     NL
     *     EN
     *     FR
     *     ES
     *     DE
     *     IT
     *     UK
     */
    public $language;

}

class BackupAgent_GetClientMessageResponse extends BackupAgent_to_array {

    /**
     * @var (object)ClientMessage
     */
    public $GetClientMessageResult;

}

class BackupAgent_GetClientMessageLink extends BackupAgent_to_array {

}

class BackupAgent_GetClientMessageLinkResponse extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $GetClientMessageLinkResult;

}

class BackupAgent_UploadBatchWithByteInformation extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfAnyType
     */
    public $parms;

    /**
     * @var boolean
     */
    public $resend;

}

class BackupAgent_ArrayOfAnyType extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of anonymous0
     */
    public $anyType;

}

class BackupAgent_UploadBatchWithByteInformationResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfBatchFileResult
     */
    public $UploadBatchWithByteInformationResult;

    /**
     * @var long
     */
    public $byteSend;

}

class BackupAgent_ArrayOfBatchFileResult extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)BatchFileResult
     */
    public $BatchFileResult;

}

class BackupAgent_BatchFileResult extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $FileInfo;

    /**
     * @var string
     *     NOTE: Result should follow the following restrictions
     *     You can have one of the following value
     *     Sucess
     *     TemporaryError
     *     FatalError
     */
    public $Result;

    /**
     * @var int
     */
    public $NbVersionsToDelete;

    /**
     * @var int
     */
    public $NbRetries;

    /**
     * @var long
     */
    public $FileSize;

}

class BackupAgent_UploadBatch extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfAnyType
     */
    public $parms;

    /**
     * @var boolean
     */
    public $resend;

}

class BackupAgent_UploadBatchResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfBatchFileResult
     */
    public $UploadBatchResult;

}

class BackupAgent_UploadBatchEx extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfBatchFileItem
     */
    public $parms;

    /**
     * @var boolean
     */
    public $resend;

}

class BackupAgent_ArrayOfBatchFileItem extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)BatchFileItem
     */
    public $BatchFileItem;

}

class BackupAgent_BatchFileItem extends BackupAgent_to_array {

    /**
     * @var (object)FileInformation
     *    Or one of following derived class(es)
     *       FileVersion
     */
    public $fileOnClient;

    /**
     * @var string
     */
    public $ComputerName;

    /**
     * @var int
     */
    public $KeyUsedType;

    /**
     * @var int
     */
    public $Compressed;

    /**
     * @var int
     */
    public $CompressionType;

    /**
     * @var int
     */
    public $CompressionLevel;

    /**
     * @var int
     */
    public $EncryptionType;

    /**
     * @var string
     */
    public $ClientVersion;

    /**
     * @var int
     */
    public $NbVersionsToDelete;

    /**
     * @var int
     */
    public $Version;

    /**
     * @var string
     */
    public $FileType;

    /**
     * @var int
     */
    public $Retries;

    // You need to set only one from the following two vars

    /**
     * @var Plain Binary
     */
    public $DataChecksum;

    /**
     * @var base64Binary
     */
    public $DataChecksum_encoded;


    // You need to set only one from the following two vars

    /**
     * @var Plain Binary
     */
    public $DataSource;

    /**
     * @var base64Binary
     */
    public $DataSource_encoded;

    /**
     * @var type
     */
    public $Type;

}

class BackupAgent_FileInformation extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $Name;

    /**
     * @var long
     */
    public $Size;

    /**
     * @var dateTime
     */
    public $dateCreated;

    /**
     * @var dateTime
     */
    public $dateChanged;

    /**
     * @var string
     */
    public $Path;

}

class BackupAgent_FileVersion extends BackupAgent_FileInformation {

    /**
     * @var int
     */
    public $versionNumber;

    /**
     * @var string
     */
    public $fullName;

    /**
     * @var long
     */
    public $SizeOnServer;

    /**
     * @var int
     */
    public $KeyUsed;

    /**
     * @var int
     */
    public $Compressed;

    /**
     * @var boolean
     */
    public $isPending;

    /**
     * @var string
     */
    public $ClientVersion;

    /**
     * @var int
     */
    public $CompressionType;

    /**
     * @var int
     */
    public $CompressionLevel;

    /**
     * @var int
     */
    public $EncryptionType;

}

class BackupAgent_UploadBatchExResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfBatchFileResult
     */
    public $UploadBatchExResult;

}

class BackupAgent_UploadFileWithByteInformation extends BackupAgent_to_array {

    /**
     * @var (object)FileBlock
     */
    public $ReceivedBlock;

    /**
     * @var boolean
     */
    public $DoResume;

}

class BackupAgent_FileBlock extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $fullName;

    /**
     * @var string
     */
    public $clientFileName;

    /**
     * @var int
     */
    public $blockSize;

    /**
     * @var long
     */
    public $blockNumber;

    /**
     * @var long
     */
    public $blockCount;

    // You need to set only one from the following two vars

    /**
     * @var Plain Binary
     */
    public $Data;

    /**
     * @var base64Binary
     */
    public $Data_encoded;


}

class BackupAgent_UploadFileWithByteInformationResponse extends BackupAgent_to_array {

    /**
     * @var long
     */
    public $UploadFileWithByteInformationResult;

    /**
     * @var long
     */
    public $byteSend;

}

class BackupAgent_UploadFile extends BackupAgent_to_array {

    /**
     * @var (object)FileBlock
     */
    public $ReceivedBlock;

    /**
     * @var boolean
     */
    public $DoResume;

}

class BackupAgent_UploadFileResponse extends BackupAgent_to_array {

    /**
     * @var long
     */
    public $UploadFileResult;

}

class BackupAgent_UploadFileExWithByteInformation extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $DoResume;

    /**
     * @var long
     */
    public $blockCount;

    /**
     * @var long
     */
    public $blockNumber;

    /**
     * @var int
     */
    public $blockSize;

    /**
     * @var string
     */
    public $blockFullName;

}

class BackupAgent_UploadFileExWithByteInformationResponse extends BackupAgent_to_array {

    /**
     * @var long
     */
    public $UploadFileExWithByteInformationResult;

    /**
     * @var long
     */
    public $byteSend;

}

class BackupAgent_UploadFileEx extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $DoResume;

    /**
     * @var long
     */
    public $blockCount;

    /**
     * @var long
     */
    public $blockNumber;

    /**
     * @var int
     */
    public $blockSize;

    /**
     * @var string
     */
    public $blockFullName;

}

class BackupAgent_UploadFileExResponse extends BackupAgent_to_array {

    /**
     * @var long
     */
    public $UploadFileExResult;

}

class BackupAgent_DownloadFileWithByteInformation extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $fullName;

    /**
     * @var long
     */
    public $blockNumber;

    /**
     * @var int
     */
    public $blockSize;

    /**
     * @var boolean
     */
    public $DoDecompress;

    /**
     * @var boolean
     */
    public $DoDecrypt;

}

class BackupAgent_DownloadFileWithByteInformationResponse extends BackupAgent_to_array {

    /**
     * @var (object)FileBlock
     */
    public $DownloadFileWithByteInformationResult;

    /**
     * @var long
     */
    public $byteSend;

}

class BackupAgent_DownloadFile extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $fullName;

    /**
     * @var long
     */
    public $blockNumber;

    /**
     * @var int
     */
    public $blockSize;

    /**
     * @var boolean
     */
    public $DoDecompress;

    /**
     * @var boolean
     */
    public $DoDecrypt;

}

class BackupAgent_DownloadFileResponse extends BackupAgent_to_array {

    /**
     * @var (object)FileBlock
     */
    public $DownloadFileResult;

}

class BackupAgent_VerifyUpload extends BackupAgent_to_array {

    /**
     * @var long
     */
    public $oldSize;

    /**
     * @var long
     */
    public $newSize;

}

class BackupAgent_VerifyUploadResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $VerifyUploadResult;

}

class BackupAgent_AddVersionWithByteInformation extends BackupAgent_to_array {

    /**
     * @var (object)FileInformation
     *    Or one of following derived class(es)
     *       FileVersion
     */
    public $FileOnClient;

    /**
     * @var string
     */
    public $ComputerName;

    /**
     * @var string
     */
    public $FileType;

    /**
     * @var int
     */
    public $KeyUsedType;

    /**
     * @var int
     */
    public $intCompressed;

    /**
     * @var int
     */
    public $intCompressionType;

    /**
     * @var int
     */
    public $intCompressionLevel;

    /**
     * @var int
     */
    public $intEncryptionType;

    /**
     * @var string
     */
    public $strClientVersion;

}

class BackupAgent_AddVersionWithByteInformationResponse extends BackupAgent_to_array {

    /**
     * @var (object)FileVersion
     */
    public $AddVersionWithByteInformationResult;

    /**
     * @var long
     */
    public $byteSend;

}

class BackupAgent_addVersion extends BackupAgent_to_array {

    /**
     * @var (object)FileInformation
     *    Or one of following derived class(es)
     *       FileVersion
     */
    public $FileOnClient;

    /**
     * @var string
     */
    public $ComputerName;

    /**
     * @var string
     */
    public $FileType;

    /**
     * @var int
     */
    public $KeyUsedType;

    /**
     * @var int
     */
    public $intCompressed;

    /**
     * @var int
     */
    public $intCompressionType;

    /**
     * @var int
     */
    public $intCompressionLevel;

    /**
     * @var int
     */
    public $intEncryptionType;

    /**
     * @var string
     */
    public $strClientVersion;

}

class BackupAgent_addVersionResponse extends BackupAgent_to_array {

    /**
     * @var (object)FileVersion
     */
    public $addVersionResult;

}

class BackupAgent_AddLogEntry extends BackupAgent_to_array {

    /**
     * @var (object)LogErrorEntryWS
     */
    public $objLogEntry;

}

class BackupAgent_LogEntry extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $User_Name;

    /**
     * @var boolean
     */
    public $IsUploadDownloadError;

}

class BackupAgent_LogErrorEntryWS extends BackupAgent_LogEntry {

    /**
     * @var dateTime
     */
    public $entryDate;

    /**
     * @var string
     *     NOTE: LogGUID should follow the following restrictions
     *     The pattern should be
     * [0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}
     */
    public $LogGUID;

    /**
     * @var string
     */
    public $LogMessage;

    /**
     * @var string
     */
    public $StackTrace;

    /**
     * @var (object)ArrayOfString
     */
    public $paramNames;

    /**
     * @var (object)ArrayOfString
     */
    public $paramValules;

}

class BackupAgent_ArrayOfString extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of string
     */
    public $string;

}

class BackupAgent_AddLogEntryResponse extends BackupAgent_to_array {

}

class BackupAgent_AddClientLogs extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

    /**
     * @var string
     */
    public $ComputerName;

    /**
     * @var (object)ArrayOfClientLog
     */
    public $logs;

}

class BackupAgent_ArrayOfClientLog extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)ClientLog
     */
    public $ClientLog;

}

class BackupAgent_ClientLog extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $Message;

    /**
     * @var string
     */
    public $Description;

    /**
     * @var dateTime
     */
    public $Date;

    /**
     * @var string
     *     NOTE: Severity should follow the following restrictions
     *     You can have one of the following value
     *     Error
     *     Warning
     *     Information
     */
    public $Severity;

}

class BackupAgent_AddClientLogsResponse extends BackupAgent_to_array {

}

class BackupAgent_AddAccountType extends BackupAgent_to_array {

    /**
     * @var (object)AccountType
     */
    public $objAccountType;

}

class BackupAgent_AccountType extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $accountTypeID;

    /**
     * @var string
     */
    public $typeName;

    /**
     * @var string
     *     NOTE: descriptionType should follow the following restrictions
     *     You can have one of the following value
     *     Normal_Account
     *     Trial_Account
     *     Free_Account
     *     Normal_And_Trial_Account
     */
    public $descriptionType;

    /**
     * @var int
     */
    public $maxSize;

    /**
     * @var boolean
     */
    public $isActive;

    /**
     * @var int
     */
    public $Period;
    
    /**
     * @var string
     *     NOTE: Type should follow the following restrictions
     *     You can have one of the following value
     *     Professional
     *     Home
     */
    public $Type;

}

class BackupAgent_AddAccountTypeResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $AddAccountTypeResult;

}

class BackupAgent_AddAccountType_Ex extends BackupAgent_to_array {

    /**
     * @var (object)AccountType
     */
    public $objAccountType;

}

class BackupAgent_AddAccountType_ExResponse extends BackupAgent_to_array {

}

class BackupAgent_ActivateAccountType extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $intAccountTypeID;

}

class BackupAgent_ActivateAccountTypeResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $ActivateAccountTypeResult;

}

class BackupAgent_DeActivateAccountType extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $intAccountTypeID;

}

class BackupAgent_DeActivateAccountTypeResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $DeActivateAccountTypeResult;

}

class BackupAgent_GetAccountTypes extends BackupAgent_to_array {

    /**
     * @var string
     *     NOTE: objCriteria should follow the following restrictions
     *     You can have one of the following value
     *     Only_Active
     *     Only_Non_Active
     *     All_AccountTypes
     */
    public $objCriteria;

    /**
     * @var string
     *     NOTE: objCriteria2 should follow the following restrictions
     *     You can have one of the following value
     *     Normal_Account
     *     Trial_Account
     *     Free_Account
     *     Normal_And_Trial_Account
     */
    public $objCriteria2;

}

class BackupAgent_GetAccountTypesResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfAccountType
     */
    public $GetAccountTypesResult;

}

class BackupAgent_ArrayOfAccountType extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)AccountType
     */
    public $AccountType;

}

class BackupAgent_DeleteChecksumFile extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $strChecksumPath;

    /**
     * @var string
     */
    public $strChecksumFileName;

}

class BackupAgent_DeleteChecksumFileResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $DeleteChecksumFileResult;

}

class BackupAgent_DeleteVersionEx extends BackupAgent_to_array {

    /**
     * @var (object)FileVersion
     */
    public $VersionToDelete;

    /**
     * @var boolean
     */
    public $full;

}

class BackupAgent_DeleteVersionExResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfFileVersion
     */
    public $DeleteVersionExResult;

}

class BackupAgent_ArrayOfFileVersion extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)FileVersion
     */
    public $FileVersion;

}

class BackupAgent_DeleteAllExchangeStoreVersions extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $serverFullName;

}

class BackupAgent_DeleteAllExchangeStoreVersionsResponse extends BackupAgent_to_array {

}

class BackupAgent_DeleteExchangeStoreVersion extends BackupAgent_to_array {

    /**
     * @var (object)FileVersion
     */
    public $VersionToDelete;

}

class BackupAgent_DeleteExchangeStoreVersionResponse extends BackupAgent_to_array {

}

class BackupAgent_DeleteVersion extends BackupAgent_to_array {

    /**
     * @var (object)FileVersion
     */
    public $VersionToDelete;

    /**
     * @var boolean
     */
    public $full;

}

class BackupAgent_DeleteVersionResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfFileVersion
     */
    public $DeleteVersionResult;

}

class BackupAgent_DeleteVersion_MC extends BackupAgent_to_array {

    /**
     * @var (object)FileVersion
     */
    public $VersionToDelete;

    /**
     * @var boolean
     */
    public $full;

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_DeleteVersion_MCResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $DeleteVersion_MCResult;

}

class BackupAgent_GetComputerNamesWithByteInformation extends BackupAgent_to_array {

}

class BackupAgent_GetComputerNamesWithByteInformationResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $GetComputerNamesWithByteInformationResult;

    /**
     * @var long
     */
    public $byteSend;

}

class BackupAgent_GetComputerNames extends BackupAgent_to_array {

}

class BackupAgent_GetComputerNamesResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $GetComputerNamesResult;

}

class BackupAgent_GetUserComputerNames extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_GetUserComputerNamesResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $GetUserComputerNamesResult;

}

class BackupAgent_GetFolderObjectWithByteInformation extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $path;

    /**
     * @var string
     */
    public $computer;

}

class BackupAgent_GetFolderObjectWithByteInformationResponse extends BackupAgent_to_array {

    /**
     * @var (object)RemoteMap
     */
    public $GetFolderObjectWithByteInformationResult;

    /**
     * @var long
     */
    public $byteSend;

}

class BackupAgent_RemoteMap extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $listOfSubDirs;

    /**
     * @var (object)ArrayOfRemoteMapFile
     */
    public $objFolderFiles;

    /**
     * @var string
     */
    public $Path;

}

class BackupAgent_ArrayOfRemoteMapFile extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)RemoteMapFile
     */
    public $RemoteMapFile;

}

class BackupAgent_RemoteMapFile extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $fileName;

    /**
     * @var int
     */
    public $fileVersionCount;

    /**
     * @var string
     */
    public $clientVersion;

    /**
     * @var dateTime
     */
    public $dataChanged;

    /**
     * @var dateTime
     */
    public $dataCreated;

    /**
     * @var string
     */
    public $fileType;

    /**
     * @var boolean
     */
    public $fileIsPending;

    /**
     * @var long
     */
    public $fileSize_LastVersion;

}

class BackupAgent_GetFolderObject extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $path;

    /**
     * @var string
     */
    public $computer;

}

class BackupAgent_GetFolderObjectResponse extends BackupAgent_to_array {

    /**
     * @var (object)RemoteMap
     */
    public $GetFolderObjectResult;

}

class BackupAgent_GetFolderObject_MC extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $path;

    /**
     * @var string
     */
    public $computer;

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_GetFolderObject_MCResponse extends BackupAgent_to_array {

    /**
     * @var (object)RemoteMap
     */
    public $GetFolderObject_MCResult;

}

class BackupAgent_GetFileObjectWithByteInformation extends BackupAgent_to_array {

    /**
     * @var (object)FileInformation
     *    Or one of following derived class(es)
     *       FileVersion
     */
    public $FileOnClient;

    /**
     * @var string
     */
    public $ComputerName;

}

class BackupAgent_GetFileObjectWithByteInformationResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfFileVersion
     */
    public $GetFileObjectWithByteInformationResult;

    /**
     * @var long
     */
    public $byteSend;

}

class BackupAgent_GetFileObject extends BackupAgent_to_array {

    /**
     * @var (object)FileInformation
     *    Or one of following derived class(es)
     *       FileVersion
     */
    public $FileOnClient;

    /**
     * @var string
     */
    public $ComputerName;

}

class BackupAgent_GetFileObjectResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfFileVersion
     */
    public $GetFileObjectResult;

}

class BackupAgent_GetFileObject_MC extends BackupAgent_to_array {

    /**
     * @var (object)FileInformation
     *    Or one of following derived class(es)
     *       FileVersion
     */
    public $fileOnClient;

    /**
     * @var string
     */
    public $computerName;

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_GetFileObject_MCResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfFileVersion
     */
    public $GetFileObject_MCResult;

}

class BackupAgent_GetFileObjectForBackupWithByteInformation extends BackupAgent_to_array {

    /**
     * @var (object)FileInformation
     *    Or one of following derived class(es)
     *       FileVersion
     */
    public $FileOnClient;

    /**
     * @var string
     */
    public $ComputerName;

}

class BackupAgent_GetFileObjectForBackupWithByteInformationResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfFileVersion
     */
    public $GetFileObjectForBackupWithByteInformationResult;

    /**
     * @var long
     */
    public $byteSend;

}

class BackupAgent_GetFileObjectForBackup extends BackupAgent_to_array {

    /**
     * @var (object)FileInformation
     *    Or one of following derived class(es)
     *       FileVersion
     */
    public $FileOnClient;

    /**
     * @var string
     */
    public $ComputerName;

}

class BackupAgent_GetFileObjectForBackupResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfFileVersion
     */
    public $GetFileObjectForBackupResult;

}

class BackupAgent_CreateFolder extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $FolderToCreate;

    /**
     * @var string
     */
    public $Path;

    /**
     * @var string
     */
    public $Computer;

}

class BackupAgent_CreateFolderResponse extends BackupAgent_to_array {

}

class BackupAgent_DeleteFolder extends BackupAgent_to_array {

    /**
     * @var (object)RemoteMap
     */
    public $FolderToDelete;

    /**
     * @var string
     */
    public $Computer;

}

class BackupAgent_DeleteFolderResponse extends BackupAgent_to_array {

}

class BackupAgent_ChangeOwner extends BackupAgent_to_array {

    /**
     * @var (object)OwnerInfo
     */
    public $objOwnerInfo;

}

class BackupAgent_OwnerInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $resellerName;

    /**
     * @var string
     */
    public $Owner;

    /**
     * @var int
     */
    public $OwnerID;

    /**
     * @var string
     */
    public $StorageLocation;

    /**
     * @var string
     */
    public $LoginName;

    /**
     * @var string
     */
    public $DomainName;

    /**
     * @var string
     */
    public $LoginPassword;

    /**
     * @var int
     */
    public $UserCount;

    /**
     * @var int
     */
    public $ResellerCount;

    /**
     * @var long
     */
    public $usedSize;

}

class BackupAgent_ChangeOwnerResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $ChangeOwnerResult;

}

class BackupAgent_ChangeOwner_Ex extends BackupAgent_to_array {

    /**
     * @var (object)OwnerInfo
     */
    public $objOwnerInfo;

}

class BackupAgent_ChangeOwner_ExResponse extends BackupAgent_to_array {

}

class BackupAgent_ChangeDefaultOwner extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $strReseller;

    /**
     * @var string
     */
    public $ownerName;

}

class BackupAgent_ChangeDefaultOwnerResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $ChangeDefaultOwnerResult;

}

class BackupAgent_ChangeDefaultOwner_Ex extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $strReseller;

    /**
     * @var string
     */
    public $ownerName;

}

class BackupAgent_ChangeDefaultOwner_ExResponse extends BackupAgent_to_array {

}

class BackupAgent_CreateOwner extends BackupAgent_to_array {

    /**
     * @var (object)OwnerInfo
     */
    public $objOwnerInfo;

    /**
     * @var boolean
     */
    public $setAsDefault;

}

class BackupAgent_CreateOwnerResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $CreateOwnerResult;

}

class BackupAgent_CreateOwner_Ex extends BackupAgent_to_array {

    /**
     * @var (object)OwnerInfo
     */
    public $objOwnerInfo;

    /**
     * @var boolean
     */
    public $setAsDefault;

}

class BackupAgent_CreateOwner_ExResponse extends BackupAgent_to_array {

}

class BackupAgent_GetDBCountries extends BackupAgent_to_array {

}

class BackupAgent_GetDBCountriesResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $GetDBCountriesResult;

}

class BackupAgent_GetDBStates extends BackupAgent_to_array {

}

class BackupAgent_GetDBStatesResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $GetDBStatesResult;

}

class BackupAgent_CreateReseller extends BackupAgent_to_array {

    /**
     * @var (object)UserInfo
     */
    public $objUserInfo;

    /**
     * @var (object)ResellerInfo
     */
    public $resellerInfo;

    /**
     * @var string
     */
    public $userName;

    /**
     * @var string
     */
    public $passWord;

    /**
     * @var string
     */
    public $keyQuestion;

}

class BackupAgent_UserInfo extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $ID;

    /**
     * @var string
     */
    public $ParentReseller;

    /**
     * @var string
     */
    public $firstName;

    /**
     * @var string
     */
    public $lastName;

    /**
     * @var string
     */
    public $telephoneNr;

    /**
     * @var string
     */
    public $street;

    /**
     * @var string
     */
    public $zipcode;

    /**
     * @var string
     */
    public $city;

    /**
     * @var int
     */
    public $countryID;

    /**
     * @var int
     */
    public $stateID;

    /**
     * @var string
     */
    public $email;

    /**
     * @var string
     */
    public $sex;

    /**
     * @var string
     */
    public $occupation;

    /**
     * @var string
     */
    public $company;

    /**
     * @var int
     */
    public $userTypeID;

    /**
     * @var string
     */
    public $activeDirectoryGUID;

    /**
     * @var string
     *     NOTE: defaultEmailLanguage should follow the following restrictions
     *     You can have one of the following value
     *     LanguageType_NL
     *     LanguageType_EN
     *     LanguageType_FR
     *     LanguageType_ES
     *     LanguageType_DE
     *     LanguageType_IT
     *     LanguageType_DA
     *     LanguageType_PL
     *     LanguageType_HU
     *     LanguageType_PT
     *     LanguageType_RO
     *     LanguageType_RU
     *     LanguageType_TR
     *     LanguageType_JA
     *     LanguageType_PTBR
     *     LanguageType_VI
     */
    public $defaultEmailLanguage;

}

class BackupAgent_ResellerInfo extends BackupAgent_to_array {

    public $isAllowedRestore;

    /**
     * @var int
     */
    public $ResellerConfigID;

    /**
     * @var string
     */
    public $ParentReseller;

    /**
     * @var boolean
     */
    public $accountUpgrade;

    /**
     * @var boolean
     */
    public $accountDowngrade;

    /**
     * @var boolean
     */
    public $automaticDigiNotar;

    /**
     * @var boolean
     */
    public $allowWhiteLabeling;

    /**
     * @var boolean
     */
    public $allowResellerCreation;

    /**
     * @var boolean
     */
    public $allowHighlighDataGridRow;

    /**
     * @var int
     */
    public $defaultOwner;

    /**
     * @var string
     */
    public $ResellerUserName;

    /**
     * @var string
     */
    public $supportEMail;

    /**
     * @var string
     */
    public $supportURL;

    /**
     * @var string
     */
    public $termsURL;

    /**
     * @var string
     */
    public $registrationURL;

    /**
     * @var string
     */
    public $logoURL;

    /**
     * @var string
     */
    public $trialUpgradeURL;

    /**
     * @var string
     */
    public $stylesheetURL;

    /**
     * @var string
     */
    public $homepageURL;

    /**
     * @var string
     */
    public $clientdownloadURL;

    /**
     * @var string
     */
    public $serviceName;

    /**
     * @var string
     */
    public $HTMLHeader;

    /**
     * @var string
     */
    public $HTMLFooter;

    /**
     * @var double
     */
    public $usagePercentageBeforeEmail;

    /**
     * @var int
     */
    public $dataGridRowCount;

    /**
     * @var (object)ArrayOfString
     */
    public $ownerIDs;

    /**
     * @var int
     */
    public $defaultRegistrationCountry;

    /**
     * @var int
     */
    public $daysWithNoBackupBeforeWarning;

    /**
     * @var int
     */
    public $maxHomeCALS;

    /**
     * @var int
     */
    public $maxProfessionalCALS;

    /**
     * @var int
     */
    public $maxStorageGB;

    /**
     * @var boolean
     */
    public $allowTrials;

    /**
     * @var boolean
     */
    public $sendLogs;

    /**
     * @var boolean
     */
    public $accountUpgradeByGroup;

    /**
     * @var boolean
     */
    public $accountDowngradeByGroup;
    
    /**
     * @var string
     */
    public $AllowAccountDeletion;

}

class BackupAgent_CreateResellerResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $CreateResellerResult;

}

class BackupAgent_GetHTMLContent extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $strResellerName;

    /**
     * @var string
     *     NOTE: htmlContentType should follow the following restrictions
     *     You can have one of the following value
     *     HTMLContent_LogIn
     */
    public $htmlContentType;

    /**
     * @var string
     *     NOTE: languageType should follow the following restrictions
     *     You can have one of the following value
     *     LanguageType_NL
     *     LanguageType_EN
     *     LanguageType_FR
     *     LanguageType_ES
     *     LanguageType_DE
     *     LanguageType_IT
     */
    public $languageType;

}

class BackupAgent_GetHTMLContentResponse extends BackupAgent_to_array {

    /**
     * @var (object)HTMLContentInfo
     */
    public $GetHTMLContentResult;

}

class BackupAgent_HTMLContentInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $UserName;

    /**
     * @var string
     */
    public $Title;

    /**
     * @var string
     */
    public $Content;

}

class BackupAgent_SetHTMLContent extends BackupAgent_to_array {

    /**
     * @var (object)HTMLContentInfo
     */
    public $objHtmlContent;

    /**
     * @var string
     *     NOTE: htmlContent should follow the following restrictions
     *     You can have one of the following value
     *     HTMLContent_LogIn
     */
    public $htmlContent;

    /**
     * @var string
     *     NOTE: languageType should follow the following restrictions
     *     You can have one of the following value
     *     LanguageType_NL
     *     LanguageType_EN
     *     LanguageType_FR
     *     LanguageType_ES
     *     LanguageType_DE
     *     LanguageType_IT
     */
    public $languageType;

}

class BackupAgent_SetHTMLContentResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $SetHTMLContentResult;

}

class BackupAgent_GetResellerEmails extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $ResellerUserName;

    /**
     * @var string
     *     NOTE: languageType should follow the following restrictions
     *     You can have one of the following value
     *     LanguageType_NL
     *     LanguageType_EN
     *     LanguageType_FR
     *     LanguageType_ES
     *     LanguageType_DE
     *     LanguageType_IT
     */
    public $languageType;

    /**
     * @var string
     */
    public $Criteria;

}

class BackupAgent_GetResellerEmailsResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfEmail
     */
    public $GetResellerEmailsResult;

}

class BackupAgent_ArrayOfEmail extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)Email
     */
    public $Email;

}

class BackupAgent_Email extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $UserNm;

    /**
     * @var string
     */
    public $From;

    /**
     * @var string
     */
    public $Adres;

    /**
     * @var string
     */
    public $Bcc;

    /**
     * @var string
     *     NOTE: EmailType should follow the following restrictions
     *     You can have one of the following value
     *     Email_ForgotPassword
     *     Email_Registration
     *     Email_UploadDisabled
     *     Email_AccountAlmostFull
     *     Email_ForgotEncryptionKey
     *     Email_AccountUpgraded
     *     Email_BackupReport
     *     Email_RestoreReport
     *     Email_TrialWarning
     *     Email_TrialDeletionNotice
     *     Email_MonthlyOverView
     *     Email_AccountChanged
     *     Email_RegistrationNoActivation
     *     Email_NoBackupWarning
     *     Email_AccountWillExpireWarning
     *     Email_AccountExpired
     *     Email_AccountsStatus
     *     Email_ResetPassword
     *     Email_ASPNETversion
     *     Email_LicenseExpired
     */
    public $EmailType;

    /**
     * @var string
     *     NOTE: EmailFormat should follow the following restrictions
     *     You can have one of the following value
     *     Text
     *     HTML
     */
    public $EmailFormat;

    /**
     * @var string
     */
    public $Subject;

    /**
     * @var string
     */
    public $Text;

    /**
     * @var string
     */
    public $serviceProvider;

    /**
     * @var boolean
     */
    public $Activated;

    /**
     * @var string
     *     NOTE: priority should follow the following restrictions
     *     You can have one of the following value
     *     Normal
     *     Low
     *     High
     */
    public $priority;

}

class BackupAgent_ChangeResellerEmails extends BackupAgent_to_array {

    /**
     * @var (object)Email
     */
    public $objMailInfo;

    /**
     * @var string
     *     NOTE: emailType should follow the following restrictions
     *     You can have one of the following value
     *     Email_ForgotPassword
     *     Email_Registration
     *     Email_UploadDisabled
     *     Email_AccountAlmostFull
     *     Email_ForgotEncryptionKey
     *     Email_AccountUpgraded
     *     Email_BackupReport
     *     Email_RestoreReport
     *     Email_TrialWarning
     *     Email_TrialDeletionNotice
     *     Email_MonthlyOverView
     *     Email_AccountChanged
     *     Email_RegistrationNoActivation
     *     Email_NoBackupWarning
     *     Email_AccountWillExpireWarning
     *     Email_AccountExpired
     *     Email_AccountsStatus
     *     Email_ResetPassword
     *     Email_ASPNETversion
     *     Email_LicenseExpired
     */
    public $emailType;

    /**
     * @var string
     *     NOTE: languageType should follow the following restrictions
     *     You can have one of the following value
     *     LanguageType_NL
     *     LanguageType_EN
     *     LanguageType_FR
     *     LanguageType_ES
     *     LanguageType_DE
     *     LanguageType_IT
     */
    public $languageType;

}

class BackupAgent_ChangeResellerEmailsResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $ChangeResellerEmailsResult;

}

class BackupAgent_CreateAccount extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $strParentUser;

    /**
     * @var (object)UserInfo
     */
    public $objUserInfo;

    /**
     * @var (object)AccountInfo
     */
    public $objAccountinfo;

    /**
     * @var string
     */
    public $userName;

    /**
     * @var string
     */
    public $passWord;

    /**
     * @var string
     */
    public $keyQuestion;

    /**
     * @var string
     */
    public $computer;

    /**
     * @var boolean
     */
    public $sendMail;

}

class BackupAgent_AccountInfo extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $accountInfoID;

    /**
     * @var int
     */
    public $userInfoID;

    /**
     * @var int
     */
    public $status;

    /**
     * @var boolean
     */
    public $active;

    /**
     * @var long
     */
    public $usedSize;

    /**
     * @var int
     */
    public $maxSize;

    /**
     * @var string
     */
    public $accountTypeName;

    /**
     * @var int
     */
    public $accountType;

    /**
     * @var boolean
     */
    public $trialUser;

    /**
     * @var int
     */
    public $ownerID;

    /**
     * @var string
     */
    public $filesLocation;

    /**
     * @var string
     */
    public $mountUserName;

    /**
     * @var string
     */
    public $mountPassWord;

    /**
     * @var string
     */
    public $changedBy;

    /**
     * @var string
     */
    public $ownerName;

    /**
     * @var int
     */
    public $hasDigiNotarKey;

    /**
     * @var boolean
     */
    public $digiNotarKeyIsStored;

    /**
     * @var int
     */
    public $maxVersions;

    /**
     * @var dateTime
     */
    public $startDate;

    /**
     * @var dateTime
     */
    public $endDate;

    /**
     * @var (object)Key
     */
    public $objDigiNotarKey;

    /**
     * @var boolean
     */
    public $hasEncryptionKey;

    /**
     * @var boolean
     */
    public $freeAccount;

    /**
     * @var string
     *     NOTE: userProfile should follow the following restrictions
     *     You can have one of the following value
     *     Professional
     *     Home
     */
    public $userProfile;

    /**
     * @var int
     */
    public $maxBandWidthUsage;

    /**
     * @var int
     */
    public $period;

}

class BackupAgent_Key extends BackupAgent_to_array {

    // You need to set only one from the following two vars

    /**
     * @var Plain Binary
     */
    public $Data;

    /**
     * @var base64Binary
     */
    public $Data_encoded;


    // You need to set only one from the following two vars

    /**
     * @var Plain Binary
     */
    public $HashData;

    /**
     * @var base64Binary
     */
    public $HashData_encoded;


}

class BackupAgent_CreateAccountResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $CreateAccountResult;

}

class BackupAgent_GetAccountHistory extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $str_UserName;

}

class BackupAgent_GetAccountHistoryResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfAccountInfo
     */
    public $GetAccountHistoryResult;

}

class BackupAgent_ArrayOfAccountInfo extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)AccountInfo
     */
    public $AccountInfo;

}

class BackupAgent_ChangeAccountInfoWithChangeAuthor extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $username;

    /**
     * @var (object)AccountInfo
     */
    public $newAccountInfo;

    /**
     * @var string
     */
    public $changedByUserName;

}

class BackupAgent_ChangeAccountInfoWithChangeAuthorResponse extends BackupAgent_to_array {

}

class BackupAgent_ChangeAccountInfoAdmin extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $username;

    /**
     * @var (object)AccountInfo
     */
    public $newAccountInfo;

}

class BackupAgent_ChangeAccountInfoAdminResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $ChangeAccountInfoAdminResult;

}

class BackupAgent_ChangeAccountInfoAdmin_Ex extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $username;

    /**
     * @var (object)AccountInfo
     */
    public $newAccountInfo;

}

class BackupAgent_ChangeAccountInfoAdmin_ExResponse extends BackupAgent_to_array {

}

class BackupAgent_ChangeResellerInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $resellerUsername;

    /**
     * @var (object)UserInfo
     */
    public $newUserInfo;

    /**
     * @var (object)ResellerInfo
     */
    public $newResellerInfo;

}

class BackupAgent_ChangeResellerInfoResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $ChangeResellerInfoResult;

}

class BackupAgent_ChangeResellerInfo_Ex extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $resellerUsername;

    /**
     * @var (object)UserInfo
     */
    public $newUserInfo;

    /**
     * @var (object)ResellerInfo
     */
    public $newResellerInfo;

}

class BackupAgent_ChangeResellerInfo_ExResponse extends BackupAgent_to_array {

}

class BackupAgent_AddComputer extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $ComputerName;

}

class BackupAgent_AddComputerResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $AddComputerResult;

}

class BackupAgent_RemoveComputer extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $ComputerName;

}

class BackupAgent_RemoveComputerResponse extends BackupAgent_to_array {

}

class BackupAgent_RemoveOwner extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $strOwnerToDelete;

}

class BackupAgent_RemoveOwnerResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $RemoveOwnerResult;

}

class BackupAgent_DeleteAccount extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $arrUsername;

    /**
     * @var boolean
     */
    public $isReseller;

}

class BackupAgent_DeleteAccountResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $DeleteAccountResult;

}

class BackupAgent_RemoveDataFromAccount extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $arrUsername;

}

class BackupAgent_RemoveDataFromAccountResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $RemoveDataFromAccountResult;

}

class BackupAgent_ChangeEmailActivation extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $emailType;

    /**
     * @var boolean
     */
    public $activate;

}

class BackupAgent_ChangeEmailActivationResponse extends BackupAgent_to_array {

}

class BackupAgent_DeActivateAccount_Ex extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $arrUserName;

    /**
     * @var string
     */
    public $deactivatedBy;

}

class BackupAgent_DeActivateAccount_ExResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $DeActivateAccount_ExResult;

}

class BackupAgent_DeActivateAccount extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $arrUserName;

}

class BackupAgent_DeActivateAccountResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $DeActivateAccountResult;

}

class BackupAgent_RecalculateUsedSize extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $arrUserName;

}

class BackupAgent_RecalculateUsedSizeResponse extends BackupAgent_to_array {

}

class BackupAgent_CleanUpAccounts extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $arrUserName;

}

class BackupAgent_CleanUpAccountsResponse extends BackupAgent_to_array {

}

class BackupAgent_GetNew_Or_Upgraded_UserAccounts extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $strResellerName;

    /**
     * @var string
     *     NOTE: objDateInterval should follow the following restrictions
     *     You can have one of the following value
     *     none
     *     day_1
     *     week_1
     *     month_1
     *     year_1
     */
    public $objDateInterval;

    /**
     * @var string
     */
    public $StartOfPeriod;

    /**
     * @var string
     */
    public $EndOfPeriod;

    /**
     * @var boolean
     */
    public $blnGetUpgrades;

    /**
     * @var string
     */
    public $Criteria;

}

class BackupAgent_GetNew_Or_Upgraded_UserAccountsResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfUserAccountInfo
     */
    public $GetNew_Or_Upgraded_UserAccountsResult;

}

class BackupAgent_ArrayOfUserAccountInfo extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)UserAccountInfo
     */
    public $UserAccountInfo;

}

class BackupAgent_UserAccountInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $parentUser;

    /**
     * @var string
     */
    public $userName;

    /**
     * @var string
     */
    public $firstName;

    /**
     * @var string
     */
    public $lastName;

    /**
     * @var string
     */
    public $userAccountTypeName;

    /**
     * @var boolean
     */
    public $isTrial;

    /**
     * @var int
     */
    public $period;

    /**
     * @var string
     */
    public $company;

    /**
     * @var string
     */
    public $email;

    /**
     * @var (object)StatusInfo
     */
    public $objUserstatus;

    /**
     * @var string
     *     NOTE: userProfileType should follow the following restrictions
     *     You can have one of the following value
     *     Professional
     *     Home
     */
    public $userProfileType;

    /**
     * @var dateTime
     */
    public $accountStartDate;

}

class BackupAgent_StatusInfo extends BackupAgent_to_array {

    /**
     * @var double
     */
    public $usageSizePercentage;

    /**
     * @var boolean
     */
    public $isFull;

    /**
     * @var boolean
     */
    public $inProgress;

    /**
     * @var dateTime
     */
    public $lastestBackup;

    /**
     * @var string
     */
    public $backupClientVersionNr;

    /**
     * @var string
     *     NOTE: backupStatusEnum should follow the following restrictions
     *     You can have one of the following value
     *     Unknown
     *     Completed_Successfully
     *     Completed_Successfully_Skipped
     *     Completed_Successfully_NonExistentResources
     *     Completed_Successfully_Skipped_NonExistentResources
     *     Completed_UnSuccessfully
     *     Completed_UnSuccessfully_Skipped
     *     Completed_UnSuccessfully_NonExistentResources
     *     Completed_UnSuccessfully_Skipped_NonExistentResources
     *     UserCancelled
     */
    public $backupStatusEnum;

    /**
     * @var dateTime
     */
    public $lastestRestore;

    /**
     * @var string
     */
    public $restoreClientVersionNr;

    /**
     * @var string
     *     NOTE: restoreStatusEnum should follow the following restrictions
     *     You can have one of the following value
     *     Unknown
     *     Completed_Successfully
     *     Completed_Successfully_Skipped
     *     Completed_Successfully_NonExistentResources
     *     Completed_Successfully_Skipped_NonExistentResources
     *     Completed_UnSuccessfully
     *     Completed_UnSuccessfully_Skipped
     *     Completed_UnSuccessfully_NonExistentResources
     *     Completed_UnSuccessfully_Skipped_NonExistentResources
     *     UserCancelled
     */
    public $restoreStatusEnum;

}

class BackupAgent_GetAccountInfoAdmin extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $str_UserName;

}

class BackupAgent_GetUserInfoAdmin extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $str_UserName;

}

class BackupAgent_GetAccountInfoAdminResponse extends BackupAgent_to_array {

    /**
     * @var (object)AccountInfo
     */
    public $GetAccountInfoAdminResult;

}

class BackupAgent_GetSpecificUserInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_GetSpecificUserInfoResponse extends BackupAgent_to_array {

    /**
     * @var (object)UserInfo
     */
    public $GetSpecificUserInfoResult;

}

class BackupAgent_GetUserInfo extends BackupAgent_to_array {
}

class BackupAgent_GetUserInfoResponse extends BackupAgent_to_array {

    /**
     * @var (object)UserInfo
     */
    public $GetUserInfoResult;

}

class BackupAgent_GetResellerInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $str_ResellerName;

}

class BackupAgent_GetResellerInfoResponse extends BackupAgent_to_array {

    /**
     * @var (object)ResellerInfo
     */
    public $GetResellerInfoResult;

}

class BackupAgent_GetResellerClientMessageEnabled extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $resellerName;

}

class BackupAgent_GetResellerClientMessageEnabledResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $GetResellerClientMessageEnabledResult;

}

class BackupAgent_UpdateResellerClientMessageEnabled extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $resellerName;

    /**
     * @var boolean
     */
    public $enabled;

}

class BackupAgent_UpdateResellerClientMessageEnabledResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $UpdateResellerClientMessageEnabledResult;

}

class BackupAgent_GetResellerDashBoardInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $str_ResellerName;

}

class BackupAgent_GetResellerDashBoardInfoResponse extends BackupAgent_to_array {

    /**
     * @var (object)ResellerDashBoardInfo
     */
    public $GetResellerDashBoardInfoResult;

}

class BackupAgent_ResellerDashBoardInfo extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $totalUsers;

    /**
     * @var int
     */
    public $maxHomeUsers;

    /**
     * @var int
     */
    public $usedHomeUsers;

    /**
     * @var int
     */
    public $maxProfessionalUsers;

    /**
     * @var int
     */
    public $usedProfessionalUsers;

    /**
     * @var int
     */
    public $maxStorageGB;

    /**
     * @var int
     */
    public $assignedStorageMB;

    /**
     * @var long
     */
    public $usedStorage;

}

class BackupAgent_GetAdminDashBoardInfo extends BackupAgent_to_array {

}

class BackupAgent_GetAdminDashBoardInfoResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfAdminDashBoardInfo
     */
    public $GetAdminDashBoardInfoResult;

}

class BackupAgent_ArrayOfAdminDashBoardInfo extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)AdminDashBoardInfo
     */
    public $AdminDashBoardInfo;

}

class BackupAgent_AdminDashBoardInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $resellerUserName;

    /**
     * @var string
     */
    public $resellerFirstName;

    /**
     * @var string
     */
    public $resellerLastName;

    /**
     * @var string
     */
    public $resellerCompanyName;

    /**
     * @var string
     */
    public $resellerEmail;

    /**
     * @var long
     */
    public $reseller_UserCount;

    /**
     * @var long
     */
    public $reseller_TotalUserUsage;

}

class BackupAgent_GetLoggedErrors extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $intMaxRowsToBeReturned;

    /**
     * @var string
     */
    public $searchCriteria;

}

class BackupAgent_GetLoggedErrorsResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfLoggedEntryInfo
     */
    public $GetLoggedErrorsResult;

}

class BackupAgent_ArrayOfLoggedEntryInfo extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)LoggedEntryInfo
     */
    public $LoggedEntryInfo;

}

class BackupAgent_LoggedEntryInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

    /**
     * @var dateTime
     */
    public $entryDate;

    /**
     * @var string
     */
    public $errorMessage;

    /**
     * @var string
     */
    public $errorStackTrace;

    /**
     * @var string
     *     NOTE: identifier should follow the following restrictions
     *     The pattern should be
     * [0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}
     */
    public $identifier;

    /**
     * @var (object)ArrayOfString
     */
    public $parametersName;

    /**
     * @var (object)ArrayOfString
     */
    public $parametersValue;

}

class BackupAgent_GetClientLogs extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $computerName;

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_GetClientLogsResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfClientLog
     */
    public $GetClientLogsResult;

}

class BackupAgent_RemoveLoggedErrors extends BackupAgent_to_array {

}

class BackupAgent_RemoveLoggedErrorsResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $RemoveLoggedErrorsResult;

}

class BackupAgent_GetChildAccountInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $ParentUser;

    /**
     * @var string
     *     NOTE: objTypesToRetrieve should follow the following restrictions
     *     You can have one of the following value
     *     Normal_Account
     *     Trial_Account
     *     Free_Account
     *     Normal_And_Trial_Account
     */
    public $objTypesToRetrieve;

    /**
     * @var boolean
     */
    public $getAllAccounts;

    /**
     * @var string
     *     NOTE: requestType should follow the following restrictions
     *     You can have one of the following value
     *     Normal
     *     Active
     *     New
     */
    public $requestType;

}

class BackupAgent_GetChildAccountInfoResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfUserAccountInfo
     */
    public $GetChildAccountInfoResult;

}

class BackupAgent_GetUsers extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $Total;

    /**
     * @var boolean
     */
    public $Top10RecentAccounts;

    /**
     * @var string
     */
    public $Criteria;

}

class BackupAgent_GetUsersResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfString
     */
    public $GetUsersResult;

}

class BackupAgent_GetUsersMC extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $Total;

    /**
     * @var boolean
     */
    public $Top10RecentAccounts;

    /**
     * @var boolean
     */
    public $GetAllAcounts;

    /**
     * @var string
     */
    public $strParentUser;

    /**
     * @var string
     */
    public $Criteria;

}

class BackupAgent_GetUsersMCResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfUserAccountInfo
     */
    public $GetUsersMCResult;

}

class BackupAgent_GetResellers extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $strParentUser;

    /**
     * @var boolean
     */
    public $TotalResellerCount;

    /**
     * @var string
     */
    public $Criteria;

}

class BackupAgent_GetResellersResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfAnyType
     */
    public $GetResellersResult;

}

class BackupAgent_GetResellersUsageInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $strParentUser;

}

class BackupAgent_GetResellersUsageInfoResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfResellerUsageInfo
     */
    public $GetResellersUsageInfoResult;

}

class BackupAgent_ArrayOfResellerUsageInfo extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)ResellerUsageInfo
     */
    public $ResellerUsageInfo;

}

class BackupAgent_ResellerUsageInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $ResellerUserName;

    /**
     * @var int
     */
    public $maxHomeUsers;

    /**
     * @var int
     */
    public $usedHomeUsers;

    /**
     * @var int
     */
    public $maxProfessionalUsers;

    /**
     * @var int
     */
    public $usedProfessionalUsers;

    /**
     * @var int
     */
    public $maxStorageGB;

    /**
     * @var int
     */
    public $assignedStorageMB;

    /**
     * @var long
     */
    public $usedStorage;

    /**
     * @var (object)ArrayOfInt
     */
    public $assignedStorageLocation;

}

class BackupAgent_ArrayOfInt extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of int
     */
    public $int;

}

class BackupAgent_ResetPassword extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $Username;

}

class BackupAgent_ResetPasswordResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $ResetPasswordResult;

}

class BackupAgent_ResetPassword_Ex extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $Username;

}

class BackupAgent_ResetPassword_ExResponse extends BackupAgent_to_array {

}

class BackupAgent_ChangePassword extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $OldPassword;

    /**
     * @var string
     */
    public $NewPassword;

}

class BackupAgent_ChangePasswordResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $ChangePasswordResult;

}

class BackupAgent_ChangeAdministratorPassword extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $newPassword;

}

class BackupAgent_ChangeAdministratorPasswordResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $ChangeAdministratorPasswordResult;

}

class BackupAgent_ChangeUserInfoAdmin extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $username;

    /**
     * @var (object)UserInfo
     */
    public $UserInfo;

}

class BackupAgent_ChangeUserInfoAdminResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $ChangeUserInfoAdminResult;

}

class BackupAgent_ChangeUserInfoAdmin_Ex extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $username;

    /**
     * @var (object)UserInfo
     */
    public $UserInfo;

}

class BackupAgent_ChangeUserInfoAdmin_ExResponse extends BackupAgent_to_array {

}

class BackupAgent_ActivateAccount extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

    /**
     * @var string
     */
    public $password;

    /**
     * @var boolean
     */
    public $blnEcrypted;

}

class BackupAgent_ActivateAccountResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $ActivateAccountResult;

}

class BackupAgent_ActivateAccount_Ex extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

    /**
     * @var string
     */
    public $password;

    /**
     * @var boolean
     */
    public $blnEcrypted;

    /**
     * @var string
     */
    public $activatedBy;

}

class BackupAgent_ActivateAccount_ExResponse extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $ActivateAccount_ExResult;

}

class BackupAgent_ActivateAccount_Ex2 extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

    /**
     * @var string
     */
    public $password;

    /**
     * @var boolean
     */
    public $blnEcrypted;

    /**
     * @var string
     */
    public $activatedBy;

}

class BackupAgent_ActivateAccount_Ex2Response extends BackupAgent_to_array {

}

class BackupAgent_ActivateNoBackupMail extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_ActivateNoBackupMailResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $ActivateNoBackupMailResult;

}

class BackupAgent_DeactivateNoBackupMail extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_DeactivateNoBackupMailResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $DeactivateNoBackupMailResult;

}

class BackupAgent_GetNoBackupMailActivated extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_GetNoBackupMailActivatedResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $GetNoBackupMailActivatedResult;

}

class BackupAgent_SendEMail extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $mailType;

    /**
     * @var string
     */
    public $UserName;

    /**
     * @var string
     */
    public $Text;

}

class BackupAgent_SendEMailResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $SendEMailResult;

}

class BackupAgent_SendStatusReportEMail extends BackupAgent_to_array {

    /**
     * @var (object)BackupReportInfo
     */
    public $objBackupReport;

    /**
     * @var string
     *     NOTE: languageType should follow the following restrictions
     *     You can have one of the following value
     *     LanguageType_NL
     *     LanguageType_EN
     *     LanguageType_FR
     *     LanguageType_ES
     *     LanguageType_DE
     *     LanguageType_IT
     */
    public $languageType;

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_BackupReportInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

    /**
     * @var string
     */
    public $completeText;

    /**
     * @var boolean
     */
    public $isBackup;

    /**
     * @var string
     */
    public $status;

    /**
     * @var string
     */
    public $longDate;

    /**
     * @var string
     */
    public $computer;

    /**
     * @var string
     */
    public $clientVersionNr;

    /**
     * @var string
     */
    public $operatingSystem;

    /**
     * @var dateTime
     */
    public $startDate;

    /**
     * @var dateTime
     */
    public $endDate;

    /**
     * @var string
     */
    public $duration;

    /**
     * @var int
     */
    public $totalFiles;

    /**
     * @var string
     */
    public $totalSize;

    /**
     * @var string
     */
    public $restoreSizeDownloaded;

    /**
     * @var int
     */
    public $restoreSkippedResources;

    /**
     * @var string
     */
    public $restorePath;

    /**
     * @var int
     */
    public $backupChangedFiles;

    /**
     * @var string
     */
    public $backupSizeChangedFiles;

    /**
     * @var string
     */
    public $backupSizeUploaded;

    /**
     * @var int
     */
    public $backupSkippedResources;

    /**
     * @var int
     */
    public $backupNonExistentResources;

    /**
     * @var (object)ArrayOfString
     */
    public $backupResources;

    /**
     * @var (object)ArrayOfString
     */
    public $NonExistentResources;

    /**
     * @var int
     */
    public $NbNewFiles;

    /**
     * @var long
     */
    public $NbNewFilesUploadedSize;

    /**
     * @var int
     */
    public $NbPatches;

    /**
     * @var long
     */
    public $NbPatchesUploadedSize;

    /**
     * @var int
     */
    public $NbUnchangedFiles;

    /**
     * @var int
     */
    public $DBID;

    /**
     * @var string
     */
    public $ClientLogs;

    /**
     * @var string
     *     NOTE: statusEnum should follow the following restrictions
     *     You can have one of the following value
     *     Unknown
     *     Completed_Successfully
     *     Completed_Successfully_Skipped
     *     Completed_Successfully_NonExistentResources
     *     Completed_Successfully_Skipped_NonExistentResources
     *     Completed_UnSuccessfully
     *     Completed_UnSuccessfully_Skipped
     *     Completed_UnSuccessfully_NonExistentResources
     *     Completed_UnSuccessfully_Skipped_NonExistentResources
     *     UserCancelled
     */
    public $statusEnum;

}

class BackupAgent_SendStatusReportEMailResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $SendStatusReportEMailResult;

}

class BackupAgent_SaveStatusReportEMail extends BackupAgent_to_array {

    /**
     * @var (object)BackupReportInfo
     */
    public $objBackupReport;

    /**
     * @var string
     *     NOTE: languageType should follow the following restrictions
     *     You can have one of the following value
     *     LanguageType_NL
     *     LanguageType_EN
     *     LanguageType_FR
     *     LanguageType_ES
     *     LanguageType_DE
     *     LanguageType_IT
     */
    public $languageType;

    /**
     * @var string
     */
    public $userName;

    /**
     * @var (object)ArrayOfString
     */
    public $logMessages;

    /**
     * @var (object)ArrayOfDateTime
     */
    public $logDates;

}

class BackupAgent_ArrayOfDateTime extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of dateTime
     */
    public $dateTime;

}

class BackupAgent_SaveStatusReportEMailResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $SaveStatusReportEMailResult;

}

class BackupAgent_SendStatusReportEMail_Ex extends BackupAgent_to_array {

    /**
     * @var (object)BackupReportInfo
     */
    public $objBackupReport;

    /**
     * @var string
     *     NOTE: languageType should follow the following restrictions
     *     You can have one of the following value
     *     LanguageType_NL
     *     LanguageType_EN
     *     LanguageType_FR
     *     LanguageType_ES
     *     LanguageType_DE
     *     LanguageType_IT
     */
    public $languageType;

    /**
     * @var string
     */
    public $userName;

    /**
     * @var (object)ArrayOfString
     */
    public $logMessages;

    /**
     * @var (object)ArrayOfDateTime
     */
    public $logDates;

}

class BackupAgent_SendStatusReportEMail_ExResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $SendStatusReportEMail_ExResult;

}

class BackupAgent_SendTestEMail extends BackupAgent_to_array {

}

class BackupAgent_SendTestEMailResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $SendTestEMailResult;

}

class BackupAgent_TrialUserWarningMail extends BackupAgent_to_array {

}

class BackupAgent_TrialUserWarningMailResponse extends BackupAgent_to_array {

}

class BackupAgent_TrialUserDeletion extends BackupAgent_to_array {

}

class BackupAgent_TrialUserDeletionResponse extends BackupAgent_to_array {

}

class BackupAgent_MonthlyOverViewEmail extends BackupAgent_to_array {

}

class BackupAgent_MonthlyOverViewEmailResponse extends BackupAgent_to_array {

}

class BackupAgent_GetFullBackupReportInfo extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $BackupReportID;

}

class BackupAgent_GetFullBackupReportInfoResponse extends BackupAgent_to_array {

    /**
     * @var (object)BackupReportInfo
     */
    public $GetFullBackupReportInfoResult;

}

class BackupAgent_GetUserBackupResults extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $strUser_Name;

    /**
     * @var string
     */
    public $language;

}

class BackupAgent_GetUserBackupResultsResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfBackupReportInfo
     */
    public $GetUserBackupResultsResult;

}

class BackupAgent_ArrayOfBackupReportInfo extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)BackupReportInfo
     */
    public $BackupReportInfo;

}

class BackupAgent_GetRequestSize extends BackupAgent_to_array {

    // You need to set only one from the following two vars

    /**
     * @var Plain Binary
     */
    public $data;

    /**
     * @var base64Binary
     */
    public $data_encoded;


}

class BackupAgent_GetRequestSizeResponse extends BackupAgent_to_array {

    /**
     * @var long
     */
    public $GetRequestSizeResult;

}

class BackupAgent_GetServerFileLocation extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $TokenUserName;

}

class BackupAgent_GetServerFileLocationResponse extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $GetServerFileLocationResult;

}

class BackupAgent_CheckServer extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $strClientVersionNr;

}

class BackupAgent_CheckServerResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $CheckServerResult;

}

class BackupAgent_CheckUserLogin extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $sUserName;

    /**
     * @var string
     */
    public $sPassWord;

    /**
     * @var string
     */
    public $strClientVersionNr;

}

class BackupAgent_CheckUserLoginResponse extends BackupAgent_to_array {

    /**
     * @var (object)LoginInfo
     */
    public $CheckUserLoginResult;

}

class BackupAgent_LoginInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

    /**
     * @var string
     */
    public $passWord;

    /**
     * @var int
     */
    public $userTypeID;

    /**
     * @var int
     */
    public $LoginStatus;

}

class BackupAgent_CheckUserLogin_Ex extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $sUserName;

    /**
     * @var string
     */
    public $sPassWord;

}

class BackupAgent_CheckUserLogin_ExResponse extends BackupAgent_to_array {

    /**
     * @var (object)LoginInfo
     */
    public $CheckUserLogin_ExResult;

}

class BackupAgent_GetServerInfo extends BackupAgent_to_array {

}

class BackupAgent_GetServerInfoResponse extends BackupAgent_to_array {

    /**
     * @var (object)ServerInfo
     */
    public $GetServerInfoResult;

}

class BackupAgent_ServerInfo extends BackupAgent_to_array {

    /**
     * @var long
     */
    public $totalProfessionalAccounts;

    /**
     * @var long
     */
    public $totalHomeAccounts;

    /**
     * @var long
     */
    public $totalResellers;

    /**
     * @var long
     */
    public $totalSpace;

    /**
     * @var long
     */
    public $totalUsedSpace;

    /**
     * @var long
     */
    public $totalFreeSpace;

    /**
     * @var long
     */
    public $totalOnline;

    /**
     * @var long
     */
    public $totalCurrentBackup;

    /**
     * @var long
     */
    public $totalCurrentDownload;

}

class BackupAgent_GetLicenseInfo extends BackupAgent_to_array {

}

class BackupAgent_GetLicenseInfoResponse extends BackupAgent_to_array {

    /**
     * @var (object)LicenseInfo
     */
    public $GetLicenseInfoResult;

}

class BackupAgent_LicenseInfo extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $maxTTPs;

    /**
     * @var int
     */
    public $maxProfessionalCALS;

    /**
     * @var int
     */
    public $maxHomeCALS;

    /**
     * @var int
     */
    public $maxWhiteLabels;

    /**
     * @var int
     */
    public $maxAccountSize;

    /**
     * @var int
     */
    public $usedProfessionalCALS;

    /**
     * @var int
     */
    public $usedHomeCALS;

    /**
     * @var int
     */
    public $usedHomeTrials;

    /**
     * @var int
     */
    public $usedProfessionalFree;

    /**
     * @var int
     */
    public $usedHomeFree;

    /**
     * @var int
     */
    public $usedProfessionalTrials;

    /**
     * @var int
     */
    public $usedResellerWhiteLabels;

    /**
     * @var int
     */
    public $usedResellerDefault;

    /**
     * @var int
     */
    public $usedTTPs;

    /**
     * @var dateTime
     */
    public $startDate;

    /**
     * @var dateTime
     */
    public $endDate;

    /**
     * @var boolean
     */
    public $isValid;

    /**
     * @var boolean
     */
    public $isActive;

    /**
     * @var string
     */
    public $registeredDNS;

    /**
     * @var string
     *     NOTE: serverType should follow the following restrictions
     *     You can have one of the following value
     *     Trial
     *     Express
     *     Standard
     *     Enterprise
     *     Appliance_Trial
     *     Appliance_Express
     *     Appliance_Standard
     *     Appliance_Professional
     *     Appliance_Corporate
     *     Appliance_Enterprise
     *     BackupAgentProvider
     */
    public $serverType;

    /**
     * @var int
     */
    public $freeAccounts;

    /**
     * @var int
     */
    public $freeAccountSize;

    /**
     * @var int
     */
    public $maxApplianceGBSize;

    /**
     * @var int
     */
    public $totalHomeTrials;

    /**
     * @var int
     */
    public $totalProfessionalTrials;

    /**
     * @var int
     */
    public $totalFreeProfessionalAccounts;

    /**
     * @var int
     */
    public $totalFreeHomeAccounts;

    /**
     * @var string
     */
    public $productName;

    /**
     * @var string
     */
    public $productEdition;

    /**
     * @var boolean
     */
    public $usedLoadBalancing;

    /**
     * @var boolean
     */
    public $usedProvisioning;

    /**
     * @var boolean
     */
    public $usedWebserviceIntegration;

    /**
     * @var boolean
     */
    public $usedActiveDirectory;

}

class BackupAgent_SetServiceLicense extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $licenseKey;

}

class BackupAgent_SetServiceLicenseResponse extends BackupAgent_to_array {

}

class BackupAgent_SetServiceSettings extends BackupAgent_to_array {

    /**
     * @var (object)ServiceSettingsInfo
     */
    public $objServiceSettings;

}

class BackupAgent_ServiceSettingsInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $SMTP_IP;

    /**
     * @var string
     */
    public $serviceName;

    /**
     * @var string
     */
    public $computerName;

    /**
     * @var string
     */
    public $supportEmail;

    /**
     * @var string
     */
    public $logoUrl;

    /**
     * @var string
     */
    public $ownerName;

    /**
     * @var string
     */
    public $filesLocation;

    /**
     * @var string
     */
    public $mountUserName;

    /**
     * @var string
     */
    public $mountPassWord;

}

class BackupAgent_SetServiceSettingsResponse extends BackupAgent_to_array {

}

class BackupAgent_SetRegistrationInfo extends BackupAgent_to_array {

    /**
     * @var (object)ServiceRegistrationInfo
     */
    public $objRegistrationInfo;

}

class BackupAgent_ServiceRegistrationInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $address;

    /**
     * @var string
     */
    public $zipCode;

    /**
     * @var string
     */
    public $city;

    /**
     * @var int
     */
    public $country;

    /**
     * @var string
     */
    public $phoneNr;

    /**
     * @var string
     */
    public $emailAddress;

    /**
     * @var string
     */
    public $companyName;

    /**
     * @var string
     *     NOTE: languageType should follow the following restrictions
     *     You can have one of the following value
     *     LanguageType_NL
     *     LanguageType_EN
     *     LanguageType_FR
     *     LanguageType_ES
     *     LanguageType_DE
     *     LanguageType_IT
     */
    public $languageType;

}

class BackupAgent_SetRegistrationInfoResponse extends BackupAgent_to_array {

}

class BackupAgent_GetConfiguration extends BackupAgent_to_array {

}

class BackupAgent_GetConfigurationResponse extends BackupAgent_to_array {

    /**
     * @var (object)WSConfiguration
     */
    public $GetConfigurationResult;

}

class BackupAgent_WSConfiguration extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $DBServer;

    /**
     * @var string
     */
    public $Failover_DBServer;

    /**
     * @var string
     */
    public $DBName;

    /**
     * @var string
     */
    public $DBUserName;

    /**
     * @var string
     */
    public $DBPassword;

    /**
     * @var string
     */
    public $MetaFileName;

    /**
     * @var string
     */
    public $WebServiceURL;

    /**
     * @var string
     */
    public $WebServiceActivationURL;

    /**
     * @var string
     */
    public $SmtpServerIP;

    /**
     * @var boolean
     */
    public $UsageLogging;

    /**
     * @var boolean
     */
    public $LogErrorToEventLog;

    /**
     * @var string
     */
    public $CertificaatHash;

    /**
     * @var string
     */
    public $DigiNotarID;

    /**
     * @var int
     */
    public $SubResellerLevel;

    /**
     * @var int
     */
    public $SessionDuration;

    /**
     * @var int
     */
    public $DefaultTrialAccountType;

    /**
     * @var int
     */
    public $MaxBackupReportRecords;

    /**
     * @var int
     */
    public $BackupAgentSessionManagerPort;

    /**
     * @var string
     */
    public $BackupAgentSessionManagerURL;

    /**
     * @var string
     */
    public $BackupAgentSessionManagerURLFailover;

    /**
     * @var boolean
     */
    public $DFSSupport;

    /**
     * @var boolean
     */
    public $UserCanChangePassword;

    /**
     * @var boolean
     */
    public $UserCanChangeAccountInfo;

    /**
     * @var boolean
     */
    public $ShowRegistrationLink;

    /**
     * @var boolean
     */
    public $UseHTTPS;

    /**
     * @var boolean
     */
    public $UseActiveDirectory;

    /**
     * @var string
     */
    public $ActiveDirectoryDomainName;

    /**
     * @var boolean
     */
    public $ShowDownloadLink;

}

class BackupAgent_ChangeConfiguration extends BackupAgent_to_array {

    /**
     * @var (object)WSConfiguration
     */
    public $NewConfiguration;

}

class BackupAgent_ChangeConfigurationResponse extends BackupAgent_to_array {

}

class BackupAgent_GetOwnerInfo extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $owner;

}

class BackupAgent_GetOwnerInfoResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfOwnerInfo
     */
    public $GetOwnerInfoResult;

}

class BackupAgent_ArrayOfOwnerInfo extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)OwnerInfo
     */
    public $OwnerInfo;

}

class BackupAgent_GetOwnerInfo_Ex extends BackupAgent_to_array {

    /**
     * @var int
     */
    public $ownerID;

}

class BackupAgent_GetOwnerInfo_ExResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfOwnerInfo
     */
    public $GetOwnerInfo_ExResult;

}

class BackupAgent_GetCheckSums extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $fullName;

    /**
     * @var long
     */
    public $BlockNumber;

    /**
     * @var int
     */
    public $blocksize;

}

class BackupAgent_GetCheckSumsResponse extends BackupAgent_to_array {

    /**
     * @var (object)fileCheckSumBlock
     */
    public $GetCheckSumsResult;

}

class BackupAgent_fileCheckSumBlock extends BackupAgent_to_array {

    /**
     * @var (object)fileCheckSums
     */
    public $fileCheckSums;

    /**
     * @var long
     */
    public $blockCount;

    /**
     * @var long
     */
    public $blockNumber;

    /**
     * @var int
     */
    public $blockSize;

}

class BackupAgent_fileCheckSums extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfFileCheckSum
     */
    public $CS;

    /**
     * @var int
     */
    public $lastBlockSize;

}

class BackupAgent_ArrayOfFileCheckSum extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)fileCheckSum
     */
    public $fileCheckSum;

}

class BackupAgent_fileCheckSum extends BackupAgent_to_array {

    /**
     * @var long
     */
    public $weakCS;

    // You need to set only one from the following two vars

    /**
     * @var Plain Binary
     */
    public $strongCS;

    /**
     * @var base64Binary
     */
    public $strongCS_encoded;


}

class BackupAgent_GetSourceVersion extends BackupAgent_to_array {

    /**
     * @var (object)FileVersion
     */
    public $PatchVersion;

}

class BackupAgent_GetSourceVersionResponse extends BackupAgent_to_array {

    /**
     * @var (object)FileVersion
     */
    public $GetSourceVersionResult;

}

class BackupAgent_GetSourceVersion_MC extends BackupAgent_to_array {

    /**
     * @var (object)FileVersion
     */
    public $PatchVersion;

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_GetSourceVersion_MCResponse extends BackupAgent_to_array {

    /**
     * @var (object)FileVersion
     */
    public $GetSourceVersion_MCResult;

}

class BackupAgent_ResetConfiguration extends BackupAgent_to_array {

}

class BackupAgent_ResetConfigurationResponse extends BackupAgent_to_array {

}

class BackupAgent_DepositDigiNotarKey extends BackupAgent_to_array {

    /**
     * @var (object)Key
     */
    public $bClientKey;

}

class BackupAgent_DepositDigiNotarKeyResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $DepositDigiNotarKeyResult;

}

class BackupAgent_ManageDigiNotarKey extends BackupAgent_to_array {

    // You need to set only one from the following two vars

    /**
     * @var Plain Binary
     */
    public $clientKeyHash;

    /**
     * @var base64Binary
     */
    public $clientKeyHash_encoded;


}

class BackupAgent_ManageDigiNotarKeyResponse extends BackupAgent_to_array {

    /**
     * @var string
     *     NOTE: ManageDigiNotarKeyResult should follow the following restrictions
     *     You can have one of the following value
     *     KeyHash_NotStoredLocally
     *     KeyHash_StoredLocally
     *     KeyHash_ValuesNOTEqual
     *     KeyHash_ValuesEqual
     */
    public $ManageDigiNotarKeyResult;

}

class BackupAgent_VerifyExistenceOfDigiNotarKey extends BackupAgent_to_array {

    // You need to set only one from the following two vars

    /**
     * @var Plain Binary
     */
    public $clientKeyHash;

    /**
     * @var base64Binary
     */
    public $clientKeyHash_encoded;


}

class BackupAgent_VerifyExistenceOfDigiNotarKeyResponse extends BackupAgent_to_array {

    /**
     * @var string
     *     NOTE: VerifyExistenceOfDigiNotarKeyResult should follow the following restrictions
     *     You can have one of the following value
     *     KeyHash_NotStoredLocally
     *     KeyHash_StoredLocally
     *     KeyHash_ValuesNOTEqual
     *     KeyHash_ValuesEqual
     */
    public $VerifyExistenceOfDigiNotarKeyResult;

}

class BackupAgent_SetEncryptionKey_Question extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $strUserText;

}

class BackupAgent_SetEncryptionKey_QuestionResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $SetEncryptionKey_QuestionResult;

}

class BackupAgent_SendDigiNotarKey extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $sUsername;

}

class BackupAgent_SendDigiNotarKeyResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $SendDigiNotarKeyResult;

}

class BackupAgent_GetFileObjectTransferStatisticsForOneUser extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

    /**
     * @var dateTime
     */
    public $periodStartTime;

    /**
     * @var dateTime
     */
    public $periodEndTime;

}

class BackupAgent_GetFileObjectTransferStatisticsForOneUserResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfFileTransferStatistics
     */
    public $GetFileObjectTransferStatisticsForOneUserResult;

}

class BackupAgent_ArrayOfFileTransferStatistics extends BackupAgent_to_array {

    /**
     * @var array[0, unbounded] of (object)FileTransferStatistics
     */
    public $FileTransferStatistics;

}

class BackupAgent_FileTransferStatistics extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

    /**
     * @var dateTime
     */
    public $periodStartTime;

    /**
     * @var dateTime
     */
    public $periodEndTime;

    /**
     * @var long
     */
    public $numberOfBytesTransferred;

    /**
     * @var int
     */
    public $numberOfFilesTransferred;

    /**
     * @var boolean
     */
    public $isUpload;

}

class BackupAgent_SendAccountExpiredMail extends BackupAgent_to_array {

}

class BackupAgent_SendAccountExpiredMailResponse extends BackupAgent_to_array {

}

class BackupAgent_SendAccountWillExpireWarningMail extends BackupAgent_to_array {

}

class BackupAgent_SendAccountWillExpireWarningMailResponse extends BackupAgent_to_array {

}

class BackupAgent_SendNoBackupWarningmail extends BackupAgent_to_array {

}

class BackupAgent_SendNoBackupWarningmailResponse extends BackupAgent_to_array {

}

class BackupAgent_SendAccountStatusOverviewMail extends BackupAgent_to_array {

}

class BackupAgent_SendAccountStatusOverviewMailResponse extends BackupAgent_to_array {

}

class BackupAgent_GetAllUsersAndStorageLocation extends BackupAgent_to_array {

}

class BackupAgent_GetAllUsersAndStorageLocationResponse extends BackupAgent_to_array {

    /**
     * @var (object)ArrayOfAnyType
     */
    public $GetAllUsersAndStorageLocationResult;

}

class BackupAgent_IsUserBackuping extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_IsUserBackupingResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $IsUserBackupingResult;

}

class BackupAgent_UpdateUserAccountUsedSize extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

    /**
     * @var long
     */
    public $newSize;

}

class BackupAgent_UpdateUserAccountUsedSizeResponse extends BackupAgent_to_array {

}

class BackupAgent_ClearPendingBackups extends BackupAgent_to_array {

}

class BackupAgent_ClearPendingBackupsResponse extends BackupAgent_to_array {

}

class BackupAgent_UpdateDBFromActiveDirectoryChanges extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $xmlADChanges;

}

class BackupAgent_UpdateDBFromActiveDirectoryChangesResponse extends BackupAgent_to_array {

}

class BackupAgent_GetWebserviceVersion extends BackupAgent_to_array {

}

class BackupAgent_GetWebserviceVersionResponse extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $GetWebserviceVersionResult;

}

class BackupAgent_SendWhiteLabelTestEmail extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $subject;

    /**
     * @var string
     */
    public $text;

    /**
     * @var boolean
     */
    public $isText;

    /**
     * @var string
     */
    public $username;

}

class BackupAgent_SendWhiteLabelTestEmailResponse extends BackupAgent_to_array {

    /**
     * @var boolean
     */
    public $SendWhiteLabelTestEmailResult;

}

class BackupAgent_ChangeDefaultLaungage extends BackupAgent_to_array {

    /**
     * @var string
     *     NOTE: language should follow the following restrictions
     *     You can have one of the following value
     *     NL
     *     EN
     *     FR
     *     ES
     *     DE
     *     IT
     *     UK
     */
    public $language;

}

class BackupAgent_ChangeDefaultLaungageResponse extends BackupAgent_to_array {

}

class BackupAgent_GetHostname extends BackupAgent_to_array {

}

class BackupAgent_GetHostnameResponse extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $GetHostnameResult;

}

class BackupAgent_ClearUserSession extends BackupAgent_to_array {

    /**
     * @var string
     */
    public $userName;

}

class BackupAgent_ClearUserSessionResponse extends BackupAgent_to_array {

}
